#========================================================================================
# package      DE_EPAGES::Address::API::Object::Address
# state        public
# base         DE_EPAGES::Object::API::Object::Object
# description  object interface for Address.
#========================================================================================
package DE_EPAGES::Address::API::Object::Address;
use base DE_EPAGES::Object::API::Object::Object;

use strict;
use DE_EPAGES::Address::API::Address qw ( SecureAddressFields );
use DE_EPAGES::Address::API::Constants qw ( COMPARE_ADDRESS_PARAMETERS ANONYMIZE_ADDRESS_PARAMETERS ANONYMIZE_ADDRESS_MASK);
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use constant AGE_LIMIT_ENFORCED_BY_UI => 1752; #Year

sub ErrorDefinition {
    return {
        'BIRTHYEAR' => 'Got $Birthday->year = #BirthYear. Expected 4 digit number && number > 1752. see EPG-19235 for more info',
        'EncryptedAddressAliasAlreadyExists' => 'New Alias #Alias for encrypted address already exists in path #Parent.'
    }
}
#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Address->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'Address' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $Address->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Address = DE_EPAGES::Address::API::Object::Address->insert($hInfo);
# example      my $Class = LoadClassByAlias('Address');
#               my $Address = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertAddress' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $Address | new object | object
# hook         OBJ_InsertAddress | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AddressID     - address identifier   - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    # filter potential malicious characters
    $hInfo = SecureAddressFields($hInfo);

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertAddress', {'Object' => $self, 'AddressID' => $self->id });
    delete $hInfo->{$_} foreach qw ( DisplayName AddressExtension Street Zipcode City State CountryID EMail Phone Fax VerifiedOn Street2 CodePorte ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Address->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteAddress' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteAddress | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AddressID     - address identifier   - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteAddress', {'Object' => $self, 'AddressID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Address->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateAddress' and 'OBJ_AfterUpdateAddress'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateAddress | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AddressID     - address identifier   - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateAddress | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AddressID     - address identifier   - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    my $Birthday = $hValues->{'Birthday'};
    if (defined $Birthday) {
        # Person born before 1753? May happen if only 2 diggits for year is used (deprecated!) see EPG-19235
        if ($Birthday->year <= AGE_LIMIT_ENFORCED_BY_UI) {
            Error( 'BIRTHYEAR', {BirthYear => $Birthday->year});
        }
    }

    # filter potential malicious characters
    $hValues = SecureAddressFields($hValues);

    TriggerHook('OBJ_BeforeUpdateAddress', {'Object'=>$self, 'Values'=> $hValues, 'AddressID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateAddress',  {'Object'=>$self, 'Values'=> $hValues, 'AddressID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     importXML
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = __PACKAGE__->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Adds references to direct or indirect child objects for billing address
#               and shipping address.
#               This is only required for compatibility with old import files (pre 6.9.2).
#========================================================================================
sub importXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;

    my $TagName = $hElement->{'LocalName'};
    my $XMLParentObject = $Handler->topID('Object');

    my $Address = $ObjectPackage->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);
    if( $TagName =~ /^BillingAddress|ShippingAddress$/ ) {
        # for users the format was:
        # <User Alias="mmustermann"...>
        #   <BillingAddress  reference="1" Class="Address" Path="Addresses/BillingAddress_1" />
        # for customers and orders the format was:
        # <Customer Alias='1001'...>
        #   <BillingAddress Class='Address' Alias='1001' ... />
        # </Customer>
        # <CustomerOrder Alias='1001' ... >
        #   <BillingAddress Class='Address' Alias='BillingAddress' ... />
        # </CustomerOrder>
        $XMLParentObject->set({ $TagName => $Address });
    }

    return $Address;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Address::API::Object::Address->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
    };
}

#========================================================================================
# function     CompareAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Equals = DE_EPAGES::Address::API::Object::Address->CompareAddress( $Address );
#----------------------------------------------------------------------------------------
# description  Returns 1 if the address of the parameter have the same fields like the Object
#----------------------------------------------------------------------------------------
# input        $Address | Address object | object
# return       $Equals | Integer (1=true,0=false)
#========================================================================================
sub CompareAddress {
    my $self = shift;
    my ($Address) = @_;

    # EPG-29264: Change done in order to avoid the messages on error.log because of the undef fields
    foreach my $key ( @{(COMPARE_ADDRESS_PARAMETERS)} ){

        my $Value1 = $self->get( $key ) // '' ;
        my $Value2 = $Address->get( $key ) // '' ;

        if ($Value1 ne $Value2){
            return 0;
        }
    }

    return 1;

}

#========================================================================================
# function     cloneAddressToEncryptedAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EncryptedAddress = $Address->cloneAddressToEncryptedAddress();
#----------------------------------------------------------------------------------------
# description  Returns $EncryptedAddress which is a clone of address with new class AddressEnc
#               Alias is changed to old Alias extended by string '_Enc'
#               attributes and parent stay the same like origin
#               Address attribute AddressEncrypted is set to reference to $EncryptedAddress
#----------------------------------------------------------------------------------------
# return       $EncryptedAddress | encrypted address object (class AddressEnc) | Object
#========================================================================================
sub cloneAddressToEncryptedAddress {
    my $self = shift;
    my $EncryptedAddress;
    unless (defined $self->get('AddressEncrypted')) {
        my $AliasEnc=$self->alias . '_Enc';
        my $Parent = $self->parent();
        my $AddressEncClass = LoadClassByAlias('AddressEnc');
        unless ($Parent->existsChild($AliasEnc)) {
            #clone address with new class addressenc
            $EncryptedAddress = $self->clone({'Alias' => $AliasEnc } , undef, undef, $AddressEncClass );
            #workaround the issue that on obj->insert the attributes are not encrypted
            my @EncrpytedAttributes =
                map { $_->alias }
                    grep { $_->package =~ /Attributes::Encrypted/ }
                        @{ $AddressEncClass->attributes };
            $EncryptedAddress->set( $EncryptedAddress->get(\@EncrpytedAttributes) );
            #set reference attribute AddressEncrypted to cloned encrypted address
            $self->set({'AddressEncrypted' => $EncryptedAddress });
        } else {
            Error('EncryptedAddressAliasAlreadyExists', {'Parent' => $Parent->pathString, 'Alias' => $AliasEnc});
        }
    } else {
        $EncryptedAddress= $self->get('AddressEncrypted');
    }
    return $EncryptedAddress;
}

#========================================================================================
# function     anonymize
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Address->anonymize();
#----------------------------------------------------------------------------------------
# description  Anonymizes/Masks address attributes ANONYMIZE_ADDRESS_PARAMETERS with ANONYMIZE_ADDRESS_MASK
#========================================================================================
sub anonymize {
    my $self = shift;

    my $hValues = $self->get(ANONYMIZE_ADDRESS_PARAMETERS);
    foreach my $key ( @{(ANONYMIZE_ADDRESS_PARAMETERS)} ){
        $hValues->{$key} = _anonymizeText(ANONYMIZE_ADDRESS_MASK, $hValues->{$key});
    }
    $self->set($hValues);
}

#========================================================================================
# function     deanonymize
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Address->deanonymize();
#----------------------------------------------------------------------------------------
# description  Deanonymizes address attributes ANONYMIZE_ADDRESS_PARAMETERS
#               if a linked encrypted address with original values is found
#========================================================================================
sub deanonymize {
    my $self = shift;

    my $AddressEncrypted = $self->get('AddressEncrypted');
    if (defined $AddressEncrypted) {
        my $hValues = $AddressEncrypted->get(ANONYMIZE_ADDRESS_PARAMETERS);
        $self->set($hValues);
        $AddressEncrypted->delete;
    }
}

#see https://stackoverflow.com/questions/1871092/masking-a-string-in-perl-using-a-mask-string
sub _anonymizeText {
    my ($mask, $text) = @_;
    $mask =~ s/-/'o' x (length $text - length $mask)/e;
    $text =~ s/(.)/substr($mask, pos $text, 1) eq 'o' ? $1 : 'x'/eg;
    return $text;
}

1;
