#========================================================================================
# package      DE_EPAGES::Address::API::Object::AddressEnc
# state        public
# base         DE_EPAGES::Object::API::Object::Object
# description  object interface for AddressEnc.
#========================================================================================
package DE_EPAGES::Address::API::Object::AddressEnc;
use base DE_EPAGES::Object::API::Object::Object;

use strict;
use DE_EPAGES::Address::API::Address qw ( SecureAddressFields );
use DE_EPAGES::Address::API::Constants qw ( COMPARE_ADDRESS_PARAMETERS );
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use constant AGE_LIMIT_ENFORCED_BY_UI => 1752; #Year

sub ErrorDefinition {
    return {
        'BIRTHYEAR' => 'Got $Birthday->year = #BirthYear. Expected 4 digit number && number > 1752. see EPG-19235 for more info',
    }
}
#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $AddressEnc->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'AddressEnc' }

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $Package->new('ObjectID' => $ObjectID, ... );
#----------------------------------------------------------------------------------------
# description  overloads DE_EPAGES::Core::API::BaseObject->new
#               to decrypt attributes before store in cache
#----------------------------------------------------------------------------------------
# input        %Attributes | attribute value hash | hash
# return       $Object | existing object | object
#========================================================================================
sub new {
    my $self = shift;
    my (%options) = @_;

    $self->decryptBaseAttributes(\%options);

    return $self->SUPER::new(%options);
}

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $AddressEnc->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Address = DE_EPAGES::Address::API::Object::AddressEnc->insert($hInfo);
# example      my $Class = LoadClassByAlias('Address');
#               my $AddressEnc = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertAddressEnc' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $AddressEnc | new object | object
# hook         OBJ_InsertAddressEnc | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AddressID     - address identifier   - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    # filter potential malicious characters
    $hInfo = SecureAddressFields($hInfo);

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertAddressEnc', {'Object' => $self, 'AddressID' => $self->id });
    delete $hInfo->{$_} foreach qw ( DisplayName AddressExtension Street Zipcode City State CountryID EMail Phone Fax VerifiedOn Street2 CodePorte ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $AddressEnc->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteAddressEnc' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteAddress | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AddressID     - address identifier   - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteAddressEnc', {'Object' => $self, 'AddressID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $AddressEnc->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateAddress' and 'OBJ_AfterUpdateAddress'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateAddressEnc | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AddressID     - address enc identifier   - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateAddressEnc | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AddressID     - address enc identifier   - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    my $Birthday = $hValues->{'Birthday'};
    if (defined $Birthday) {
        # Person born before 1753? May happen if only 2 diggits for year is used (deprecated!) see EPG-19235
        if ($Birthday->year <= AGE_LIMIT_ENFORCED_BY_UI) {
            Error( 'BIRTHYEAR', {BirthYear => $Birthday->year});
        }
    }

    # filter potential malicious characters
    $hValues = SecureAddressFields($hValues);

    TriggerHook('OBJ_BeforeUpdateAddressEnc', {'Object'=>$self, 'Values'=> $hValues, 'AddressID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateAddressEnc',  {'Object'=>$self, 'Values'=> $hValues, 'AddressID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Address::API::Object::AddressEnc->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
    };
}

#========================================================================================
# function     CompareAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Equals = DE_EPAGES::Address::API::Object::Address->CompareAddress( $AddressEnc );
#----------------------------------------------------------------------------------------
# description  Returns 1 if the address of the parameter have the same fields like the Object
#----------------------------------------------------------------------------------------
# input        $AddressEnc | AddressEnc object | object
# return       $Equals | Integer (1=true,0=false)
#========================================================================================
sub CompareAddress {
    my $self = shift;
    my ($AddressEnc) = @_;

    # EPG-29264: Change done in order to avoid the messages on error.log because of the undef fields
    foreach my $key ( @{(COMPARE_ADDRESS_PARAMETERS)} ){

        my $Value1 = $self->get( $key ) // '' ;
        my $Value2 = $AddressEnc->get( $key ) // '' ;

        if ($Value1 ne $Value2){
            return 0;
        }
    }

    return 1;

}

1;
