use strict;
use Getopt::Long;
use Time::HiRes qw( time );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError GetLog );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Amazon::API::Cartridge;
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

RunScript(Sub => \&Main, Debug => 0);

#========================================================================================
# ?function         InitializeColdStorageNotification
# ?state            public
#----------------------------------------------------------------------------------------
# ?syntax           InitializeColdStorageNotification();
#----------------------------------------------------------------------------------------
# ?description      Adds and activates the amazon cold storage notification
#                   to all Unity shops that have the related feature enabled.
#========================================================================================
sub InitializeColdStorageNotification {
    my ($hOptions) = @_;

    print "Reading all Unity shops\n" unless $hOptions->{nooutput};
    my $aShopIDs = GetUnityShopIDs();

    my $Cartridge = DE_EPAGES::Amazon::API::Cartridge->new;

    my $CountShops = 0;
    foreach my $ShopID (@$aShopIDs) {
        eval {
            my $Shop = LoadObject($ShopID);
            my $FeatureCount = $Shop->featureMaxValue('Amazon');
            my $NotificationExists = $Shop->folder('ShopNotifications')->existsChild('ColdStorage') ? 1 : 0;
            if ($hOptions->{debug}) {
                printf("featureMaxValue(Amazon) %s NotificationExists: %s for Shop %s\n", $FeatureCount, $NotificationExists, $Shop->alias);
            }
            next unless $FeatureCount;
            unless ($NotificationExists) {
                ImportObjects($Cartridge->xmlDirectory() . '/ShopNotificationColdStorage.xml', $Shop->pathString);
                $CountShops++;
            }
        };
        warn('Patch failed in shop ' . $ShopID . " with error \"" . GetError() . "\".") if ExistsError();
    }

    eval {SynchronizeCache();};
    LogError(GetError()) if ExistsError();

    return $CountShops;
}

sub Main {

    local $| = 1;
    my %Options = (nooutput => 0);
    my @GetOpt = qw(storename=s passwd=s help nooutput debug);
    usage("Unkown parameter specified.\n") unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    for (qw(storename)) {
        usage(sprintf("Missing parameter -%s\n",$_)) unless (defined $Options{$_});
    }

    my $StartTime = time;
    my $CountShops = 0;
    RunOnStore(
        Store      => $Options{storename},
        DBPassword => $Options{passwd},
        Sub        => sub {
            $CountShops = InitializeColdStorageNotification(\%Options);
        },
    );
    my $LogMessage =  sprintf('[%s] InitializeColdStorageNotification for %d shops took %.3f seconds', $Options{storename}, $CountShops, time() - $StartTime);
    GetLog()->debug($LogMessage);
    printf "%s\n", $LogMessage unless $Options{nooutput};
}

sub usage {
    my ($ErrorNote) = @_;

    print $ErrorNote if $ErrorNote;
    print <<END_USAGE;
Description:
    Adds and activates the notification for Amazon ColdStorage
    to all existing Unity shops with feature Amazon (in this store).
Usage:
    perl $0 [options] [flags] datafiles
options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options
Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}
