use strict;
use Getopt::Long;
use Time::HiRes qw ( time );
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime SubtractDuration GetLoopFromDoubleArray );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadObjectByPath LoadObject );
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

my %Options = (maxminutes => 60, minage => 30, nooutput => 0);

RunScript(Sub => \&Main, Debug => 0);

sub Main {
    local $| = 1;
    my @GetOpt = qw(storename=s passwd=s help nooutput shop=s from=s minage=s maxminutes=s);
    usage("Unkown parameter specified.\n") unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    for (qw(storename from)) {
        usage(sprintf("Missing parameter -%s\n",$_)) unless (defined $Options{$_});
    }

    my $CountShops = 0;
    my $MaxDateTime = GetCurrentDateTime()->add('minutes'=>$Options{maxminutes});
    my $StartTime = time();

    RunOnStore(
        Store => $Options{storename},
        DBPassword => $Options{passwd},
        Sub => sub {
            unless (ExistsCartridgeByPackage('DE_EPAGES::Amazon')) {
                print "Store: $Options{storename} : Cartridge Amazon is not installed.\n" unless $Options{nooutput};
                return;
            }
            my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new(locale => 'de_DE');
            my $FromDateTime = $Formatter->parse_datetime($Options{from}, 'datetime', '%d.%m.%Y');
            my $ToDateTime   = SubtractDuration(GetCurrentDateTime(), 'days', $Options{minage} );
            if (defined $Options{shop}) {
                my $Shop;
                eval { $Shop = LoadObjectByPath('/Shops')->child($Options{shop}); };
                if (ExistsError()) {
                    printf STDERR "[%s] WARN: Shop '%s' does not exist.\n", $Options{storename}, $Options{shop};
                } else {
                    my $aContainerIDs = findNonEncryptedAmazonLineItemContainers($Shop, $FromDateTime, $ToDateTime);
                    if (scalar @$aContainerIDs) {
                        anonymizeAddressDataForShop($Shop, $aContainerIDs);
                        $CountShops++;
                    }
                }
            } else {
                printf "Running script until %s\n", $MaxDateTime unless $Options{nooutput};
                my $AmazonConfigClass = LoadClassByAlias('AmazonConfig');
                my @AmazonConfigs = @{ $AmazonConfigClass->objects };
                my $Feature = LoadFeatureByAlias('Amazon');
                foreach my $AmazonConfig (@AmazonConfigs) {
                    printf "Found %s AmazonConfigs\n", scalar @AmazonConfigs unless $Options{nooutput};
                    my $Shop = $AmazonConfig ->getSite;
                    next if (not $Shop->featureMaxValue($Feature));
                    next if ($Shop->get('ClosedByProvider'));
                    next unless $Shop->get('AmazonAnonymizeAddresses');
                    my $aContainers = findNonEncryptedAmazonLineItemContainers($Shop, $FromDateTime, $ToDateTime);
                    if (scalar @$aContainers) {
                        anonymizeAddressDataForShop($Shop, $aContainers);
                        $CountShops++;
                    }
                    last if (GetCurrentDateTime() > $MaxDateTime);
                }
            }
        }
    );
    my $LogMessage =  sprintf('[%s] Anonymizing address data for Amazon orders for %d shops took %.3f seconds', $Options{storename}, $CountShops, time() - $StartTime);
    GetLog()->debug($LogMessage);
    printf "%s\n", $LogMessage unless $Options{nooutput};
    eval { SynchronizeCache(); };
    Warning(GetError()) if ExistsError();
}


sub findNonEncryptedAmazonLineItemContainers {
    my ($Shop, $From, $To) = @_;

    my $dbi = GetCurrentDBHandle();
    my $ClassLineItemContainerAmazon = LoadClassByAlias('LineItemContainerAmazon');
    my $ClassCustomerOrder           = LoadClassByAlias('CustomerOrder');
    my $AttributeID                  = $ClassLineItemContainerAmazon->attribute('IsAmazonOrderAnonymized')->id;
    my $Result = $dbi->execute(<<SQL, [$ClassCustomerOrder->id, $ClassLineItemContainerAmazon->id, $Shop->id, $From, $To, $AttributeID], ['Integer']);
        SELECT aco.objectid
        FROM   object aco
        INNER  JOIN object paco ON aco.parentid=paco.objectid
        AND    paco.classid = ?
        WHERE  aco.classid = ?
        AND    aco.siteid = ?
        AND    aco.creationdate BETWEEN ? AND ?
        AND    NOT EXISTS (SELECT 1
                           FROM   booleanattribute ba
                           WHERE  ba.objectid    = aco.objectid
                           AND    ba.attributeid = ?
                           AND    value = 1)
SQL
    return GetLoopFromDoubleArray($Result, [qw (ObjectID) ]);
}

sub anonymizeAddressDataForShop {
    my ($Shop, $aContainers) = @_;

    my $StartTime = time();
    my $CountAddresses = 0;
    foreach my $Container (@$aContainers) {
        my @EncryptAddresses  = ();
        my $LineItemContainer = LoadObject($Container->{ObjectID});
        my $Order             = $LineItemContainer->parent();
        push @EncryptAddresses, values %{ $Order->get(['BillingAddress', 'ShippingAddress']) };
        push @EncryptAddresses, $Order->get('Customer')->get('BillingAddress');
        foreach my $Address (@EncryptAddresses)  {
            next unless defined $Address;
            $Address->cloneAddressToEncryptedAddress();
            $Address->anonymize();
            $CountAddresses++;
        }
        $LineItemContainer->set({'IsAmazonOrderAnonymized' => 1});
    }
    unless ($Options{nooutput}) {
        printf "Shop %s : %s addresses encrypted/anonymized took %.3fs\n", $Shop->alias, $CountAddresses, time() - $StartTime
    }
}

sub usage {
    my ($ErrorNote) = @_;

    print $ErrorNote if $ErrorNote;
    print <<END_USAGE;
Description:
    This script moves old addresses from amazon orders to an encrypted table and anonymizes the rows in the
    original table

Usage:
    perl $0 [options] [flags]

options:
    -storename     store name for the database connection
    -passwd        database user password
    -shop          only run the script for this shop
    -from          only encrypt amazon orders created after this date
    -minage        only encrypt amazon orders that are at least that many days old  (30)
    -maxminutes    how many minutes the script should run maximally                 (60)


flags:
    -help          show the command line options
    -nooutput      do not print any output
Example:
    perl $0 -storename Store -from 31.10.2019 -minage 30
    perl $0 -storename Store -shop DemoShop -from 31.10.2019 -minage 30
END_USAGE
    exit 2;
}

1;
