use strict;
use Getopt::Long;
use Time::HiRes qw ( time );
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Core::API::PerlTools qw ( GetLoopFromDoubleArray );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadObjectByPath LoadObject );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

my %Options = (nooutput => 0);

RunScript(Sub => \&Main, Debug => 0);

sub Main {
    local $| = 1;
    my @GetOpt = qw(storename=s passwd=s help nooutput shop=s force);
    usage("Unkown parameter specified.\n") unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    for (qw(storename shop)) {
        usage(sprintf("Missing parameter -%s\n",$_)) unless (defined $Options{$_});
    }

    my $StartTime = time();
    my $Shop;

    RunOnStore(
        Store => $Options{storename},
        DBPassword => $Options{passwd},
        Sub => sub {
            unless (ExistsCartridgeByPackage('DE_EPAGES::Amazon')) {
                print "Store: $Options{storename} : Cartridge Amazon is not installed.\n" unless $Options{nooutput};
                return;
            }
            eval { $Shop = LoadObjectByPath('/Shops')->child($Options{shop}); };
            if (ExistsError()) {
                printf STDERR "[%s] WARN: Shop '%s' does not exist.\n", $Options{storename}, $Options{shop};
            } else {
                # prevent automated re-anonymizing by cron script
                $Shop->set({'AmazonAnonymizeAddresses' => 0});
                my $aContainers = findEncryptedAmazonLineItemContainers($Shop, $Options{force});
                if (scalar @$aContainers) {
                    restoreAddressDataForShop($Shop, $aContainers);
                }
            }
        }
    );
    my $LogMessage =  sprintf('[%s] Restoring anonymized addresses of Amazon orders for shop %s took %.3f seconds',
        $Options{storename}, $Options{shop}, time() - $StartTime );
    GetLog()->debug($LogMessage);
    printf "%s\n\n", $LogMessage unless $Options{nooutput};
    printf <<TEXT;
To reactivate the AmazonAnonymizeAddressData Scheduler please issue:

\$PERL \$EPAGES/Cartridges/DE_EPAGES/Object/Scripts/set.pl -storename $Options{storename} -path '/Shops/$Options{shop}' AmazonAnonymizeAddresses=1

TEXT
    eval { SynchronizeCache(); };
    Warning(GetError()) if ExistsError();
}

sub findEncryptedAmazonLineItemContainers {
    my ($Shop, $force) = @_;

    my $dbi = GetCurrentDBHandle();
    my $ClassLineItemContainerAmazon = LoadClassByAlias('LineItemContainerAmazon');
    my $ClassCustomerOrder           = LoadClassByAlias('CustomerOrder');
    my $AttributeID                  = $ClassLineItemContainerAmazon->attribute('IsAmazonOrderAnonymized')->id;
    my $SQL = <<SQL;
        SELECT aco.objectid
        FROM   object aco
        INNER  JOIN object paco ON aco.parentid=paco.objectid
        AND    paco.classid = ?
        AND    aco.classid = ?
        AND    aco.siteid = ?
SQL
    if (not $force) {
        $SQL .= 'INNER JOIN booleanattribute ba ON  ba.objectid = aco.objectid AND ba.attributeid = ? AND value = 1'
    }
    # if we force the restore we select the containers regardless of the value of 'IsAmazonOrderAnonymized'
    my $aParameters = [$ClassCustomerOrder->id, $ClassLineItemContainerAmazon->id, $Shop->id];
    push @$aParameters, $AttributeID unless $force;
    my $Result = $dbi->execute($SQL, $aParameters, ['Integer']);
    return GetLoopFromDoubleArray($Result, [qw (ObjectID) ]);
}

sub restoreAddressDataForShop {
    my ($Shop, $aContainers) = @_;

    my $StartTime = time();
    my $CountAddresses = 0;
    foreach my $Container (@$aContainers) {
        my @Addresses  = ();
        my $LineItemContainer = LoadObject($Container->{ObjectID});
        my $Order             = $LineItemContainer->parent();
        push @Addresses, values %{ $Order->get(['BillingAddress', 'ShippingAddress']) };
        push @Addresses, $Order->get('Customer')->get('BillingAddress');
        for (@Addresses)  {
            $_->deanonymize() if defined $_;
            $CountAddresses++;
        }
        $LineItemContainer->set({'IsAmazonOrderAnonymized' => 0});
    }
    unless ($Options{nooutput}) {
        printf "Shop %s : %s addresses from Amazon orders restored took %.3fs\n", $Shop->alias, $CountAddresses, time() - $StartTime
    }
}

sub usage {
    my ($ErrorNote) = @_;

    print $ErrorNote if $ErrorNote;
    print <<END_USAGE;
Description:
    This script restores addresses for Amazon orders from the encrypted fields for the given
    shop (overwritting the anonymized data)

Usage:
    perl $0 [options] [flags]

options:
    -storename     store name for the database connection
    -passwd        database user password
    -shop          only run the script for this shop                          (required)
    -force         force restore even if address is no longer anonymized             (0)

flags:
    -help          show the command line options
    -nooutput      do not print any output
Example:
    perl $0 -storename Store -shop DemoShop
END_USAGE
    exit 2;
}

1;
