#==============================================================================
# package      DE_EPAGES::Core::API::Object::NumberFormat
# state        public
# base         Number::Format
#----------------------------------------------------------------------------------------
# description  At first see 'perldoc Number::Format', this module is based on this
#               CPAN module. Includes extensions like rounding (the Swiss centime) <function round>
#               and corrects the formatting of positive and negative currencies.
#               See also the test script of this package Core/t/NumberFormat.t.
#==============================================================================
package DE_EPAGES::Core::API::Object::NumberFormat;
use base Number::Format;

use strict;
use Math::BigFloat;

use DE_EPAGES::Core::API::TestParameter qw (TestInteger TestFloat);
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( fcmp );

our $CURR_POS_FORMAT = ' x';
our $CURR_NEG_FORMAT = '- x';

sub ErrorDefinition {
    return {
        'UnknownPriceFormat' => 'Letter  must be present in picture in format_price(). Format was #Format',
        'FORMAT_NOT_FLOAT'   => '#Parameter is no formatted float number (#Value), double decimal point (#DecimalPoint).',
    }
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $NumberFormat = DE_EPAGES::Core::API::Object::NumberFormat->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates new formatter/unformatter.
#----------------------------------------------------------------------------------------
# input        %options | options of formatter, keys:
#               <ul>
#                   <li>curr_pos_format - positive currency format (default 'x') - string
#                   <li>curr_neg_format - negative currency format (default '-x') - string
#                   <li>int_currencyid - international currency identifier (optional) - char(3)
#                   <li>thousands_sep     - character inserted between groups of 3 digits
#                   <li>decimal_point     - character separating integer and fractional parts
#                   <li>mon_thousands_sep - like THOUSANDS_SEP, but used for format_price
#                   <li>mon_decimal_point - like DECIMAL_POINT, but used for format_price
#                   <li>int_curr_symbol   - character(s) denoting currency (see format_price())
#                   <li>decimal_digits    - number of digits to the right of dec point (default: 6)
#                   <li>decimal_fill      - boolean; whether to add zeroes to fill out decimal
#                   <li>neg_format        - format to display negative numbers (def ``-x'')
#                   <li>kilo_suffix       - suffix to add when format_bytes formats kilobytes
#                   <li>mega_suffix       -    "    "  "    "        "         "    megabytes
#                   <li>giga_suffix       -    "    "  "    "        "         "    gigabytes
#               </ul> | hash
# return       $NumberFormat | formatter object | object
#========================================================================================
sub new {
    my $class = shift;
    my %args = @_;
    my $curr_pos_format = defined $args{curr_pos_format} ? $args{curr_pos_format} : $CURR_POS_FORMAT;delete $args{curr_pos_format};
    my $curr_neg_format = defined $args{curr_neg_format} ? $args{curr_neg_format} : $CURR_NEG_FORMAT;delete $args{curr_neg_format};
    my $int_currencyid  = defined $args{int_currencyid}  ? $args{int_currencyid}  : undef;           delete $args{int_currencyid};

    delete $args{$_} foreach grep {not defined $args{$_}} keys %args;
    my $self = bless Number::Format::new($class, %args), $class;

    $self->{decimal_digits} = 6 unless defined $args{decimal_digits};
    $self->{curr_pos_format} = $curr_pos_format;
    $self->{curr_neg_format} = $curr_neg_format;
    $self->{int_currencyid} = $int_currencyid if defined $int_currencyid;
    $self->{int_curr_symbol} = $int_currencyid unless defined $args{int_curr_symbol};

    return $self;
}

#========================================================================================
# function     round
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Rounded = $NumberFormat->round($Number, $Precision);
# syntax       my $Rounded = $NumberFormat->round($Number, $Precision, $RoundMode);
#----------------------------------------------------------------------------------------
# description  Round the number.
#----------------------------------------------------------------------------------------
# input        $Number | number in perl | float
# input        $Precision | number of decimal digits (default option decimal_digits of <function new>) | int
# input        $RoundMode | (optional) round mode, see Math::BigFloat->round_mode.
#                   Default: standard commercial practice (Kaufmnnisches Runden http://de.wikipedia.org/wiki/Rundung).
#                   Possible values are: even|odd|+inf|-inf|zero|trunc|ceil | string
# return       $Rounded | rounded number | string
#========================================================================================
sub round {
    my $self = shift;
    my ( $Number, $Precision, $RoundMode ) = @_;
    TestFloat('Number', $Number);
    TestInteger('Precision', $Precision) if defined $Precision;

    $RoundMode //= $Number > 0 ? '+inf' : '-inf';
    $Precision = $self->{decimal_digits} unless defined $Precision;

    my $BigNumber = Math::BigFloat->new($Number)->ffround(-$Precision, $RoundMode);

    # overwrite if we want to round up
    $BigNumber = $self->roundUp($Number, $Precision) if $RoundMode eq 'ceil';

    # convert Math::BigFloat object to perl string
    my $Num = $BigNumber->bstr;

    # remove trailing zeros
    # case 1: remove decimal point and zeros; e.g. 5.000 => 5
    $Num =~ s/\.0+$//;

    # case 2: remove only zeros; e.g. 5.300 => 5.3
    $Num =~ s/(.*\.[0-9]*?)0+$/$1/x;

    return $Num;
}

#========================================================================================
# function     roundUp
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Rounded = $NumberFormat->roundUp($Number, $Precision);
#----------------------------------------------------------------------------------------
# description  Round the number up.
#----------------------------------------------------------------------------------------
# input        $Number | number in perl | float
# input        $Precision | number of decimal digits (default option decimal_digits of <function new>) | int
# return       $Rounded | rounded number | float
#========================================================================================
sub roundUp {
    my $self = shift;
    my ($Number,$Precision) = @_;

    my $RoundedBigNumber    = Math::BigFloat->new($Number)->ffround(-$Precision);
    my $OldBigNumber        = Math::BigFloat->new($Number);

    my $RoundingDirection = $RoundedBigNumber->bcmp($OldBigNumber);

    # we have already rounded up
    return $RoundedBigNumber if $RoundingDirection == abs($RoundingDirection);

    # because we rounded down we now need to add 1 to the last digit of the number
    my $FixRounding = Math::BigFloat->new('0.'.'0' x ($Precision-1).'1');

    $RoundedBigNumber->badd($FixRounding);

    return $RoundedBigNumber;
}

#========================================================================================
# function     roundTotal
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Rounded = $NumberFormat->roundTotal($Number, $Precision);
#----------------------------------------------------------------------------------------
# description  Round the number. Round to 0 or 5 an decimal value for currency CHF.
#----------------------------------------------------------------------------------------
# input        $Number | number in perl | float
# input        $Precision | number of decimal digits (default option decimal_digits of <function new>) | int
# return       $Rounded | rounded number | float
#========================================================================================
sub roundTotal {
    my $self = shift;
    my ($Number, $Precision) = @_;
    TestFloat('Number', $Number);
    TestInteger('Precision', $Precision) if defined $Precision;

    if( defined $self->{'int_currencyid'} && $self->{'int_currencyid'} eq 'CHF' ) {
        # Rappenrundung
        return $self->round($Number / 5, $Precision) * 5;
    }

    return $self->round($Number, $Precision);
}

#========================================================================================
# function     format_price
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Formatted = $NumberFormat->format_price($Number, $Precision);
# syntax       my $Formatted = $NumberFormat->format_price($Number, $Precision, $FormatInformation);
#----------------------------------------------------------------------------------------
# description  Rounds and formats the number to an currency string. If $FormatInformation
#               is set to 'microdata' the returned number is surrounded by a 'span' tag with
#               schema.org micro data information for price and currency, see schema.org/price .
#----------------------------------------------------------------------------------------
# input        $Number | number in perl | float
# input        $Precision | number of decimal digits (default option decimal_digits of <function new>) | int
# input        $FormatInformation | (optional) additional format information, possible data: microdata | string
# return       $Formatted | formatted number | string
#========================================================================================
sub format_price {
    my $self = shift;
    my ($number, $precision, $FormatInformation) = @_;
    TestFloat( 'Number', $number );

    my $IncreasePrecision = 0;
    if (!defined $precision) {
        $precision = $self->{decimal_digits} + 1;
        $IncreasePrecision = 1;
    }
    $precision = 2 unless defined $precision; # default

    my $symbol = $self->{int_curr_symbol};
    my $format = '';
    my $numberString;

    # if wrap currency parts in microdata elements
    if (defined $FormatInformation && $FormatInformation eq 'microdata') {
        $numberString = $self->format_number($number, $precision); # format it first
        $numberString =~ s/0$// if $IncreasePrecision;
        $numberString =~ s/\Q$self->{mon_decimal_point}\E$//x;

        my $CurrencyID = (defined $self->{'int_currencyid'}) ? $self->{'int_currencyid'} : '';
        my $ContentNumberFormatter = Number::Format->new(-thousands_sep   => '',
            -decimal_point   => '.',
            -decimal_digits  => $precision,
            -int_curr_symbol => '',
            -decimal_fill    => 1);
        my $numberContentString = $ContentNumberFormatter->format_number($number);
        $numberContentString =~ s/0$// if $IncreasePrecision;
        $numberContentString =~ s/\Q.\E$//x;

        $numberString = '<span itemprop=\'price\' content=\'' . $numberContentString . '\'>' . $numberString . '</span>';
        $symbol = '<span itemprop=\'priceCurrency\' content=\'' . $CurrencyID . '\'>' . $symbol . '</span>';

        # format
        $format = $self->{curr_pos_format};
    }
    else {
        # format
        my $sign = $number <=> 0;
        $number = abs($number) if $sign < 0;

        $numberString = $self->format_number($number, $precision); # format it first
        $numberString =~ s/0$// if $IncreasePrecision;
        $numberString =~ s/\Q$self->{mon_decimal_point}\E$//x;

        $format = $sign < 0 ? $self->{curr_neg_format} : $self->{curr_pos_format};
    }

    Error('UnknownPriceFormat', { 'Format' => $format }) unless $format =~ //;
    $symbol = '' unless defined $symbol;
    $format =~ s//$symbol/;
    $format =~ s/x/$numberString/;

    return $format;
}

#========================================================================================
# function     unformat_number
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Number = $NumberFormat->unformat_number($Formatted);
#----------------------------------------------------------------------------------------
# description  Picks up the digits of an currency string.
#----------------------------------------------------------------------------------------
# input        $Formatted | formatted number | string
# return       $Number | number in perl | float
#========================================================================================
sub unformat_number {##no critic(RequireArgUnpacking)
    my ($self, $formatted) = Number::Format::_get_self @_;##no critic(ProtectPrivateSubs)
    $self->_check_seps();##no critic(ProhibitPrivateSubroutineUse)
    my $DecimalPoint = defined $self->{int_currencyid} ? $self->{mon_decimal_point} : $self->{decimal_point};
    my $ThousandsSep = defined $self->{int_currencyid} ? $self->{mon_thousands_sep} : $self->{thousands_sep};
    Error('FORMAT_NOT_FLOAT' , {'Parameter' => 'Formatted', 'Value' => $formatted, 'DecimalPoint'=> $DecimalPoint}) unless $formatted =~ /\d/;

    # remove or replace currency symbol with decimal point if it is around numbers, like 45$25.
    if (defined $self->{int_currencyid}) {
        $formatted =~ s/^\s*\Q$self->{int_curr_symbol}\E\s*//x
            or
            $formatted =~ s/\s*\Q$self->{int_curr_symbol}\E\s*$//x
            or
            $formatted !~ /\Q$DecimalPoint\E/x
                and
                $formatted =~ s/(\d)\Q$self->{int_curr_symbol}\E(\d)/$1$DecimalPoint$2/x
            or
            $formatted =~ s/\s*\Q$self->{int_curr_symbol}\E\s*//x
    }

    # remove all spaces
    $formatted =~ s/\s+//g;

    # It's negative if the first non-digit character is a -
    my $sign = ($formatted =~ s/^-// or $formatted =~ s/-$//) ? -1 : 1;
    my($before_re, $after_re) = split /x/, $self->{neg_format}, 2;
    $sign = -1 if $formatted =~ /\Q$before_re\E(.+)\Q$after_re\E/x;

    # Split number into integer and decimal parts
    my ($integer, $decimal, @cruft) = split(/\Q$DecimalPoint\E/, $formatted);
    Error('FORMAT_NOT_FLOAT' , {'Parameter' => 'Formatted', 'Value' => $formatted, 'DecimalPoint'=> $DecimalPoint}) if @cruft;

    # Strip out all non-digits from integer and decimal parts
    $integer = 0 if (!defined $integer || $integer eq '');
    $decimal = 0 unless defined $decimal;

    # remove thousands sep
    $integer =~ s/\Q$ThousandsSep\E//g;

    TestInteger('IntegerPart', $integer);
    TestInteger('DecimalPart', $decimal);

    # Join back up, using period, and add 0 to make Perl think it's a number
    my $number = join('.', $integer, $decimal) + 0;
    $number = -$number if $sign < 0;

    return $number;
}

#========================================================================================
# function     ceiling
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Ceil = $NumberFormat->ceiling($Number);
#----------------------------------------------------------------------------------------
# description  Round to ceiling.
#----------------------------------------------------------------------------------------
# input        $Number | number in perl | float
# return       $Ceil | rounded number | int
#========================================================================================
sub ceiling {
    my $self = shift;
    my ($number) = @_;
    TestFloat('Number', $number);

    my $floor  = int($number);
    return $self->compare($number, $floor) ? $floor + 1 : $floor;
}

#========================================================================================
# function     compare
# state        public
#----------------------------------------------------------------------------------------
# syntax       $result = $Formatter->compare($Amount1, $Amount2);
#----------------------------------------------------------------------------------------
# description  Compares two float numbers (avoid numeric problems).
#----------------------------------------------------------------------------------------
# input        $Amount1 | first float number | float
# input        $Amount2 | second float number | float
# return       $result | 0 if equal / -1 if smaller / 1 if greater / undef if one is no float number | int
#========================================================================================
sub compare {
    my $self = shift;
    my ($Amount1, $Amount2) = @_;
    TestFloat('Amount1', $Amount1);
    TestFloat('Amount2', $Amount2);

    return fcmp($Amount1, $Amount2);
}

#========================================================================================
# function     removeTrailingZeros
# state        public
#----------------------------------------------------------------------------------------
# syntax       $result = $Formatter->removeTrailingZeros( $Number );
#----------------------------------------------------------------------------------------
# description  Remove trailing zeros after decimal point.
#----------------------------------------------------------------------------------------
# input        $Number | The number where the zero should be removed | float
# return       $result | The number without trailing zero | float
#========================================================================================
sub removeTrailingZeros {
    my $self = shift;
    my ($number) = @_;

    return if not defined $number;

    $number =~ s/\.0*$//;

    return $number;
}

1;
