    use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath );

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName = 'Store';
    my $Help;
    my $ObjectPath;

    GetOptions(
        'help'          => \$Help,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'path=s'        => \$ObjectPath,
    );

    usage() if $Help;
    print "\nMissing ebay shop environment path!\n" and usage() unless $ObjectPath;

    my $time1 = time;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            # make sure that the cartridge is installed
            unless (ExistsCartridgeByPackage('DE_EPAGES::Ebay')) {
                print "Store: $StoreName : Cartridge Ebay is not installed.\n";
                return;
            }
            GetSiteDetails( $ObjectPath );
        },
    );
    my $time2 = time;
    my $seconds = $time2-$time1;
    my $logmsg = sprintf ("need %.3f seconds", $seconds);
    print $logmsg."\n";

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -path         path of eBay shop environment
flags:
    -help         show the command line options

Example:
    perl $0 -storename Store -path /Shops/DemoShop/EbayShopEnvironments/EbayShopEnvironment1
END_USAGE
    exit 2;
}

sub GetSiteDetails {
    my ($ObjectPath) = @_;

    my $EbayShopEnv = LoadObjectByPath( $ObjectPath );
    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new('ShopEnvironment'=>$EbayShopEnv);
    my $result;
    if (0) {
        $result = $RemoteEbay->getEbayDetails('RegionDetails');
        foreach my $Element (@{$result->{'RegionDetails'}}) {
            printf "<EbayRegion EbayRegionId=\"%d\" CountryCode=\"%s\" Name=\"%s\" />\n",
                $Element->{'RegionID'},
                $EbayShopEnv->get('EbayCountry'),
                $Element->{'Description'};
            }
    }
    if (1) {
        $result = $RemoteEbay->getEbayDetails('ShippingServiceDetails');
        print "      <EbayShippingMethods Class=\"Object\" Alias=\"EbayShippingMethods\">\n";
        foreach my $Element (@{$result->{'ShippingServiceDetails'}}) {
            printf "        <EbayShippingMethod Alias=\"%s\" Name=\"%s\" IsInternationalService=\"%s\" IsExpeditedService=\"%s\" />\n",
                $Element->{'ShippingService'},
                $Element->{'Description'},
                $Element->{'InternationalService'} ? 1 : 0,
                $Element->{'ExpeditedService'} ? 1 : 0 if $Element->{'ValidForSellingFlow'};
        }
        print "      </EbayShippingMethods>\n";
    }
    if (1) {
        print "      <EbayPaymentMethods Class=\"Object\" Alias=\"EbayPaymentMethods\">\n";
        $result = $RemoteEbay->getEbayDetails('PaymentOptionDetails');
        foreach my $Element (@{$result->{'PaymentOptionDetails'}}) {
            printf "        <EbayPaymentMethod Alias=\"%s\" Name=\"%s\" />\n",
                $Element->{'PaymentOption'},
                $Element->{'Description'};
        }
        print "      </EbayPaymentMethods>\n";
    }
    if (1) {
        $result = $RemoteEbay->getEbayDetails('ShippingLocationDetails');
        print "      <EbayShippingLocations Class=\"Object\" Alias=\"EbayShippingLocations\">\n";
        foreach my $Element (@{$result->{'ShippingLocationDetails'}}) {
            printf "        <EbayShippingLocation Alias=\"%s\" Name=\"%s\" />\n",
                $Element->{'ShippingLocation'},
                $Element->{'Description'};
        }
        print "      </EbayShippingLocations>";
    }
    if (0) {
        $result = $RemoteEbay->getEbayDetails('DispatchTimeMaxDetails');
        print "\n\n" . $EbayShopEnv->get('EbayCountry') . "\n";
        foreach my $Element (@{$result->{'DispatchTimeMaxDetails'}}) {
            printf "<DispatchTimeMax Alias=\"%s\" Name=\"%s\" />\n",
                $Element->{'DispatchTimeMax'},
                $Element->{'Description'};
        }
    }
}

RunScript(Sub=>\&Main, Debug=>1);
