#========================================================================================
# package      DE_EPAGES::GoogleAdWords::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  CleverGoogleAds tools
#========================================================================================
package DE_EPAGES::GoogleAdWords::API::Tools;

use DE_EPAGES::Shop::API::Object::Shop;
use DE_EPAGES::Core::API::Error qw ( ExistsError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::GoogleAdWords::API::AppStoreCleverGoogleAds;

use strict;

use constant APPSTORE_FEATURE => 'AppStore';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self | self | object
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {}, $class;

    return $self;
}

#========================================================================================
# function     IsAppStoreVisible
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsAppStoreVisible( $Shop );
#----------------------------------------------------------------------------------------
# description  Checks whether AppStore is visibile in a given shop or not.
#----------------------------------------------------------------------------------------
# input        $Shop              | A Shop.                       | object
# return       $IsAppStoreVisible | True iff AppStore is visible. | boolean
#========================================================================================
sub IsAppStoreVisible {
    my $self = shift;
    my ($Shop) = @_;

    my $Feature = $self->_getFeature(APPSTORE_FEATURE);

    return $self->_isFeatureVisible($Shop, $Feature);
}

#========================================================================================
# function     IsCleverGoogleAdsAppAvailable
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsCleverGoogleAdsAppAvailable( $Shop );
#----------------------------------------------------------------------------------------
# description  Checks if there's a CleverGoogleAds App available in the AppStore.
#----------------------------------------------------------------------------------------
# input        $Shop                   | A Shop.                          | object
# return       $IsCleverGoogleAdsAppAvailable | True iff there is a CleverGoogleAds App
#                                       available in the AppStore.
#                                                                          | boolean
#========================================================================================
sub IsCleverGoogleAdsAppAvailable {
    my $self = shift;
    my $AppId = $self->CleverGoogleAdsAppID(@_);

    return isValid($AppId) ? 1 : 0;
}

sub CleverGoogleAdsAppID {
    my $self = shift;
    my ($Shop) = @_;

    return if !$self->IsAppStoreVisible($Shop);

    my $AppStoreCleverGoogleAds = $self->_createAppStoreCleverGoogleAds();
    my $AppId;

    eval {
        ## no critic (DE_EPAGES::Subroutines::ProhibitPrivateSubroutineUse)
        $AppId = $AppStoreCleverGoogleAds->getCleverGoogleAdsAppID($Shop);
        ## use critic
    };
    if(ExistsError()) {
        return;
    }

    return $AppId;
}


#---------------------------------------------------------------------------
#  Private methods
#---------------------------------------------------------------------------
sub _getFeature {
    my $self = shift;
    my ($FeatureName) = @_;

    my $System = $self->_LoadRootObject();

    return $System->child('Features')->child($FeatureName);
}

sub _isFeatureVisible {
    my $self = shift;
    my ($Shop, $Feature) = @_;

    return $Shop->featureIsVisible($Feature);
}

#---------------------------------------------------------------------------
#  Wrappers
#---------------------------------------------------------------------------
sub _createAppStoreCleverGoogleAds {
    my $self = shift;

    return DE_EPAGES::GoogleAdWords::API::AppStoreCleverGoogleAds->new();
}

sub _LoadRootObject {
    my $self = shift;

    return LoadRootObject();
}

1;
