#========================================================================================
# package      DE_EPAGES::Object::API::Factory
# state        public
# description  This package provides helper functions for working with objects that
#               are stored in the 'object' table. You can create, retrieve and delete
#               such objects.<br>
#               Use the functions of this package if you don't have a reference to object
#               yet. Once you have an object reference, use the methods of the
#               <package DE_EPAGES::Object::API::Object::Object> or derived package.
#----------------------------------------------------------------------------------------
# example      my $Shop = LoadObjectByPath( 'Shops/DemoShop' );
#               my $Product = LoadClassByAlias('Product')->insertObject({
#                   Parent => $Shop->child('Products'),
#                   Alias => '0815'
#               } );
#========================================================================================
package DE_EPAGES::Object::API::Factory;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    LoadObject
    DeleteObject
    InsertObject
    ExistsObject
    ExistsObjectByGUID
    ExistsObjectByPath
    LoadObjectByGUID
    LoadObjectByPath
    GetRootObjectID
    LoadRootObject

    SplitObjectPath
    JoinObjectPath
    GenerateSplittedGUIDPath

    LoadClassByAlias
    ExistsClassByAlias

    SetObjectAtCache
    UpdateObjectAtCache
    DeleteObjectAtCache
    DisableObjectCache

    DeleteClassAtCache
    ResetFactoryCaches

    Transaction

    SortObjects
);

use Scalar::Util qw ( blessed weaken );
use DE_EPAGES::Core::API::CSV qw( ParseCSV CombineCSV );
use DE_EPAGES::Core::API::Error qw ( CallerError Error );
use DE_EPAGES::Core::API::TestParameter qw ( TestString TestInteger );
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::DataCache::API::Cache; # GetCache UpdateCache UpdateAll UpdateLocal
use DE_EPAGES::DataCache::API::SessionCache; # GetCache UpdateCache UpdateAll
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );  #Transaction
use DE_EPAGES::Database::API::DAL;

my $DAL_PACKAGE = 'DE_EPAGES::Database::API::DAL';
my $PATH_SEPARATOR = '/';
my $PATH_CSV_OPTIONS = { 'sep_char' => $PATH_SEPARATOR };

our $DO_CACHE = 1; # use DisableObjectCache() to temporary disable adding objects to the cache

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NoResults_ObjectByGUID' => 'GUID \'#GUID\' doesn\'t exist',
        'NoResults_ObjectByPath' => 'Path \'#Path\' doesn\'t exist at object \'#RootObjectID\'',
        'NoResults_ObjectID' => 'Object (ObjectID=#ObjectID) doesn\'t exist',
        'NoObjectID'         => 'Can\'t load an undefined Object (LoadObject without ObjectID).',
        'NoClass'           =>  'Class #ClassID doesn\'t exist',
        'NoClassName'       =>  'Class #Alias doesn\'t exist',
        'PARAMETER_MISSING' => 'Missing parameter #Parameter',
    }
}
#ERRORDEFINITION END

sub GetObjectCache() {
    return DE_EPAGES::DataCache::API::Cache::GetCache(GetCurrentDBHandle()->cacheKey, 'Object');
}

sub GetClassCache() {
    return DE_EPAGES::DataCache::API::SessionCache::GetCache('Class');
}

#========================================================================================
# function     InsertObject
#----------------------------------------------------------------------------------------
# description  obsolete
#========================================================================================
sub InsertObject($$) { ## no critic qw(RequireCommentForExportedFunctions)
    my ($Class, $hObject) = @_;
    $Class = LoadClassByAlias($Class) unless ref $Class;

    return $Class->insertObject( $hObject );
}

#========================================================================================
# function     LoadObject
#----------------------------------------------------------------------------------------
# syntax       $Object = LoadObject( $ObjectID );
# syntax       $Object = LoadObject( $ObjectID, $DALPackage );
#----------------------------------------------------------------------------------------
# description  Loads an object by object id.
#----------------------------------------------------------------------------------------
# input        $ObjectID | object id | int
# input        $DALPackage | (optional) DAL package name, used for optimization of
#                   $Class->loadObject() | string
# return       $Object | object | object
#========================================================================================
sub LoadObject($;$);
sub LoadObject($;$) {
    my ($ObjectID, $DALPackage) = @_;
    Error('NoObjectID') unless defined $ObjectID;

    my $ObjectCache = GetObjectCache();
    my $Object = $ObjectCache->{$ObjectID};
    if(not defined $Object or $Object->{'Removed'} ) {

        $DALPackage //= $DAL_PACKAGE->load('DE_EPAGES::Object::API::Object::Object');
        my $hObject = $DALPackage->info( $ObjectID );
        Error( 'NoResults_ObjectID', { ObjectID => $ObjectID } ) unless defined $hObject;

        my $ClassID = $hObject->{'ClassID'};
        my $Class = $ObjectCache->{$ClassID};
        if (defined $Class) {
            $Object = $Class->loadObject( $ObjectID, 1 ); # 1 = load direct
        }
        else {
            my $hClass = $DAL_PACKAGE->load('DE_EPAGES::Object::API::Object::Class')->info($ClassID) or Error('NoClass', { ClassID=>$ClassID });
            my $Package = $hClass->{Package};
            UsePackage($Package);
            $Object = $Package->load( $ObjectID );

            # AD-8977 (Copy the link to this issue scheduler
            # DE_EPAGES/ProductPortal/Scripts/exportFeedFile.pl @ strato: Out of memory!)
            #
            # Adding Object, into the cache immediately post it's creation, avoids a recursion bug
            # in which the child call via '$Object->class' expects the parent Object
            # to already be in the ObjectCache.
            $ObjectCache->{$ObjectID} = $Object;

            $ObjectCache->{$ClassID} = $Object->class unless exists $ObjectCache->{$ClassID}; # could be now in
        }

        $ObjectCache->{$ObjectID} = $Object;
    }
    return $Object;
}

#========================================================================================
# function     DeleteObject
#----------------------------------------------------------------------------------------
# syntax       $Object = DeleteObject( $ObjectID );
#----------------------------------------------------------------------------------------
# description  Deletes an object by object id, if object exists.
#----------------------------------------------------------------------------------------
# input        $ObjectID | object id | int
# return       $Object | object | object
#========================================================================================
sub DeleteObject($) {
    my ($ObjectID) = @_;
    TestInteger( 'ObjectID', $ObjectID );

    LoadObject( $ObjectID )->delete if ExistsObject($ObjectID);
}

#========================================================================================
# function     ExistsObject
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsObject( $ObjectID );
#----------------------------------------------------------------------------------------
# description  Returns true if the object id exists in the database.
#----------------------------------------------------------------------------------------
# input        $ObjectID | object id | int
# return       $Exists | true if the object exists | boolean
#========================================================================================
sub ExistsObject($) {
    my ($ObjectID) = @_;
    TestInteger( 'ObjectID', $ObjectID );
    my $ObjectCache = GetObjectCache();
    return 1 if defined $ObjectCache->{$ObjectID};
    return $DAL_PACKAGE->load('DE_EPAGES::Object::API::Object::Object')->exists( $ObjectID );
}

#========================================================================================
# function     GetRootObjectID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ObjectID = GetRootObjectID();
#----------------------------------------------------------------------------------------
# description  Returns the root object identifier of the system.
#----------------------------------------------------------------------------------------
# return       $ObjectID | object id | int
#========================================================================================
sub GetRootObjectID () { return 1; }

#========================================================================================
# function     LoadRootObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = LoadRootObject();
#----------------------------------------------------------------------------------------
# description  Returns the root object.
#----------------------------------------------------------------------------------------
# return       $Object | root object | object
#========================================================================================
sub LoadRootObject () {
    return GetObjectCache()->{1} // LoadObject( 1 );
}

#========================================================================================
# function     LoadObjectByGUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = LoadObjectByGUID( $GUID );
# example      $Object = LoadObjectByGUID( "b09e53f4-60e9-41a6-8b30-526b69c28b5d" );
#----------------------------------------------------------------------------------------
# description  Returns the object by GUID.
#               Throws an error if no such objects exists.
#----------------------------------------------------------------------------------------
# input        $GUID | general unique id of the object | string
# return       $Object | object  | object
#========================================================================================
sub LoadObjectByGUID ($) {
    my ( $GUID ) = @_;
    TestString( 'GUID', $GUID );

    my $dal = $DAL_PACKAGE->load('DE_EPAGES::Object::API::Object::Object');
    my $Objects = $dal->get('ByGUID', { 'GUID'=>$GUID });
    Error('NoResults_ObjectByGUID', { GUID=>$GUID }) unless scalar @$Objects;
    return LoadObject($Objects->[0]{'ObjectID'});
}

#========================================================================================
# function     LoadObjectByPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = LoadObjectByPath( $Path );
# syntax       $Object = LoadObjectByPath( $Path, $RootObjectID );
# example      $Object = LoadObjectByPath( "Shops/MyShop/HotDeals" );
#----------------------------------------------------------------------------------------
# description  Finds an object by a path of alias names from a given root object.
#               The alias names are separated by slash (/) characters.
#----------------------------------------------------------------------------------------
# input        $Path | path of alias names from the root object | string
# input        $RootObjectID | (optional) object id of a parent or grand parent | int
# return       $Object | object | object
#========================================================================================
sub LoadObjectByPath ($;$) { ## no critic qw(ProhibitPathConcatenation)
    my ( $Path, $RootObjectID ) = @_;
    CallerError('PARAMETER_MISSING', {'Parameter' => 'Path'}) unless $Path;
    TestInteger( 'RootObjectID', $RootObjectID ) if defined $RootObjectID;
    $RootObjectID = GetRootObjectID() unless defined $RootObjectID;

    return LoadObject($RootObjectID)->path($Path);
}

#========================================================================================
# function     ExistsObjectByGUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsObjectByGUID( $GUID );
# example      if( ExistsObjectByGUID( "b09e53f4-60e9-41a6-8b30-526b69c28b5d" ) ) { ... }
#----------------------------------------------------------------------------------------
# description  Finds an object by a general unique id.
#----------------------------------------------------------------------------------------
# input        $GUID | general unique id of object | string
# return       $Exists | true if the specified object exists | boolean
#========================================================================================
sub ExistsObjectByGUID ($) {
    my ( $GUID ) = @_;
    TestString( 'GUID', $GUID );
    my $dal = $DAL_PACKAGE->load('DE_EPAGES::Object::API::Object::Object');
    my $Objects = $dal->get('ByGUID', { 'GUID'=>$GUID });
    return scalar @$Objects ? 1 : 0;
}

#========================================================================================
# function     ExistsObjectByPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsObjectByPath( $Path );
# syntax       $Exists = ExistsObjectByPath( $Path, $RootObjectID );
# example      if( ExistsObjectByPath( "Shops/MyShop/HotDeals" ) ) { ... }
#----------------------------------------------------------------------------------------
# description  Finds an object by a path of alias names from a given root object.
#               The alias names are separated by slash (/) characters.
#----------------------------------------------------------------------------------------
# input        $Path | path of alias names from the root object |
# return       $Exists | true if the specified object exists | boolean
#========================================================================================
sub ExistsObjectByPath ($;$) { ## no critic qw(ProhibitPathConcatenation RequireCommentForExportedFunctions)
    my ( $Path, $RootObjectID ) = @_;
    TestString( 'Path', $Path );
    TestInteger( 'RootObjectID', $RootObjectID ) if defined $RootObjectID;
    $RootObjectID = GetRootObjectID() unless defined $RootObjectID;

    my $ChildID = LoadObject($RootObjectID)->findPath($Path);
    return defined $ChildID ? 1 : 0;
}

#========================================================================================
# function     SplitObjectPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aPath = SplitObjectPath( $Path );
# example      $aPath = SplitObjectPath( '/Shops/DemoShop/Products/"08/15"' );
#----------------------------------------------------------------------------------------
# description  Splits path on '/'.
#               Unquotes any parts that are delimited with doublequotes.
#----------------------------------------------------------------------------------------
# input        $Path | path of alias names from the root object | string
# return       $aPath | list of object alias names | ref.array.string
#========================================================================================
sub SplitObjectPath ($) { ## no critic qw(RequireArgUnpacking RequireCommentForExportedFunctions)
    return ParseCSV( $_[0], $PATH_CSV_OPTIONS );
}

#========================================================================================
# function     JoinObjectPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Path = JoinObjectPath( $aPath );
# example      $Path = JoinObjectPath( [ 'Shops', 'DemoShop', 'Products', '08/15' ] );
#----------------------------------------------------------------------------------------
# description  Joins alias names with '/'.
#               Quotes alias alias names that contain a slash or doublequote character.
#----------------------------------------------------------------------------------------
# input        $aPath | list of object alias names | ref.array.string
# return       $Path | path of alias names from the root object | string
#========================================================================================
sub JoinObjectPath ($) {
    my ($aPath) = @_;

    return CombineCSV( $aPath, $PATH_CSV_OPTIONS );
}

#========================================================================================
# function     GenerateSplittedGUIDPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Path = GenerateSplittedGUIDPath( $GUID );
# example      $Path = GenerateSplittedGUIDPath( "4C17-7D68-9953-E9F3-9C23-FE09-0F99-977A" );
#----------------------------------------------------------------------------------------
# description  Generates a directory path by the given GUID.
#               The GUID is splitted into four-character sets, leading to a subdir
#               per set.
#----------------------------------------------------------------------------------------
# input        $GUID | GUID to take for generating the path| string
# return       $Path | generated directory path | string
#========================================================================================
sub GenerateSplittedGUIDPath ($) {
    my ($GUID) = @_;
    $GUID =~ s/-//g;
    my @Parts;
    for (my $i=0; $i<length($GUID); $i += 4) {
        push @Parts, substr($GUID,$i,4);
    }
    return join '/', @Parts;
}

#========================================================================================
# function     _LoadClassByAlias
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Class = _LoadClassByAlias( $Alias );
#----------------------------------------------------------------------------------------
# description  Loads an existing class by alias.
#               Returns undef if the class does not exist.
#----------------------------------------------------------------------------------------
# input        $Alias | class alias | string
# return       $Class | class object or undef | object
#========================================================================================
sub _LoadClassByAlias ($) {
    my ($Alias) = @_;

    my $hClassCache = GetClassCache();
    my $Class = $hClassCache->{$Alias};
    return $Class if defined $Class;

    my $Parent = LoadRootObject()->child('Classes');
    my $dal = DE_EPAGES::Database::API::DAL->load('DE_EPAGES::Object::API::Object::Class');
    my $ahClasses = $dal->get('ByParentIDAlias', [$Parent->id, $Alias]);
    return undef if 0 == scalar @$ahClasses;

    my $foundAlias = $ahClasses->[0]{'Alias'};
    $hClassCache->{$foundAlias} = LoadObject($ahClasses->[0]{'ObjectID'});
    $Class = $hClassCache->{$foundAlias};
    weaken $hClassCache->{$foundAlias};
    SetObjectAtCache($Class);
    return $Class;
}

#========================================================================================
# function     LoadClassByAlias
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Class = LoadClassByAlias( $Alias );
#----------------------------------------------------------------------------------------
# description  Loads an existing class by alias.
#               Throws the error 'NoClassName' if the class does not exist.
#----------------------------------------------------------------------------------------
# input        $Alias | class alias | string
# return       $Class | class object | object
#========================================================================================
sub LoadClassByAlias ($) {
    my ($Alias) = @_;
    CallerError('PARAMETER_MISSING', {'Parameter' => 'Alias'}) unless $Alias;

    my $Class = _LoadClassByAlias( $Alias );
    Error('NoClassName', { 'Alias' => $Alias }) unless defined $Class;

    return $Class;
}

#========================================================================================
# function     ExistsClassByAlias
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsClassByAlias( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns true if a class with the given alias exists.
#----------------------------------------------------------------------------------------
# input        $Alias  | class alias | string
# return       $Exists | true if the class exists | boolean
#========================================================================================
sub ExistsClassByAlias ($) { ## no critic qw(RequireArgUnpacking RequireCommentForExportedFunctions)
    return defined _LoadClassByAlias($_[0]) ? 1 : 0;
}

#========================================================================================
# function     SetObjectAtCache
# state        private
#----------------------------------------------------------------------------------------
# syntax       SetObjectAtCache( $Object );
#----------------------------------------------------------------------------------------
# description  Adds the object to the data cache, so other functions can easy load this
#               object and get the same object.
#               Does nothing if called within <function DisableObjectCache>
#----------------------------------------------------------------------------------------
# input        $Object | object | object
#========================================================================================
sub SetObjectAtCache ($) {

    return unless $DO_CACHE;

    my ($Object) = @_;

    my $ObjectCache = GetObjectCache();
    $ObjectCache->{$Object->id} = $Object;
    return;
}

#========================================================================================
# function     DisableObjectCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       DisableObjectCache( $Sub );
# example      DisableObjectCache( sub {
#                   my $ChildObjects = $Parent->childrenIterator;
#                   say $_->pathString while <$ChildObjects>;
#               } );
#----------------------------------------------------------------------------------------
# description  Disable <function SetObjectAtCacheadding> during execution of a function.
#               This is useful for export processes that load many objects, but only once.
#----------------------------------------------------------------------------------------
# input        $Sub | function to execute without adding new objects to the cache | ref.code
#========================================================================================
sub DisableObjectCache($) {
    my ($Sub) = @_;

    local $DO_CACHE = 0;
    $Sub->();
}

#========================================================================================
# function     DeleteObjectAtCache
# state        private
#----------------------------------------------------------------------------------------
# syntax       DeleteObjectAtCache( $Object );
# syntax       DeleteObjectAtCache( $ObjectID );
#----------------------------------------------------------------------------------------
# description  Use this function to inform other appservers that the object is
#               removed.
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $ObjectID | object identifier | integer
#========================================================================================
sub DeleteObjectAtCache ($) { ##no critic qw(RequireCommentForExportedFunctions)
    my ($Object) = @_;
    DE_EPAGES::DataCache::API::Cache::UpdateCache(GetCurrentDBHandle()->cacheKey, 'Object',ref $Object ? $Object->id : $Object);
    return;
}

#========================================================================================
# function     UpdateObjectAtCache
# state        private
#----------------------------------------------------------------------------------------
# syntax       UpdateObjectAtCache( $Object );
#----------------------------------------------------------------------------------------
# description  Use this function inform other appservers that the object was
#               changed.
#----------------------------------------------------------------------------------------
# input        $Object | object | object
#========================================================================================
sub UpdateObjectAtCache ($) {
    my ($Object) = @_;
    DeleteObjectAtCache($Object);
    SetObjectAtCache($Object);
    return;
}

#========================================================================================
# function     DeleteClassAtCache
# state        private
#----------------------------------------------------------------------------------------
# syntax       DeleteClassAtCache( $Class );
#----------------------------------------------------------------------------------------
# description  Deletes class at cache, at this package because session cache
#               remembers package name of cache.
#----------------------------------------------------------------------------------------
# input        $Class | class | object
#========================================================================================
sub DeleteClassAtCache ($) {
    my ($Class) = @_;
    DE_EPAGES::DataCache::API::SessionCache::UpdateCache('Class',$Class->alias);
    return;
}

#========================================================================================
# function     ResetFactoryCaches
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetFactoryCaches();
#----------------------------------------------------------------------------------------
# description  Clear the local session cache and cears all caches for the current
#               database. Also send sends a full cache reset for the current database to
#               other appservers.
#========================================================================================
sub ResetFactoryCaches () {
    DE_EPAGES::DataCache::API::SessionCache::UpdateAll();
    DE_EPAGES::DataCache::API::Cache::UpdateConnection(GetCurrentDBHandle()->cacheKey);
    return;
}

#========================================================================================
# function     Transaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       Transaction( $cCode );
# syntax       Transaction( $cCode, $cRollback );
# example      Transaction( sub {
#                   DeleteObject( ... );
#                   DeleteObject( ... );
#               } );
# example      Transaction( sub {
#                   DeleteObject( ... );
#                   DeleteObject( ... );
#               }, sub {
#                   print STDERR "Our transaction was rolled back";
#}              } );
#----------------------------------------------------------------------------------------
# description  Runs a code block in a transaction using the current database handle.
#               If an error occurs, the local data cache is reset and the code block
#               $cRollback is executed.
#----------------------------------------------------------------------------------------
# input        $cCode | code block | code reference
# input        $cRollback | roll back code block (optional) | code reference
#========================================================================================
sub Transaction ($;$) {
    my ($cCode, $cRollback) = @_;
    DE_EPAGES::Database::API::Connection::Transaction($cCode, sub {
        DE_EPAGES::DataCache::API::SessionCache::UpdateAll();
        DE_EPAGES::DataCache::API::Cache::UpdateLocal();
        $cRollback->() if defined $cRollback;
    });
    return;
}

#========================================================================================
# function     SortObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSortedObjects = SortObjects($aObjects, $OrderBy, $OrderDesc);
# example      $aSortedObjects = SortObjects($aObjects, "IsVisible", 1);
#----------------------------------------------------------------------------------------
# description  Sorts the objects according to the 'OrderBy' and 'OrderDesc' parameters.
#----------------------------------------------------------------------------------------
# input        $aObjects | list of objects | ref.array.object
# input        $OrderBy | attribute (path) to order by | string
# input        $OrderDesc | order descending | boolean
# return       $aSortedObjects | sorted list of objects | ref.array.object
#========================================================================================
sub SortObjects ($$;$) {
    my ($aObjects, $OrderBy, $OrderDesc) = @_;
    $OrderDesc = 0 unless defined $OrderDesc;

    return [] unless scalar @$aObjects;

    # get values to sort
    my @OrderBy = defined $OrderBy ? split( /\./, $OrderBy ) : ();

    # keep original order if no sorting is required
    return $aObjects unless scalar @OrderBy;

    my %Values;
    for (my $i=0; $i < scalar @$aObjects; $i++) {
        my $Attribute = $aObjects->[$i];
        foreach my $_OrderBy (@OrderBy) {
            if (ref $Attribute eq 'HASH') {
                if (exists($Attribute->{'Object'})) {
                    $Attribute = $Attribute->{'Object'};
                    $Attribute = $Attribute->get($_OrderBy) ;
                } else {
                    $Attribute = $Attribute->{$_OrderBy};
                }
            } else {
                $Attribute = $Attribute->get($_OrderBy);
            }
            # break loop, when attribute got undef
            last unless defined $Attribute;
        }
        $Values{$i} = $Attribute;
    }
    # remove undefined values
    my @SortedKeys = grep { not defined $Values{$_} } keys %Values;

    if (ref $Values{0} eq 'ARRAY') {
        # sort by element count, if sort object is an array
        push @SortedKeys, sort {scalar @{$Values{$a}} <=> scalar @{$Values{$b}}} grep { defined $Values{$_} } keys %Values;
    } else {
        # sort different on different types
        my $Object = $aObjects->[0];
        if (ref $Object eq 'HASH' && exists($Object->{'Object'})) {
            $Object = $Object->{'Object'};
        }
        if (scalar @OrderBy > 1){
            $Object = $Object->get($_) foreach @OrderBy[0..(scalar @OrderBy - 2)];
        }
        my $Type = 'String';
        if (blessed ($Object)) {
            if ($Object->can('class')) {
                $Type = $Object->class->attribute($OrderBy[-1])->type;
            }elsif ($Object->isa('DateTime')) {
                $Type = 'DateTime';
            }
        } else { #look at the values to guess the type Integer or Float
            my @Values = grep { defined $_ } values %Values;
            if (scalar(grep { /^[-+]?\d+$/ } @Values) == scalar @Values) {
                $Type = 'Integer';
            } elsif (scalar(grep { /^[-+]?(?:\d*(?:\.\d*)|\d+)(?:[eE][-+]?\d+)?$/x } @Values) == scalar @Values) {
                $Type = 'Float';
            }
        }
        if ($Type ~~ [qw (Integer Float Boolean Money DateTime)]) {
            push @SortedKeys, sort {$Values{$a} <=> $Values{$b}} grep { defined $Values{$_} } keys %Values;
        } else {
            push @SortedKeys, sort {uc $Values{$a} cmp uc $Values{$b}} grep { defined $Values{$_} } keys %Values;
        }
    }
    my @Objects = @$aObjects[@SortedKeys];

    # sorting descending ?
    @Objects = reverse @Objects if $OrderDesc;

    return \@Objects;
}

1;
