#==============================================================================
# package      DE_EPAGES::Order::UI::Basket
# state        public
# description  UI class for Basket
#==============================================================================
package DE_EPAGES::Order::UI::Basket;
use base DE_EPAGES::Order::UI::Basket_Ext;

use strict;
use DE_EPAGES::Address::API::Address qw( CheckZipcode );
use DE_EPAGES::Core::API::Error qw ( Error GetError ExistsError );
use DE_EPAGES::Core::API::PerlTools qw ( Compare );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Design::UI::Address;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject Transaction ExistsObjectByGUID LoadObjectByGUID);
use DE_EPAGES::Order::API::Basket qw( CheckBasketAddress IsFiscalCodeRequired );
use DE_EPAGES::Order::API::LineItemContainer qw( CheckSavePayment UpdateTaxModel );
use DE_EPAGES::Order::API::Order qw ( Basket2Order );
use DE_EPAGES::Order::DAL::Basket qw( DAL_UnCachedInfoBasket );
use DE_EPAGES::Presentation::API::Session qw ( SetSessionAttribute );
use DE_EPAGES::Shop::API::Address qw ( CheckStringVATID CheckVATID UpdateVATID );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;


#==============================================================================
# Event handlers
#==============================================================================
sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'ViewObjectID' => $Basket->id,
            'OrderGUID' => $Basket->get('OrderGUID'),
        },
        'ViewAction' => 'ViewBlocked',
    }) if defined $Basket->get('OrderGUID');

    $self->SUPER::Save($Servlet);

    TriggerHook('UI_AfterSaveOrder', {'Servlet' => $Servlet});

    return;
}

sub SaveAndDoOrder {
    my $self = shift;
    my ($Servlet) = @_;
    $Servlet->doAction('Save');
    $Servlet->doAction('DoOrder');
}

#========================================================================================
# function     doBasket2Order
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Order = $Package->doBasket2Order($Basket);
#----------------------------------------------------------------------------------------
# description  Triggers Basket2Order and sets 'SEND_MAIL_FAILED' if the order
#               confirmation mail couldn't be sent off. A message is shown if the basket
#               is currently being created (BASKET_BLOCKED).
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet |  object
# input        $Basket | basket |  object
# return       $Order | new customer order | object
#========================================================================================
sub doBasket2Order {
    my $self = shift;
    my ($Servlet, $Basket) = @_;

    my $Order;
    my $aWarnings = BlockWarnings( sub
    {
        eval { $Order = Basket2Order($Basket); };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) if $Error->code ne 'BASKET_BLOCKED';
            # show basket template
            Error('INVALID_FORM', {
                'ViewVars'=> {
                    'FormError' => 0,
                    'IsBlocked' => 1,
                    'ViewObjectID' => $Basket->id,
                    'OrderGUID' => $Error->vars->{'OrderGUID'},
                },
                'ViewAction' => 'ViewBlocked',
            });
        }
    }, ['SEND_MAIL_FAILED', 'MISSING_FROM']);
    $Servlet->vars('SEND_MAIL_FAILED',1) if scalar @{$aWarnings};

    return $Order;
}

sub DoOrder {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;

    my $Container = $Basket->get('LineItemContainer');
    my $aLineItems = $Container->get('LineItems');
    foreach my $LineItem ( @{$aLineItems} ) {
        next if (! $LineItem->instanceOf('LineItemProduct'));
        my $WantedQuantity = $LineItem->get('Quantity');
        my $Product = $LineItem->get('Product');
        my $LineItemsQuantity = $Product->getQuantityInOtherLineItems($Container, $LineItem);
        my $NewQuantity = $Product->availableQuantity($WantedQuantity, $LineItemsQuantity);
        $LineItem->set({'IsChanged'  => 1, 'Quantity' => $NewQuantity}) if( Compare('Float', $NewQuantity, $WantedQuantity) );
    }

    # check if customer is allowed to place an order
    my $Customer = $Basket->get('Customer');
    if ($Customer and not $Customer->get('IsDoOrderAllowed') ) {
        $Servlet->form->executeFormError({
            'Form' => 'NoForm',
            'Name' => 'BasketForm',
            'Reason' => 'CustomerCanNotOrder',
        });
    }

    # for UI_Basket2Order and UI_OrderCreated
    my %HookVars = (
        'Basket'  => $Basket,
        'Servlet' => $Servlet,
    );

    my $Order;
    my $OrderGUID = DAL_UnCachedInfoBasket($Basket->id)->{'OrderGUID'};
    my $OrderProcess = $Basket->get('Site')->get('OrderProcess');  # 0=classic 1=multistep/onepage

    # use the old order process for Appointment baskets
    $OrderProcess = undef if $Basket->get('Parent')->get('Alias') eq 'AppointmentBaskets';

    # workaround because the new checkout isn't available for mobile at the moment
    $OrderProcess = undef if $Servlet->vars('IsMobile');

    if( defined $OrderGUID ) {
        if( ExistsObjectByGUID($OrderGUID) ) {
            # order already exist -> go to payment or order confirmation
            $Order = LoadObjectByGUID($OrderGUID);
        }
        else {
            # order is beeing created by another process -> show the basket
            Error('INVALID_FORM', {
                'ViewVars'=> {
                    'FormError' => 0,
                    'IsBlocked' => 1,
                    'ObjectID' => $Basket->id,
                    'ViewObjectID' => $Basket->id,
                    'OrderGUID' => $OrderGUID,
                },
                'ViewAction' => 'ViewBlocked',
            });
        }
    }
    else {

        # Trigger UI_Basket2Order only if we have the classic order process enabled
        # test if we have a single step checkout
        if (! $OrderProcess) {
            TriggerHook('UI_Basket2Order', \%HookVars);
        } else {
            TriggerHook('UI_Basket2OrderMultiStepCheckout', \%HookVars);
        }

        # EPG-23485 - etracker: wrong etracker targets in new checkout
        # Customer order page should be separately tracked if new or old checkout is active.
        if ($OrderProcess && $Servlet->params('ViewAction') eq 'View') {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutCustomerOrder');
        }

        $Order = $self->doBasket2Order($Servlet, $Basket);
        $Servlet->vars('ViewObjectID', $Order->id);
    }

    # triggers payment on order
    $HookVars{'Order'} = $Order;

    # Trigger UI_OrderCreated only if we have the classic order process enabled
    # test if we have a single step checkout
    if (! $OrderProcess) {
        TriggerHook('UI_OrderCreated', \%HookVars);
    } else {
        TriggerHook('UI_OrderCreatedMultiStepCheckout', \%HookVars);
    }

    my $OrderCustomer = $Order->get('Customer');
    if (defined $OrderCustomer) {
        if ($Order->get('Shop')->get('IsRatingInvitationAcceptanceRequired')) {
            if ($Servlet->form->value('IsRatingInvitationAllowed')) {
                $OrderCustomer->set({'IsRatingInvitationAllowed' => 1});
            }
            else {
                # AD-4527: do not send any email if the customer rejected the rating invitation
                $Order->set({'IsOpenForRating' => undef}) unless $OrderCustomer->get('IsRatingInvitationAllowed');
            }
        }
    }

    return $Order;
}


sub TestTermsAndConditions {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;

    # term and conditions not accepted
    if (defined $Servlet->params('tac')) {
        $Basket->set({'IsTacOK' => 1});
    }
    else {
        $Basket->set({'IsTacOK' => 0});
        my %Vars;
        $Vars{'ObjectID'}   = $Basket->id;
        $Vars{'ViewAction'} = 'ViewOffer';
        $Vars{'TacAgain'}   = 1;
        return \%Vars;
    }
    return;
}

sub TestCustomerInformation {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;

    # term and conditions not accepted
    if (defined $Servlet->params('customerinformation')) {
        $Basket->set({'IsCustomerInformationOK' => 1});
    }
    else {
        $Basket->set({'IsCustomerInformationOK' => 0});
        my %Vars;
        $Vars{'ObjectID'}   = $Basket->id;
        $Vars{'ViewAction'} = 'ViewOffer';
        $Vars{'CustomerInformationAgain'}   = 1;
        return \%Vars;
    }
    return;
}

sub SelectAddress {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;
    my $FormValues = $Servlet->form->form($Basket,'SelectAddress');

    my $Shop = $Basket->getSite();
    my %Info;
    # set billing address
    my $BillingAddress = $FormValues->{'BillingAddress'} == -1
         ? undef
         : $Shop->loadObject($FormValues->{'BillingAddress'}, 'Address');

    # don't allow to view the address of someone else
    $Servlet->testPermission($BillingAddress, 'View')  if $BillingAddress;

    $Info{'BillingAddress'} = $BillingAddress;
    $Info{'IsAddressOK'}    = 1 if defined $BillingAddress;

    # set shipping address
    my $ShippingAddress = ($FormValues->{'ShippingAddress'} == -1)
         ? undef
         : $Shop->loadObject($FormValues->{'ShippingAddress'}, 'Address');
    $Info{'ShippingAddress'} = $ShippingAddress;

    # don't allow to view the address of someone else
    $Servlet->testPermission($ShippingAddress, 'View')  if $ShippingAddress;

    # set shop address
    my $ShopAddress = $Basket->get('ShopAddress');
    if (not defined $ShopAddress) {
        $ShopAddress = $Basket->getSite()->get('Address');
        $Info{'ShopAddress'} = $ShopAddress if defined $ShopAddress;
    }
    $Basket->set(\%Info);
    TriggerHook('UI_SelectAddress', { 'Servlet' => $Servlet });
    return;
}

sub VerifyAdresses {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;
    my $Shop = $Basket->getSite();

    my $Form = $Servlet->form;
    my $Container = $Basket->container;
    my @AddressTypes = ('BillingAddress');
    push @AddressTypes, 'ShippingAddress' if $Container->get('Shipping');
    my $aRegistrationSet = scalar @AddressTypes ? $Shop->get('RegistrationSet') : [];
    foreach my $AddressType (@AddressTypes) {
        my $Address = $Basket->get($AddressType);
        $Address = $Basket->get('BillingAddress') if not defined $Address and $AddressType eq 'ShippingAddress';
        if (not defined $Address) {
            $Form->addFormError({
                Reason=> uc $AddressType . '_EMPTY',
                Name  => $AddressType,
                Form  => $AddressType
            });
            next;
        }

        # special exception for Italy EPG-21131
        my $CountryField = $AddressType eq 'ShippingAddress' ? 'ShippingCountryID' : 'CountryID';
        if( $AddressType eq 'BillingAddress' && IsFiscalCodeRequired($Basket, $AddressType, $Form->value($CountryField))
            && !defined $Address->get( 'FiscalCode' )
        ) {
            $Form->addFormError({
                Reason      => join ('_', uc $AddressType, 'REGISTRATION_SET'),
                Name        => 'FiscalCode',
                Form        => $AddressType,
                Value       => '',
            });
        }

        my $ClassAddress = $Address->class;
        foreach my $Set (grep { $_->{'Is'.$AddressType. 'Mandatory'} } @$aRegistrationSet) {
            my $AttributeAlias = $Set->{'Attribute'}->alias;
            next unless $ClassAddress->existsAttribute( $AttributeAlias );
            $Form->addFormError({
                Reason      => join ('_', uc $AddressType, 'REGISTRATION_SET'),
                Name        => $AttributeAlias,
                Form        => $AddressType,
                Value       => '',
            }) unless defined $Address->get( $AttributeAlias );
        }
    }
}

#========================================================================================
# function     checkAddresses
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->checkAddresses($Servlet);
#----------------------------------------------------------------------------------------
# description  Checks if all required address data has been entered according to the
#               particular registration set (non-member or shipping address).
#               Adds a form error for each input field that is not correctly filled in.
#               Used at storefront only.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
# return       $hAddresses | shipping and/or billing address values. Hash keys are:
#                           <ul>
#                               <li>Billing - billing address values - ref.hash</li>
#                               <li>Shipping - (optional) shipping address values - ref.hash</li>
#                           </ul>
#                           | ref.hash
#========================================================================================
sub checkAddresses {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;
    my $Form = $Servlet->form;

    # get registration set
    my $Shop = $Basket->getSite();
    my $aRegistrationSet = $Shop->get('RegistrationSet');
    # Check if Shipping is used
    my $UsesShipping = 0;
    foreach my $Set (@$aRegistrationSet) {
        my $Value = $Form->value('Shipping'.$Set->{'Attribute'}->alias);
        $UsesShipping = 1 if (defined $Value and not $Value !~ /\S/);
    }
    $UsesShipping = $UsesShipping && $Form->value('EnableShippingAddress');

    my %Addresses;
    foreach my $AddressType (qw( Billing Shipping )) {
        next if $AddressType eq 'Shipping' && !$UsesShipping;

        my $hAddress = CheckBasketAddress($Form, $Basket, $AddressType, $UsesShipping);
        if (defined $hAddress->{'VATID'} && CheckStringVATID($hAddress->{'VATID'})) {
            $AddressType eq 'Billing' ? $Servlet->vars('VATID', $hAddress->{'VATID'}) :  $Servlet->vars('ShippingVATID', $hAddress->{'VATID'});
        }
        $Addresses{$AddressType} = $hAddress if scalar keys %$hAddress;

        TriggerHook('UI_BasketAddressValidation', {'Servlet' => $Servlet});
    }

    return \%Addresses;
}


#========================================================================================
# function     saveAddresses
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->saveAddresses($Servlet, $hAddresses);
#----------------------------------------------------------------------------------------
# description  Saves billing and shipping address data for Basket and User.
#               Sets the basket attribute IsAddressOK=1.
#               Used at storefront only.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
# input        $hAddresses | shipping and/or billing address values. Hash keys are:
#                           <ul>
#                               <li>Billing - billing address values - ref.hash</li>
#                               <li>Shipping - (optional) shipping address values - ref.hash</li>
#                           </ul>
#                           | ref.hash
#========================================================================================
sub saveAddresses {
    my $self = shift;
    my ($Servlet, $hAddresses) = @_;
    my $Basket = $Servlet->object;
    my $Form = $Servlet->form;

    my $User = $Basket->get('User');
    Transaction ( sub {
        # SaveAdress
        my %BasketInfos;
        my %UserInfos;
        if( defined $Form->value('IsHtmlEMailAllowed') ) {
            $UserInfos{'IsHtmlEMailAllowed'} = $Form->value('IsHtmlEMailAllowed') ? 1 : 0;
        }
        foreach my $AddressType (qw (Billing Shipping)) {
            next unless exists $hAddresses->{$AddressType};
            my $hAddress = $hAddresses->{$AddressType};
            my $Address = $User->get($AddressType.'Address');
            if (defined $Address) {
                # address changed
                $Address->set( $hAddress );
            } else {
                my $Folder = $User->folder('Addresses');
                $hAddress->{'Alias'} = $Folder->nextAlias($AddressType .'%d');
                $Address = $Folder->insertChild( 'Address', $hAddress );
            }
            $UserInfos{$AddressType.'Address'} = $BasketInfos{$AddressType.'Address'} = $Address;
        }
        $Basket->set(\%BasketInfos);
        $User->set(\%UserInfos);
    });
    return;
}

sub ViewAddress {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;

    my $User = $Basket->get('User');
    $Servlet->vars('IsHtmlEMailAllowed', $User->get('IsHtmlEMailAllowed'));

    # set addresses from from user if not set from basket
    my $BillingAddress = $Basket->get('BillingAddress') // $User->get('BillingAddress');
    if (defined $BillingAddress) {
        my $hValues = $BillingAddress->exportableAttributes;
        $Servlet->vars($_, $hValues->{$_}) foreach keys %$hValues;
    }

    my $ShippingAddress = $Basket->get('ShippingAddress') // $User->get('ShippingAddress');
    if (defined $ShippingAddress) {
        my $hValues = $ShippingAddress->exportableAttributes;
        $Servlet->vars('Shipping'. $_, $hValues->{$_}) foreach keys %$hValues;
    }

    # trigger hook
    TriggerHook('UI_ViewClassicalCheckoutAddress', {'Servlet' => $Servlet});

    return;
}

sub ViewEditAddress {
    my $self = shift;
    my ($Servlet) = @_;

    my $AddressID = $Servlet->params('AddressID');
    my $Address = $Servlet->object()->getSite()->loadObject( $AddressID, 'Address' );

    # don't allow to view the address of someone else
    $Servlet->testPermission($Address, 'View');

    $Servlet->vars( 'Address', $Address->tleHash );
    return;
}

#========================================================================================
# function     NewShippingAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->NewShippingAddress($Servlet);
#----------------------------------------------------------------------------------------
# description  Location: SF / Basket / Address / Add shipping address / Save.
#               Adds a new user address and sets it as shipping address of the
#               current basket.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub NewShippingAddress {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;

    $Servlet->vars('AddressType', 'ShippingAddress');

    # add a user address
    my $User = $Basket->get('User');
    my $Folder = $User->folder('Addresses');
    my $Class = LoadClassByAlias('Address');

    my $Form = $Servlet->form;
    my $ZipCode = $Form->value('Zipcode');
    my $CountryID = $Form->value('CountryID');
    $Form->addFormError({
        'Reason'=> 'ZIPCODE_NOT_VALID',
        'Form'  => 'SaveRegistration',
        'Name'  => 'Zipcode',
    }) unless CheckZipcode($ZipCode, $CountryID);

    TriggerHook('UI_NewShippingAddressValidation', {'Servlet' => $Servlet});

    my $NewAddress = DE_EPAGES::Design::UI::Address->NewObject( $Servlet, $Folder, $Class, {
        Alias => $Folder->nextAlias('Address%d'),
    });

    my $VATID = _checkVATID($Servlet, $Servlet->object);
    $NewAddress->set({ VATID => $VATID });

    # set the basket shipping address
    $Basket->set({
        ShippingAddress => $NewAddress,
        IsAddressOK => 0,               # go back to the address selection
    });

    # set the user default shipping address if form value SetAsDefault is true
    $User->set({ ShippingAddress => $NewAddress }) if $Servlet->form->value('SetAsDefault');

    return;
}

sub _checkVATID {
    my ($Servlet, $Object) = @_;
    my $Form = $Servlet->form;
    my $VATID = $Form->value('VATID');
    my $Shop = $Object->getSite();

    if ($VATID) {
        my $CountryID = $Form->value('CountryID');
        $VATID = UpdateVATID($VATID, $CountryID, $Shop);
        my $check = CheckVATID($VATID, $CountryID, $Shop);
        $Servlet->vars('VATID', $VATID);
        $Form->addFormError({
            'Reason'=> $check ? 'VATIDIsNotFromSameCountry' : 'VATIDIsNotValid',
            'Form'  => 'SaveRegistration',
            'Name'  => 'VATID',
        }) unless $check==1;
        $Form->executeFormError;
    }
    return $VATID;
}

sub SavePaymentHirePurchase {
    my $self    = shift;
    my ($Servlet) = @_;
    my $Basket  = $Servlet->object;
    my $Container = $Basket->container();
    my $LineItemPayment = $Container->get('Payment');

    #--- check if hirepurchase possible
    $Servlet->form->executeFormError( {
        'Name'       => 'NumberOfMonths',
        'Reason'     => 'HirePurchaseNotAllowed',
        'ViewAction' => 'View',
    } ) unless $LineItemPayment->get('IsHirePurchaseAllowed');

    my $aHirePurchaseInstallments = $LineItemPayment->get('HirePurchaseInstallments');
    my $SelectedNoOfMonths = $Servlet->params('NumberOfMonths');

    # find installment
    my ($hInstallment) = $SelectedNoOfMonths
        ? grep {$_->{'NumberOfMonths'} == $SelectedNoOfMonths } @$aHirePurchaseInstallments
        : undef;

    # save hire purchase data
    if (defined $hInstallment) {
        $LineItemPayment->set({'HirePurchaseNoOfMonths'        => $SelectedNoOfMonths,
                               'HirePurchaseInterestRateValue' => $hInstallment->{'InterestRateValue'},
                               'HirePurchaseInstallmentAmount' => $hInstallment->{'InstallmentAmount'},
                               'BasePrice'                     => $hInstallment->{'BasePrice'},
                               'IsHirePurchaseOK'              => 1,
                              });
    } else {
        # reset
        $LineItemPayment->set({'HirePurchaseNoOfMonths'        => undef,
                               'HirePurchaseInterestRateValue' => undef,
                               'HirePurchaseInstallmentAmount' => undef,
                               'BasePrice'                     => undef,
                               'IsHirePurchaseOK'              => $SelectedNoOfMonths ? 0 : 1, # is selected month != 0 no month found
                               });

    }
    $Container->recalculate();
}

sub SaveCustomerData {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->testSecureConnection();
    $Servlet->testPostRequest();

    my $Basket = $Servlet->object;
    my $Form = $Servlet->form;
    my $UserName = $Form->value('UserName');
    my $CustomerType = $Form->value('CustomerType');

    # check input values
    my $hAddresses = $self->checkAddresses($Servlet);

    # test the Privacy Policy checkbox
    if( $Basket->getSite()->get('AcceptPrivacyPolicy') ) {
        unless( $Form->value('AcceptPrivacyPolicy') ) {
            $Form->addFormError({
                'Reason'=> 'STRING_EMPTY',
                'Form'  => 'SaveRegistration',
                'Name'  => 'AcceptPrivacyPolicy',
            });
        }
    }

    # newsletter is subscribed in checkout
    $Servlet->params('SubscribedInCheckout', 1);
    TriggerHook('UI_BeforeSaveCustomerData', { 'Servlet' => $Servlet});

    SetSessionAttribute( $Servlet->intSessionID, 'TaxModelChanged', 0);

    if ($CustomerType eq 'GetMember' and defined $UserName and $UserName =~ /\S/) {
        # registered user
        my $Shop = $Basket->getSite();
        $Servlet->vars('ObjectID', $Shop->id);
        eval {
            # check input values
            my $hAttributeValues = $self->checkRegistration($Servlet);
            $Servlet->form->executeFormError;
            # save input values
            $hAttributeValues->{'Adresses'} = $hAddresses;
            my ($User, $Customer, $Password, $NewTaxModel) = $self->saveRegistration($Servlet, $hAttributeValues);
            $self->SendCustomerWelcomeMail($Servlet, $User, $Customer, $Password) if $Shop->isUnity();
            # login user
            $self->LoginUser($Servlet, $User);
            if (defined $NewTaxModel) {
                $Basket->recalculate;
                SetSessionAttribute( $Servlet->intSessionID, 'TaxModelChanged', 1) unless $Shop->get('VATreleased');
            }
        };
        my $Error = ExistsError() ? GetError() : undef;
        Error($Error) if defined $Error;
        $Servlet->vars('ObjectID', $Basket->id);
    }

    $Servlet->form->executeFormError;
    $self->saveAddresses($Servlet, $hAddresses);


    unless ($CustomerType eq 'GetMember' and defined $UserName and $UserName =~ /\S/) {
        if (defined UpdateTaxModel($Basket->container)) {
            $Basket->recalculate;
            SetSessionAttribute( $Servlet->intSessionID, 'TaxModelChanged', 1) unless $Basket->getSite()->get('VATreleased');
        }
    }

    TriggerHook('UI_AfterSaveCustomerData', { 'Servlet' => $Servlet });

    return;
}

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;

    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'ViewObjectID' => $Basket->id,
            'OrderGUID' => $Basket->get('OrderGUID'),
        },
        'ViewAction' => 'ViewBlocked',
    }) if defined $Basket->get('OrderGUID');

    # test if we have a single step checkout
    my $OrderProcess = $Basket->get('Site')->get('OrderProcess');  # 0=classic 1=singlestep/onepage

    # workaround because the new checkout isn't available for mobile at the moment
    $OrderProcess = undef if $Servlet->vars('IsMobile');

    TriggerHook('UI_OnViewBasket', {
        'Servlet' => $Servlet,
        'Basket'  => $Basket
    });

    if (! $OrderProcess || $Basket->get('Parent')->get('Alias') eq 'AppointmentBaskets') {
        # classic handling for basket view
        my $LineItemContainer = $Basket->container;
        if( defined $LineItemContainer  ) {
            my $BasketPositions = scalar @{$LineItemContainer->get('LineItems')};
            my $MaxBasketPositions = LoadRootObject()->get('MaxBasketPositions');
            $Servlet->vars('ReachMaxBasketPositions', 1) if ($BasketPositions >= $MaxBasketPositions);
        }

        # check if the default payment method is valid
        CheckSavePayment($LineItemContainer);

        $self->SUPER::View($Servlet);


    } else {

        # get the start step from the order process list, change and call the new ViewAction
        my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);

        my $StartViewAction;
        my $OrderProcessSteps = $BasketOrderProcess->steps;
        if ( defined $OrderProcessSteps ) {
            my $StartStep = $OrderProcessSteps->[0];
            $StartViewAction = $StartStep->{'Step'}->get('ViewAction')->alias();
        } else {
            $StartViewAction = 'ViewMultiCheckoutBasket';
        }
        $Servlet->params('ViewAction',$StartViewAction);
        $Servlet->doAction($StartViewAction) ;


        return;
    }
}

sub Delete {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;

    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'ViewObjectID' => $Basket->id,
            'OrderGUID' => $Basket->get('OrderGUID'),
        },
        'ViewAction' => 'ViewBlocked',
    }) if defined $Basket->get('OrderGUID');
    $self->SUPER::Delete($Servlet);
}

sub ViewBlocked {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;

    # show the correct confirmation pages for both checkouts
    my $OrderProcess = $Basket->get('Site')->get('OrderProcess');  # 0=classic 1=singlestep/onepage

    #call ViewMultiCheckoutConfirmation if we have the new order process enabled
    if ($OrderProcess) {

        $Servlet->params('ViewAction','ViewMultiCheckoutConfirmation');
        $Servlet->doAction('ViewMultiCheckoutConfirmation') ;

    } else {
    # the old action for handling blocked baskets
        $self->SUPER::View($Servlet);
    }



}



1;
