#!perl -w
package DE_EPAGES::Presentation::t::Form;
use base DE_EPAGES::Database::API::Test;
use strict;
use DE_EPAGES::Core::API::PerlTools qw( IsA );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Presentation::API::Form;
use HTTP::Request;
use Test::More tests => 27;

sub test {

    my $Form = DE_EPAGES::Presentation::API::Form->new;
    $Form->{'Object'} = LoadRootObject();

    # test: float numbers in German using ","
    $Form->setlocale('de_DE');
    my $Value = $Form->formField( '10.000,00', { Name => 'Age', Type => 'reg_integer' });
    cmp_ok( $Value, 'eq', '10000', 'integer de_DE' );

    # test: DateTime parser
    $Value = $Form->formField( '01.02.2009', { Name => 'CreationDate', Type => 'reg_date' });
    ok( IsA($Value, 'DateTime'), 'date de_DE' );
    cmp_ok( $Value->iso8601, 'eq', '2009-02-01T00:00:00', 'date de_DE' );

    # 00:00 datetime
    $Value = $Form->formField( '00:00', { Name => 'CloseAfternoon', Type => 'reg_time' });
    ok( IsA($Value, 'DateTime'), 'time midnight' );
    cmp_ok( $Value->iso8601, 'eq', '2000-01-01T00:00:00', 'time midnight' );

    # 24:00 datetime
    $Value = $Form->formField( '24:00', { Name => 'CloseAfternoon', Type => 'reg_time' });
    ok( IsA($Value, 'DateTime'), 'time midnight 24:00' );
    cmp_ok( $Value->iso8601, 'eq', '2000-01-01T00:00:00', 'time midnight 24:00' );

    # correct integer
    $Value = $Form->formField( '-10.000.000', { 'Name' => 'FieldName', 'Type' => 'reg_integer' });
    ok( !$Form->existsFormError, 'formField reg_integer with correct integer' );
    is( $Value, -10000000, 'formField reg_integer' ); ## no critic qw(ProhibitMagicNumbers)

    # missing integer
    $Value = $Form->formField( '', {
        'Name' => 'FieldName',
        'Type' => 'reg_integer',
        'Mandatory' => 1
    });
    ok( $Form->existsFormError, 'formField reg_integer with missing value' );
    is( $Value, '', 'formField reg_integer with missing value' );
    my $hFormError = $Form->formErrors->[0];
    is( $hFormError->{'Name'}, 'FieldName', 'FormError.Name' );
    is( $hFormError->{'Reason'}, 'STRING_EMPTY', 'FormError.Reason' );
    $Form->resetFormErrors;
    ok( !$Form->existsFormError, 'resetFormErrors' );

    # invalid integer
    $Value = $Form->formField( 'wrong value', { 'Name' => 'FieldName', 'Type' => 'reg_integer' });
    ok( $Form->existsFormError, 'formField reg_integer with missing value' );
    $hFormError = $Form->formErrors->[0];
    is( $hFormError->{'Reason'}, 'FORMAT_NOT_INTEGER', 'FormError.Reason' );
    is( $Value, 'wrong value', 'formField reg_integer with missing value' );
    $Form->resetFormErrors;

    # string with html
    my $HTML = '<p><span>blah</span></p>';
    my $TargetHTML = '<p><span>blah</span></p>';
    $Value = $Form->formField( $HTML, {
        'Name' => 'HtmlField',
        'Type' => 'string',
        'IsHTML' => 1,
        'Mandatory' => 1
    });
    cmp_ok($Value, 'eq', $TargetHTML, 'HTML');

    # string with malformed html
    $HTML = '<p><span>blah</p>';
    $TargetHTML = '<p><span>blah</span></p>';
    $Value = $Form->formField( $HTML, {
        'Name' => 'HtmlField',
        'Type' => 'string',
        'IsHTML' => 1,
        'Mandatory' => 1
    });
    cmp_ok($Value, 'eq', $TargetHTML, 'Corrected HTML');

    # complex string with malformed html and javascript
    $HTML = '<table><tr><td><p><span>blah</p><div><script type="text/javascript">jQuery("<script>")</script></div></table>';
    $TargetHTML = '<table><tr><td><p><span>blah</span></p><div><script type="text/javascript">jQuery("<script>")</script></div></td></tr></table>';
    $Value = $Form->formField( $HTML, {
        'Name' => 'HtmlField',
        'Type' => 'string',
        'IsHTML' => 1,
        'Mandatory' => 1
    });
    cmp_ok($Value, 'eq', $TargetHTML, 'Corrected complex HTML');

    # string validation performance
    my $Iters = 10000;
    my $time1 = time;
    for(my $i=0; $i < $Iters; $i++) {
        $HTML = '<table><tr><td><p class="iter_'.$i.'"><span>blah</p><div><script type="text/javascript">jQuery("<script>")</script></div></table>';
        $TargetHTML = '<table><tr><td><p class="iter_'.$i.'"><span>blah</span></p><div><script type="text/javascript">jQuery("<script>")</script></div></td></tr></table>';
        $Value = $Form->formField( $HTML, {
            'Name' => 'HtmlField',
            'Type' => 'string',
            'IsHTML' => 1,
            'Mandatory' => 1
        });
        #print "$i";
    }
    my $time2 = time;
    my $TimePerIter = ($time2-$time1) / $Iters;
    ok( $TimePerIter < 0.01, 'String validates in less than 0.01 seconds');

    # wolo's test cases
    $HTML = '<p>Toll';
    $TargetHTML = '<p>Toll</p>';
    $Value = $Form->formField( $HTML, {
        'Name' => 'HtmlField',
        'Type' => 'string',
        'IsHTML' => 1,
        'Mandatory' => 1
    });
    cmp_ok( $Value , 'eq', $TargetHTML, 'auto close open tag');


    $HTML = '<x>Toll';
    $TargetHTML = '<x>Toll';
    $Value = $Form->formField( $HTML, {
        'Name' => 'HtmlField',
        'Type' => 'string',
        'IsHTML' => 0,
        'Mandatory' => 1
    });
    cmp_ok( $Value , 'eq', $TargetHTML, 'keep tags if not marked as html');

    $HTML = '<Toll>';
    $TargetHTML = '<Toll>';
    $Value = $Form->formField( $HTML, {
        'Name' => 'HtmlField',
        'Type' => 'string',
        'IsHTML' => 0,
        'Mandatory' => 1
    });
    cmp_ok( $Value , 'eq', $TargetHTML, 'keep tags if not marked as html');

    # html
    $HTML = '<p><span>blah</span></p>';
    $TargetHTML = '<p><span>blah</span></p>';
    $Value = $Form->formField( $HTML, {
        'Name' => 'HtmlField',
        'Type' => 'HTML',
        'Mandatory' => 1
    });
    cmp_ok($Value, 'eq', $TargetHTML, 'HTML');

    # FilePathPart
    my %hFieldData = ( 'Name' => 'FilePathPart' );
    my $FilePathValue = '/Shops/DemoShop/Products/cg_0100504001';
    eval {
        $Value = $Form->field(
            LoadRootObject(),
            $FilePathValue,
            \%hFieldData
        );
    };
    ok($@, 'Correct file path handled correctly');

    $FilePathValue = '../../../../../../../../etc/apache2-epages';
    eval {
        $Value = $Form->field(
            LoadRootObject(),
            $FilePathValue,
            \%hFieldData
        );
    };
    like($@, qr/INVALID_PATH/, 'Incorrect file path handled correctly');
}

__PACKAGE__->runScript;
