#========================================================================================
# package      DE_EPAGES::PriceList::API::CSV::PriceListSimpleExportImport;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements the high level interface for import and export of
#               PriceLists to CSV files.
#               This is a simplified version of the base exporter that uses a fixed pricelist
#               and has only a subset of the columns.
#========================================================================================
package DE_EPAGES::PriceList::API::CSV::PriceListSimpleExportImport;
use base DE_EPAGES::PriceList::API::CSV::PriceListExportImport;

use strict;
use DE_EPAGES::PriceList::API::Constants qw( @CSV_WARNING_CODES_PRICELIST );
use DE_EPAGES::PriceList::API::CSV::PriceListSimpleExportDriver;
use DE_EPAGES::PriceList::API::CSV::PriceListSimpleImportHandler;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );
use DE_EPAGES::Shop::API::Constants qw( TaxModel2Text ) ;

#========================================================================================
# function     exportDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->exportDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub exportDriver {
    my $self = shift;
    return 'DE_EPAGES::PriceList::API::CSV::PriceListSimpleExportDriver';
}

#========================================================================================
# function     importDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->importDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub importDriver {
    my $self = shift;
    return 'DE_EPAGES::PriceList::API::CSV::PriceListSimpleImportHandler';
}

#========================================================================================
# function     csvAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aCSVAttributes = $Driver->csvAttributes($Shop, $LocaleID, $aLanguageCode2s);
#----------------------------------------------------------------------------------------
# description  Return list of exportable csv attributes.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $LocaleID | locale identifier (e.g. en_GB) | string
# input        $aLanguageCode2s | array of languages identified by code2 | ref.array.string
# return       $aAttributes | list of attributes | ref.array.hash
#========================================================================================
sub csvAttributes {
    my $self = shift;
    my ($Shop, $LocaleID, $aLanguageCode2s) = @_;

    my $hLocale = GetInfoLocale($LocaleID);
    my $ahLanguages = $self->getLanguages($Shop, $aLanguageCode2s);

    my $CurrencyID = $Shop->get('CurrencyID'); # EUR,USD

    my $TaxModel = $Shop->get('InputTaxModel') ;
    my $TaxModelText = TaxModel2Text($TaxModel);

    my $ScalePriceClass = LoadClassByAlias('ScalePrice');

    my @PriceListAttributes = map {
        { 'Class' => $ScalePriceClass, 'Attribute' => $ScalePriceClass->attribute($_) }
    } qw (Product Quantity Price);

    my @CSVAttributes;
    foreach my $hAttribute ( @PriceListAttributes ) {
        my $Attribute = $hAttribute->{'Attribute'};
        my $Alias = $Attribute->alias;
        if ($Alias eq 'Price') {
            my $hInfo = $self->getAttributeInfo(
                $Attribute, $hLocale->{'LanguageID'}, $ahLanguages, {
                'Alias'    => $Alias,
                'Class'    => $hAttribute->{'Class'},
                'Position' => 0,
                'Type'     => 'Price'
            })->[0];
            my %AttributeInfo = %$hInfo;
            my $hHeaderStrings = $self->getHeaderStrings( \%AttributeInfo, $hLocale->{'LanguageID'}, {
                'CurrencyID'  => $CurrencyID,
                'TaxModelText'=> $TaxModelText,
            } );
            $AttributeInfo{'HeaderString'} = $hHeaderStrings->{'HeaderString'};
            $AttributeInfo{'AliasString'}  = $hHeaderStrings->{'AliasString'};
            $AttributeInfo{'TaxModel'}     = $TaxModel;
            push @CSVAttributes, \%AttributeInfo;
        } elsif (($Alias eq 'Product') and $Shop->get('IsUnityRegistered') ) {
            my $hAttributeInfo = $self->getAttributeInfo(
                $Attribute, $hLocale->{'LanguageID'}, $ahLanguages, {
                'Alias'    => $Alias,
                'Class'    => $hAttribute->{'Class'},
                'Position' => 0,
            })->[0];
            $hAttributeInfo->{'HeaderString'} =~ s/\[Product\]/\[UnityAlias\]/g;
            push @CSVAttributes, $hAttributeInfo;
        } else {
            push @CSVAttributes, @{
                $self->getAttributeInfo(
                    $Attribute, $hLocale->{'LanguageID'}, $ahLanguages, {
                    'Alias'    => $Alias,
                    'Class'    => $hAttribute->{'Class'},
                    'Position' => 0,
                })
            };
        }
    }
    return \@CSVAttributes;
}

1;
