#========================================================================================
# package      DE_EPAGES::PriceList::API::CSV::PriceListSimpleImportHandler;
# base         DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a handler to import pricelists to products
#               See <package DE_EPAGES::PriceList::API::CSV::PriceListSimpleExportImport> for
#               usage examples.
#========================================================================================
package DE_EPAGES::PriceList::API::CSV::PriceListSimpleImportHandler;
use base qw ( DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler );

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError);
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::CSVExportImport::API::Tools qw( ParseValue );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::PriceList::API::PriceList qw (GetUnityPriceList);
use DE_EPAGES::Shop::API::Constants qw( Text2TaxModel ) ;
use DE_EPAGES::Shop::API::Table::ShopCurrency qw( ExistsShopCurrency );

# indicates if we have an Unity import (identified by the headerline [UnityAlias] entry)
# see also function headerColumn()
our $IsUnityImport = undef;

#========================================================================================
# function     analyzeAttributeLine
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $hAttributeValues = $self->analyzeAttributeLine();
#----------------------------------------------------------------------------------------
# description  Analyzes the current line in the CSV file and fills the object data hash
#               with information for the actual import.
#----------------------------------------------------------------------------------------
# return       $hAttributeValues | properties of a object of CSV file with following keys
#               <li>DirectAttributes  | direct attributes of the object with following keys
#                   <ul>
#                       <li>Alias | alias of the object | string
#                       <li>Class | class object of the object | object
#                       <li>Parent | parent object of the object | object
#                       <li> more direct attributes which should set via $Object->set method,
#                            or can used by $Class->insert($hAttributeValues->{'DirectAttributes'});
#                   </ul> | ref.hash
#               <li>DoNotImport |  0 or 1, set to 1 if there are errors to set the parsed values | boolean
#               <li>Object | object if it is found or undef if it doesn't exist | object
#               <li>LocalizedAttributes |localized attributes hash with language id as key | ref.hash
#               | ref.hash
# example      hAttributeValues='HASH
#                   'DirectAttributes'  =>'HASH'
#                       'Product' =>'DE_EPAGES::Product::API::Object::Product=HASH'
#                       'Quantity'   =>'10'
#                       'Price'      =>'19.95'
#                   'DoNotImport'   =>'0'
#                   'TaxModel'      =>1
#========================================================================================
sub analyzeAttributeLine {
    my $self = shift;
    my $DoNotImport      = 0;
    my $Item              = $self->topID('Item');
    my $ImportAttributes  = $self->topID('ImportAttributes');

    my $Shop = $self->object;
    my $TaxModel = $Shop->get('InputTaxModel');

    # check if product is set and has correct value
    my $ProductPath = $Item->[$ImportAttributes->{'ProductColumn'}];
    $ProductPath =~ s/\s+$//; $ProductPath =~ s/^\s+//;
    $ProductPath = undef if $ProductPath eq '';
    unless (defined $ProductPath) {
        Warning( 'NO_MAND_VALUE', { Line => $self->lineNumber, Attribute => 'Product'} );
        return;
    }
    my $ProductFolder = $Shop->folder('Products');
    my $Product;
    eval { $Product = $self->loadShopObject($ProductFolder, $ProductPath, 'Product'); };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless ($Error->code eq 'UNKNOWN_OBJECT' || $Error->code eq 'FORBIDDEN_OBJECT');
        Warning('PRICELIST_NO_PRODUCT', { Line => $self->lineNumber, Product => $ProductPath } );
        return;
    }
    if ($Product->get('SuperProduct') && !$Product->get('HasSubOwnPrices')) {
        Warning('PRODUCT_HAS_NOT_OWN_PRICES', { Line => $self->lineNumber, Product => $ProductPath } );
        return;
    }
    my %AttributeValues;
    my $hFormat = $self->locale->get([qw(LocaleID DecimalSep ThousandSep)]);

    my %Values;
    $Values{'Quantity'} = defined $ImportAttributes->{'DiscountColumn'} ? $Item->[$ImportAttributes->{'QuantityColumn'}] : undef;
    $Values{'Price'} = defined $ImportAttributes->{'PriceColumn'} ? $Item->[$ImportAttributes->{'PriceColumn'}] : undef;

    # check all attributes
    for (my $i=0; $i<scalar(@$Item); $i++) {
        next if !exists $ImportAttributes->{$i}; # ignore extra columns

        my $hAttribute =  $ImportAttributes->{$i};
        my $Value =  $Item->[$i];

        unless ($self->_analyzeAttributeLine($i, $hAttribute, $Value, $hFormat, \%AttributeValues, \%Values)) {
            $DoNotImport = 1;
            next;
        }
    }
    $AttributeValues{'DirectAttributes'}{'Product'} = $Product;
    $AttributeValues{'TaxModel'} = $TaxModel;
    $AttributeValues{'DoNotImport'} = $DoNotImport;
    return \%AttributeValues;
}

sub priceList {
    my $self = shift;
    return undef unless $IsUnityImport;
    my $Shop = $self->object;
    $self->{'PriceList'} = defined $self->{'PriceList'} ? $self->{'PriceList'} : GetUnityPriceList($Shop, 1 );
    return $self->{'PriceList'}
}

sub _analyzeAttributeLine {
    my $self = shift;
    my ($i, $hAttribute, $Value, $hFormat, $hAttributeValues, $hValues) = @_;

    my $Alias = $hAttribute->{'Alias'};
    eval { ParseValue($Alias,$hAttribute->{'Type'}, \$Value, $hFormat) };
    if ( ExistsError() ) {
        Warning( 'VOID_FIELD', { Line => $self->lineNumber, ID => $i+1, Error => GetError()->shortMessage, Attribute => $Alias } );
        return 0;
    }

    if ($Alias eq 'Quantity') {
        $Value = 1 if defined $Value && $Value == 0;
        $Value = undef if defined $Value && $Value < 0;
        if (!$Value && ($hValues->{'Price'} || $hValues->{'Discount'})) {
            Warning( 'NO_MAND_VALUE', {
                Line => $self->lineNumber,
                Attribute => $Alias
            });
            return 0;
        }
    } elsif ($Alias eq 'Price' || $Alias eq 'Discount') {
        if ($hValues->{'Quantity'} && !$hValues->{'Price'} && !$hValues->{'Discount'}) {
            Warning( 'NO_MAND_VALUE', {
                Line => $self->lineNumber,
                Attribute => $Alias
            });
            return 0;
        }
    } elsif ($Alias eq 'Value') {
        $Value = undef if defined $Value && $Value < 0;
        if (!$Value && $hValues->{'ValueDiscount'}) {
            Warning( 'NO_MAND_VALUE', {
                Line => $self->lineNumber,
                Attribute => $Alias
            });
            return 0;
        }
    } elsif ($Alias eq 'ValueDiscount') {
        if (!$Value && $hValues->{'Value'}) {
            Warning( 'NO_MAND_VALUE', {
                Line => $self->lineNumber,
                Attribute => $Alias
            });
            return 0;
        }
    }

    $hAttributeValues->{'DirectAttributes'}{$Alias} = $Value;
    return 1;
}

sub importObject {
    my $self = shift;

    my $hAttributeValues = $self->analyzeAttributeLine;
    return if (not defined $hAttributeValues or $hAttributeValues->{'DoNotImport'});
    my $TaxModel = $hAttributeValues->{'TaxModel'};
    my $hDirectAttributes = $hAttributeValues->{'DirectAttributes'};
    my $Product = $hDirectAttributes->{'Product'};

    my $PriceList = $self->priceList;
    $self->_setScalePrice($PriceList, $Product, $TaxModel, {
        'Quantity' => $hDirectAttributes->{'Quantity'},
        'Price' => $hDirectAttributes->{'Price'},
    }) if defined $hDirectAttributes->{'Price'};

    return;
}

sub _setScalePrice {
    my $self = shift;
    my ($PriceList, $Product, $TaxModel, $hValues) = @_;

    # convert gross prices to net
    my $TaxMatrix = $Product->getSite()->get('TaxMatrix');
    my $TaxClass = $Product->getWithParent('TaxClass');
    my $CurrencyID = $PriceList->get('CurrencyID');
    $hValues->{'Price'} = $TaxMatrix->gross2net($CurrencyID, $hValues->{'Price'},
        {
            'TaxModel' => $TaxModel,
            'TaxClass' => $TaxClass,
        }
    );

    # update existing scale prices
    my $aScalePrices = $PriceList->scalePrices($Product);
    foreach my $ScalePrice (@$aScalePrices) {
        my $hScalePriceInfo = $ScalePrice->get([ qw( Price Quantity ) ]);
        if ($hValues->{'Quantity'} == $hScalePriceInfo->{'Quantity'}) {
            if ($hValues->{'Price'} != $hScalePriceInfo->{'Price'}) {
                $ScalePrice->set( {'Price' => $hValues->{'Price'} } );
            }
            return;
        }
    }

    # add new levels
    LoadClassByAlias('ScalePrice')->insertObject({
        'PriceList' => $PriceList,
        'Product'   => $Product,
        'Quantity'  => $hValues->{'Quantity'},
        'Price'     => $hValues->{'Price'},
    });
}

#========================================================================================
# function     testMissingFields
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->testMissingFields($hUsedAttributes);
#----------------------------------------------------------------------------------------
# description  test for mandatory columns Class,Parent called by analyzeHeaderLine
#               throws error MISSING_FIELD if they don't exist
#----------------------------------------------------------------------------------------
# input        $hUsedAttributes | used attributes | ref.hash
#========================================================================================
sub testMissingFields {
    my $self = shift;
    my ( $hUsedAttributes ) = @_;
    Error( 'MISSING_FIELD' => { 'Value' => 'Product', 'FileName' => $self->{'FileName'}}) unless exists $hUsedAttributes->{'ProductColumn'};
    return;
}

#========================================================================================
# function     modifyUsedAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->modifyUsedAttributes($hUsedAttributes, $ColumnIndex);
#----------------------------------------------------------------------------------------
# description  add keys:
#                 $hUsedAttributes->{$ColumnIndex}{'IsSuperProductAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'}  eq 'SuperProduct'  ? 1 : 0;
#                 $hUsedAttributes->{'SuperProductAliasColumn'} with value=$ColumnIndex  if $hUsedAttributes->{$ColumnIndex}{'IsSuperProductAliasColumn'};
#----------------------------------------------------------------------------------------
# input        $hUsedAttributes | used attributes | ref.hash
# input        $ColumnIndex | current column | integer
#========================================================================================
sub modifyUsedAttributes {
    my $self = shift;
    my ( $hUsedAttributes, $ColumnIndex) = @_;

    $hUsedAttributes->{'ProductColumn'} = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Product';
    $hUsedAttributes->{'QuantityColumn'} = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Quantity';
    $hUsedAttributes->{'PriceColumn'} = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Price';
    return;
}


#========================================================================================
# function     testHeaderColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->testHeaderColumn($hAttribute, $hHeader, $ColumnIndex);
#----------------------------------------------------------------------------------------
# description  test $hHeader,
#               throws Errors WRONG_FORMAT if the file is not a unity price list
#----------------------------------------------------------------------------------------
# input        $hAttribute | used attribute | ref.hash
# input        $hHeader    | current header info | ref.hash
# input        $ColumnIndex | current column | integer
#========================================================================================
sub testHeaderColumn {
    my $self = shift;
    my ( $hAttribute, $hHeader, $ColumnIndex ) = @_;

    my $HeaderString = $hHeader->{'HeaderString'};
    my $Type = $hAttribute->{'Type'};

    # if we have a invalid price (e.g. net with gross TaxModel or vice versa or an invalid currency)
    # the column is set to IGNORE_COLUMN=1 and Type=String in ShopObjectImportHandler.pm so we have
    # to check the HeaderString and parse it (again) manually
    if ( $Type eq 'String' and $HeaderString =~ /Price/) {
        my $aHeaderParts = $self->headerColumn($HeaderString)->{HeaderParts};
        my $CurrencyID = $aHeaderParts->[1];
        my $TaxModel;
        eval {$TaxModel = Text2TaxModel($aHeaderParts->[2]);};
        if (ExistsError() or $TaxModel eq '') {
            Error('NO_MONEY_FORMAT', { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} });
        }
        if (not ExistsShopCurrency($self->object->id, $CurrencyID)) {
            Error('NO_SHOP_CURRENCY', { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} });
        }
        if ($TaxModel != $self->object->get('InputTaxModel')) {
            Error('NO_SHOP_TAXMODEL', { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} });
        }
    }
    unless ( $IsUnityImport ) {
        Error( 'WRONG_FORMAT', { 'ID' => 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} } );
    }
    return;
}

#========================================================================================
# function     resetFlagsAndWarnings
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->resetFlagsAndWarnings();
#----------------------------------------------------------------------------------------
# description  reset the gloabl warnings counter.
#========================================================================================
sub resetFlagsAndWarnings {
    $IsUnityImport = undef;
}

sub headerColumn {
    my $self = shift;
    my ($HeaderString, $Index) = @_;

    # test if we have a field [UnityAlias] in the headerline. If so we import unity export files.
    # If we have an unity import file, set global variable and replace [UnityAlias] by [ALias]]
    $IsUnityImport = 1 if $HeaderString =~ /\[UnityAlias\]/;

    # replace obsolete attribute aliases
    $HeaderString =~ s/\[UnityAlias\]/\[Product\]/g;

    return $self->SUPER::headerColumn( $HeaderString, $Index );
}

1;

