use strict;
use Getopt::Long;
use Time::HiRes qw( time );
use DE_EPAGES::Core::API::Script  qw( RunScript );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::ShopCSVExportImport::API::CSV::ShopExportImport qw ( ExportCSV );

sub Export {
    my ($ShopAlias, $FileName, $hOptions) = @_;
    TestString( 'ShopAlias', $ShopAlias );
    TestString( 'FileName', $FileName );

    my $Shop = LoadRootObject()->child('Shops')->child($ShopAlias);
    my $aWarnings = ExportCSV($Shop, $FileName, 'DE_EPAGES::PriceList::API::CSV::PriceListSimpleExportImport', $hOptions);

    print STDERR $_->shortMessage . "\n" foreach @$aWarnings;
}

sub Main {

    local $| = 1;

    my $ShopAlias;
    my $Password;
    my $StoreName;
    my $Encoding;
    my $FileName;
    my $DelimiterName = 'Comma';
    my $Help;

    GetOptions(
        'help'          => \$Help,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'shop=s'        => \$ShopAlias,
        'delimiter=s'   => \$DelimiterName,
        'encoding=s'    => \$Encoding,
        'file=s'        => \$FileName,
    );
    usage() if $Help;
    unless( $StoreName ) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }
    unless( $ShopAlias ) {
        print STDERR "Missing parameter -shop\n\n";
        usage();
    }
    unless( $FileName ) {
        print STDERR "Missing parameter -file\n\n";
        usage();
    }

    my %DELIMITERS = (
        'Comma' => ',',
        'Semicolon' => ';',
        'Tabulator' => "\t",
        'Pipe' => '|',
    );
    die "Invalid delimiter '$DelimiterName'" unless exists $DELIMITERS{$DelimiterName};

    my %Options = ( 'sep_char' => $DELIMITERS{$DelimiterName} );
    $Options{'encoding'} = $Encoding if defined $Encoding;

    my $time1 = time;

    RunOnStore(
        Store       => $StoreName,
        DBPassword  => $Password,
        Sub         => sub  {
            Export( $ShopAlias, $FileName, \%Options );
        },
    );
    printf "need %.3f seconds\n", time()-$time1;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] flags objects

options:
    -passwd     database user password                           ()
    -shop       shop alias                                       required
    -file       output file name                                 required
    -storename  name of store                                    required
    -encoding   character encoding                               (encoding of shop locale)
    -delimiter  delimiter name (Comma|Semicolon|Tabulator|Pipe)  (Comma)
flags:
    -help       show the command line options

Examples:
    perl $0 -storename Store -shop DemoShop -file Bulkprices.csv
    perl $0 -storename Store -shop DemoShop -file Bulkprices.csv -delimiter Semicolon  -encoding utf-8
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

1;
