#==============================================================================
# package      DE_EPAGES::PriceList::UI::Product
# state        private
#==============================================================================
package DE_EPAGES::PriceList::UI::Product;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Object::API::Factory qw( Transaction );
use DateTime;
use DE_EPAGES::PriceList::API::PriceList qw ( GetUnityPriceList );

#==============================================================================
# Event handlers
#==============================================================================
sub ViewPrices {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product   = $Servlet->object;
    my $Shop      = $Product->getSite();

    # display only pricelists where the currency is enabled in the shop
    my %ShopCurrencies = map { $_->{'CurrencyID'} => undef } @{$Shop->get('Currencies')};
    my @DisplayPriceLists = grep { exists $ShopCurrencies{$_->get('CurrencyID')} }
                            @{$Shop->get('PriceLists')};
    $Servlet->vars('DisplayPriceLists', [ map { $_->tleHash } @DisplayPriceLists ]);

    # get selected pricelists or the simplified unity pricelist if available
    # if we don't find a unity pricelist or have an ep6 shop we fallback to SelectedPriceList from the form
    my @PriceListIDs = ();
    if ($Shop->get('IsUnityRegistered')) {
        my $UnityPriceList = GetUnityPriceList($Shop);
        if (defined $UnityPriceList) {
            push @PriceListIDs, $UnityPriceList->id;
        }
    } else {
        @PriceListIDs = grep {defined $_ and $_ ne ''} $Servlet->form->value('SelectedPriceList');
    }

    my @PriceLists;
    if( scalar @PriceListIDs ) {
        foreach my $PriceListID (@PriceListIDs ) {
            my $PriceList = $Shop->loadObject($PriceListID, 'PriceList');
            $Servlet->testPermission($PriceList, 'MBO-ViewGeneral');
            push @PriceLists, $PriceList;
        }
    }
    elsif( scalar @DisplayPriceLists >= 2 ) {
        # automatically select the first 2 pricelists
        @PriceLists = @DisplayPriceLists[0, 1];
    }
    elsif( scalar @DisplayPriceLists == 1 ) {
        # automatically select the first pricelist
        @PriceLists = @DisplayPriceLists;
    }
    else {
        # there are no pricelists in the currencies of the shop
        return;
    }

    #set params for selectboxes
    my @SelectedPriceLists;
    foreach my $PriceList ( @PriceLists ) {
        my $PriceListID = $PriceList->id;
        my $Info = $PriceList->tleHash;
        $Info->{'SelectedPriceListID'} = $PriceListID;
        push @SelectedPriceLists,$Info;
    }
    $Servlet->params('SelectedPriceLists',\@SelectedPriceLists );

    # collect scale prices, quantity discounts, value discounts
    my %Prices;
    my $InputTaxModel = $Product->getSite()->get('InputTaxModel');
    foreach my $PriceList (@PriceLists) {
        my $ahScalePrices =  $PriceList->getScalePrices( $Product, $InputTaxModel, 1 );
        $Prices{$_->{'Quantity'}*1}{$PriceList->id}{'Price'} = $_->{'Price'} foreach @$ahScalePrices;
        my $ahQuantityDiscounts =  $PriceList->getQuantityDiscounts( $Product, $InputTaxModel, 1 );
        $Prices{$_->{'Quantity'}*1}{$PriceList->id}{'Discount'} = $_->{'Discount'} foreach @$ahQuantityDiscounts;
    }

    #scale prices , quantity discounts
    my $Counter = 0;
    my @PriceInfoQuantities = map {
        my $Quantity = $_;
        {
            'Quantity' => $Quantity,
            'PriceInfos' => [
                map {
                    my $PriceList = $_;
                    my $hPrice = $Prices{$Quantity*1}{$PriceList->id} || {};
                    {
                        'PriceListID' => $PriceList->id,
                        'CurrencyID' => $PriceList->get('CurrencyID'),
                        'ErrorItemNo' => $Counter++,
                        'Quantity' => $Quantity,
                        'Price' => $hPrice->{'Price'},
                        'Discount' => $hPrice->{'Discount'},
                    }
                } @PriceLists
            ]
        };
    } sort { $a <=> $b } keys %Prices;
    $Servlet->params('PriceInfoQuantities',\@PriceInfoQuantities );

    #value discounts for the first pricelist
    my $ahValueDiscount = $PriceLists[0]->getValueDiscounts( $Product, $InputTaxModel, 1 );
    $Servlet->params('ValueDiscounts', $ahValueDiscount);

    return;
}

sub SaveQuantityPrices {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();

    # collect price lists, check permissions, create unity pricelist/customergroup if necessary
    my $hFormValues;
    if ($Shop->get('IsUnityRegistered')) {
        $hFormValues = $Servlet->form->form($Product, 'UnitySaveQuantityPrices', 1);
        my $PriceList = GetUnityPriceList($Shop, 1);
        $_->{'PriceListID'} = $PriceList->id for @{$hFormValues->{'QuantityPrices'}};
        $_->{'NewPriceListID'} = $PriceList->id for @{$hFormValues->{'NewQuantityPrices'}};
    } else {
        $hFormValues = $Servlet->form->form($Product, 'SaveQuantityPrices');
    }

    my %PriceLists;
    $PriceLists{$_} = $Shop->loadObject($_, 'PriceList') foreach map { $_->{'PriceListID'} } @{$hFormValues->{'QuantityPrices'}};
    $PriceLists{$_} = $Shop->loadObject($_, 'PriceList') foreach map { $_->{'NewPriceListID'} } @{$hFormValues->{'NewQuantityPrices'}};
    $Servlet->testPermission($_, 'MBO-ViewGeneral') foreach values %PriceLists;

    # initialize hashes with empty lists for each pricelist
    # ( $PriceListID => [ {}, {}, ... ], ... )
    my %ScalePrices = map { $_ => [] } keys %PriceLists;
    my %QuantityDiscounts = map { $_ => [] } keys %PriceLists;

    # get existing prices and discounts
    foreach my $hEntry ( @{$hFormValues->{'QuantityPrices'}} ) {
        next unless defined $hEntry->{'Quantity'};
        if( defined $hEntry->{'Price'} ) {
            push @{$ScalePrices{$hEntry->{'PriceListID'}}}, {
                'Quantity' => $hEntry->{'Quantity'},
                'Price' => $hEntry->{'Price'},
            };
        }
        if( defined $hEntry->{'Discount'} ) {
            push @{$QuantityDiscounts{$hEntry->{'PriceListID'}}}, {
                'Quantity' => $hEntry->{'Quantity'},
                'Discount' => $hEntry->{'Discount'},
            };
        }
    }

    # verify new scales prices or quantity discounts
    $self->_validateQuantities($Servlet, $hFormValues, \%ScalePrices, \%QuantityDiscounts);

    #add new scale prices
    my $Quantity = exists $hFormValues->{'NewQuantity'} ? $hFormValues->{'NewQuantity'} : undef;
    foreach my $hEntry ( @{$hFormValues->{'NewQuantityPrices'}} ) {
        $Quantity = exists $hEntry->{'NewQuantity'} ? $hEntry->{'NewQuantity'} : $Quantity;
        if (defined $Quantity) {
            my $PriceListID = $hEntry->{'NewPriceListID'};
            if (defined $hEntry->{'NewPrice'}) {
                push @{$ScalePrices{$PriceListID}}, {
                    'Quantity' => $Quantity,
                    'Price'    => $hEntry->{'NewPrice'},
                };
            }
            if (defined $hEntry->{'NewDiscount'}) {
                push @{$QuantityDiscounts{$PriceListID}}, {
                    'Quantity' => $Quantity,
                    'Discount' => $hEntry->{'NewDiscount'},
                };
            }
        }
    }

    # update scale prices and quantity discounts
    my $InputTaxModel = $Product->getSite()->get('InputTaxModel');
    foreach my $PriceListID (keys %PriceLists) {
        my $PriceList = $PriceLists{$PriceListID};

        my $ahScalePrices = $ScalePrices{$PriceListID};
        $PriceList->setScalePrices($Product, $InputTaxModel, $ahScalePrices);

        my $ahQuantityDiscounts = $QuantityDiscounts{$PriceListID};
        $PriceList->setQuantityDiscounts($Product, $InputTaxModel, $ahQuantityDiscounts);
    }
    return;
}

# note that for ep6 $QuantityCount{$PriceListID}{*} will normally never exceed a count of 1
# this is just used for unity, where multiple entries for a single pricelists are possible
sub _validateQuantities {
    my $self = shift;
    my ($Servlet, $hFormValues, $ScalePrices, $QuantityDiscounts) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();
    my $Index = 0;
    my %QuantityCount = ( 'ScalePrice' => {}, 'QuantityDiscount' => {});
    my $Quantity = exists $hFormValues->{'NewQuantity'} ? $hFormValues->{'NewQuantity'} : undef;
    foreach my $hEntry ( @{$hFormValues->{'NewQuantityPrices'}} ) {
        $Quantity = exists $hEntry->{'NewQuantity'} ? $hEntry->{'NewQuantity'} : $Quantity;
        if (defined $Quantity) {
            $Quantity = $Product->correctQuantity($Quantity);
            my $PriceListID = $hEntry->{'NewPriceListID'};
            $QuantityCount{'ScalePrice'}{$Quantity}++;
            $QuantityCount{'QuantityDiscount'}{$Quantity}++;
            # test for existing interval
            if ((scalar(grep {$_->{'Quantity'} == $Quantity} @{$ScalePrices->{$PriceListID}})
                 + scalar(grep {$_->{'Quantity'} == $Quantity} @{$QuantityDiscounts->{$PriceListID}})
                 > 0) || $QuantityCount{$PriceListID}{'ScalePrice'}{$Quantity}>1 ||
                $QuantityCount{$PriceListID}{'QuantityDiscount'}{$Quantity}>1 )
            {
                my $hErrorValues = {
                    'Reason'   => 'INTERVAL_EXISTS',
                    'Name'     => 'NewQuantity',
                    'Form'     => 'SaveQuantityPrices',
                };
                if ($Shop->get('IsUnityRegistered')) {
                    @$hErrorValues{ qw(Form LoopName Index) } = ('UnitySaveQuantityPrices', 'NewQuantityPrices', $Index);
                    $hEntry->{'NewQuantity'} = $Quantity;
                } else {
                    $hFormValues->{'NewQuantity'} = $Quantity;
                }
                $Servlet->form->addFormError($hErrorValues);
            }
            $Index++;
        }
    }
    $Servlet->form->executeFormError();
}

sub DeleteQuantityPrices {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();

    my $hFormValues  = $Servlet->form->form($Product, 'DeleteQuantityPrices');
    my @DeleteQuantities =  map { $_->{'DeleteQuantity'} } @{$hFormValues->{'QuantityPrices'}};
    return unless scalar @DeleteQuantities;

    # get price lists and check permissions
    my @PriceLists = map { $Shop->loadObject($_, 'PriceList') }
        grep { defined $_ && $_ ne '' }
        map { $_->{'SelectedPriceList'} }
        @{$hFormValues->{'PriceLists'}};
    $Servlet->testPermission($_, 'MBO-ViewGeneral') foreach @PriceLists;
    return unless scalar @PriceLists;


    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        # set delete mark to template

        # collect scale prices, quantity discounts, value discounts
        my %Prices;
        my $InputTaxModel = $Product->getSite()->get('InputTaxModel');
        foreach my $PriceList (@PriceLists) {
            my $ahScalePrices =  $PriceList->getScalePrices( $Product, $InputTaxModel, 1 );
            $Prices{$_->{'Quantity'}*1}{$PriceList->id}{'Price'} = $_->{'Price'} foreach @$ahScalePrices;
            my $ahQuantityDiscounts =  $PriceList->getQuantityDiscounts( $Product, $InputTaxModel, 1 );
            $Prices{$_->{'Quantity'}*1}{$PriceList->id}{'Discount'} = $_->{'Discount'} foreach @$ahQuantityDiscounts;
        }

        my @DeleteObjects = map {
            my $Quantity = $_;
            {
                'Quantity' => $Quantity,
                'PriceInfos' => [
                    map {
                        my $PriceList = $_;
                        my $hPrice = $Prices{$Quantity*1}{$PriceList->id} || {};
                        {
                            'CurrencyID' => $PriceList->get('CurrencyID'),
                            'Price' => $hPrice->{'Price'},
                            'Discount' => $hPrice->{'Discount'},
                        }
                    } @PriceLists
                ]
            };
        } @DeleteQuantities;

        $Servlet->vars('DeleteQuantities',\@DeleteObjects);
        $Servlet->vars('DELETE',1);
        return;
    }
    else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }

    my %DeleteQuantities = map { $_*1 => 1 } @DeleteQuantities;
    foreach my $PriceList (@PriceLists) {
        $_->delete foreach
            grep { exists $DeleteQuantities{$_->get('Quantity')*1} }
            @{ $PriceList->scalePrices( $Product ) };

        $_->delete foreach
            grep { exists $DeleteQuantities{$_->get('Quantity')*1} }
            @{ $PriceList->quantityDiscounts( $Product ) };
    }

    return;
}

sub SaveValueDiscounts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();

    # get value discounts
    my $hFormValues = $Servlet->form->form($Product, 'SaveValueDiscounts');
    my $PriceListID = $hFormValues->{'PriceListID'};
    return unless defined $PriceListID;

    my $PriceList = $Shop->loadObject($PriceListID, 'PriceList');
    $Servlet->testPermission($PriceList, 'MBO-ViewGeneral');

    my @ValueDiscounts = grep { defined $_->{Value} && defined $_->{Discount} }
        @{$hFormValues->{'ValueDiscounts'}};

    # new value discounts, check for duplicates
    my $NewDiscount = $hFormValues->{'NewDiscount'};
    my $NewValue = $hFormValues->{'NewValue'};
    if (defined $NewValue && defined $NewDiscount) {
        #test for existing interval
        if( scalar grep { $_->{'Value'} == $NewValue } @ValueDiscounts ) {
            $Servlet->form->executeFormError({
                'Reason'=> 'INTERVAL_EXISTS',
                'Name'  => 'NewValue',
                'Form'  => 'NewValueDiscount',
            });
        }

        # add new value to existing
        push @ValueDiscounts, {
            'Value'       => $NewValue,
            'Discount'    => $NewDiscount,
        };
    }

    # save value discounts
    my $InputTaxModel = $Shop->get('InputTaxModel');
    $PriceList->setValueDiscounts($Product, $InputTaxModel, \@ValueDiscounts);

    return;
}

sub DeleteValueDiscounts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();

    my $hFormValues  = $Servlet->form->form($Product, 'DeleteValueDiscounts');
    my $PriceListID = $hFormValues->{'PriceListID'};
    return unless defined $PriceListID;

    my $PriceList = $Shop->loadObject($PriceListID, 'PriceList');
    $Servlet->testPermission($PriceList, 'MBO-ViewGeneral');

    my $InputTaxModel = $Shop->get('InputTaxModel');
    my $ahValueDiscounts = $PriceList->getValueDiscounts( $Product, $InputTaxModel, 1 );
    my %DeleteValues = map { $_->{'DeleteValue'}*1 => 1 } @{$hFormValues->{'ValueDiscounts'}};

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        my @DeleteObjects = grep { exists $DeleteValues{$_->{'Value'}*1} } @$ahValueDiscounts;
        $Servlet->vars('DeleteObjects', \@DeleteObjects);
        $Servlet->vars('DELETE',1);
    }
    else {
        # find the remaining value discounts
        my @ValueDiscounts = grep { !exists $DeleteValues{$_->{'Value'}*1} } @$ahValueDiscounts;
        Transaction( sub {
            $PriceList->setValueDiscounts( $Product, $InputTaxModel, \@ValueDiscounts )
        } );

        $Servlet->vars('DELETE',0);
    }

    return;
}

1;
