#==============================================================================
# package      DE_EPAGES::Product::UI::Product
# state        private
#==============================================================================
package DE_EPAGES::Product::UI::Product;
use base DE_EPAGES::Customer::UI::Shop;

use strict;
use DE_EPAGES::Address::API::Address qw( CheckZipcode );
use DE_EPAGES::Product::UI::ContentObject;
use DE_EPAGES::Core::API::Constants qw( IMAGE_FILE_TYPES );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError Error );
## no critic qw(RequireCorrectImports)
use DE_EPAGES::Core::API::File qw (
    fsplit
    CopyFile
    DeleteFile
    ExistsDirectory
    DeleteDirectory
    ExistsFile
    RelativePath
    RunOnDirectory
    WriteFile
    MoveFile
    CleanPath
    IsImageFile
    IsPDFFile
);
use DE_EPAGES::Core::API::Image qw( GetImageInfo TestImage TestImageFormat );
use DE_EPAGES::Core::API::Log qw (GetLog LogDebug);
use DE_EPAGES::Core::API::PerlTools qw( ElementOf unique GetCurrentDateTime fcmp AddDuration GetLocalTimeZone GenerateGUID);
use DE_EPAGES::Core::API::TestParameter qw( TestEMailAddress TestString );
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Filename );
use DE_EPAGES::Core::API::Warning qw( BlockWarnings Warning );
use DE_EPAGES::Core::API::Url qw( StringIsURL );
use DE_EPAGES::Customer::API::Shop qw( GetCustomerForUser );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw (ExistsObject ExistsObjectByGUID GenerateSplittedGUIDPath
                                        LoadClassByAlias LoadRootObject LoadObjectByGUID
                                        Transaction);
use DE_EPAGES::Object::API::Constants qw( POSITION_STEP );
use DE_EPAGES::Order::API::LineItemContainer qw( SavePayment );
use DE_EPAGES::Object::API::Table::Currency qw ( GetInfoCurrency );
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::Presentation::API::PageCache qw( UpdatePageCache );
use DE_EPAGES::Presentation::API::Upload qw( Upload );
use DE_EPAGES::Product::API::Basket qw( AddProducts );
use DE_EPAGES::Product::API::Constants qw (
    PRODUCT_IMAGES_SIZES
    PRODUCT_IMAGE_SIZE_NAMES
    PRODUCT_IMAGE_MAX_COUNT
    MBOUI_PRODUCT_DETAIL_PAGE
);
use DE_EPAGES::Product::API::Object::PagerSubProducts;
use DE_EPAGES::Product::API::Product qw (
    InsertMultipleSubProducts
    GetSubProductByAttributeValues
    ScaleImageSizes
    UsedVariationValues
    DeleteProductImage
    RenameProductImageFile
    RenameSubProductImageFile
    StoreProductImages
    DeleteProductImageByAttribute
    GetScaledImages
);
use DE_EPAGES::Product::API::Table::ProductAvailabilityEmail qw( ExistsProductAvailabilityEmail
                                                                 InsertProductAvailabilityEmail );
use DE_EPAGES::Product::DAL::Product qw( DAL_GetSubProductIDsBySuperProductID);
use DE_EPAGES::Product::UI::ProductType;
use DE_EPAGES::Product::API::Table::SelectedVariation qw( GetInfoSelectedVariationsByProductID);
use DE_EPAGES::SEO::API::SEO qw ( PerformSEOCheckAndUpdateObject );
use DE_EPAGES::Shop::API::Constants qw(BASKETACTION_ADDNEWPOSITION);
use DE_EPAGES::Shop::API::Shop qw (GetNextNo);
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );
use DE_EPAGES::Object::API::Object::Site;
use JSON qw ( from_json to_json );

use File::Basename;
use File::Find;
use List::Util qw (all);
use MIME::Base64;
use Readonly;
use Scalar::Util qw ( blessed );

use constant PREFIX_USERDEFINED_ATTRIBUTES => 'UserDefined_';
use constant FIRST_IMAGE_ORDER_NUMBER => 10;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'OBJECT_NOT_FOUND'   => 'Object #ObjectID not found',
        'CUSTOMIZED_FILE_EXISTS' => 'Object LineItemCustomizedFile with GUID: #GUID already exists',
    }
}
#ERRORDEFINITION END

#==============================================================================
# Event handlers
#==============================================================================
sub ViewPagedChildren {
    my $self = shift;
    my ($Servlet) = @_;

    # set TLE #AtLeastOneVariationHasAnOwnPrice
    my $aSubProducts = $Servlet->object->subProducts;
    if( grep { $_->get('HasSubOwnPrices') } @$aSubProducts ) {
        $Servlet->vars('AtLeastOneVariationHasAnOwnPrice', 1);
    }

    $Servlet->vars('CanAddVariations', 1);
    my $SubProductCount = scalar(@$aSubProducts);
    my $Shop = $Servlet->object->getSite();

    if ($SubProductCount >= $Shop->featureMaxValue('ProductMaxSubProducts')) {
        $Servlet->vars('CanAddVariations',0);
    } else {
        my $count = 1;
        # get the count of variation attributes from product
        my $aUserDefinedAttributes = $Servlet->object->get('Class')->get('UserDefinedAttributes');
        foreach my $UserDefinedAttribute (@$aUserDefinedAttributes) {
            next unless $UserDefinedAttribute->get('Type') eq 'PreDefLocalizedString';
            my $aValues = $UserDefinedAttribute->get('PreDefAttributes');
            $count = $count * scalar(@$aValues);
        }

        $Servlet->vars('CanAddVariations',0) if $SubProductCount >= $count;
    }

    my $Pager = DE_EPAGES::Product::API::Object::PagerSubProducts->new({
        'Servlet' => $Servlet,
        'Object' => $Servlet->object,
        'OrderBy' => 'Alias',
        'CurrencyID' => $Servlet->object->get('Site')->get('CurrencyID'),
    });

    if ($Shop->isUnity()) {
        #load path and order of images
        my ($ParsedExist, $ParsedOrder) = $self->_getExistingFilesAndOrder($Servlet);

        $Servlet->vars('Exist', $ParsedExist);
        $Servlet->vars('Order', $ParsedOrder);


        #load existing slightshows for subproducts get image position
        my %SlideShows;
        foreach my $SingleSubProduct (@$aSubProducts) {
            my @SlideShowOrder;
            my $hSlideShowValues = $SingleSubProduct->get(['ImagesSlideShowString', 'ImageLarge', 'ImagesSlideShow']);

            if (defined $hSlideShowValues->{'ImagesSlideShowString'} and defined $hSlideShowValues->{'ImageLarge'} and
                $hSlideShowValues->{'ImagesSlideShowString'} !~ m/$hSlideShowValues->{'ImageLarge'}/) {
                push(@SlideShowOrder, $hSlideShowValues->{'ImageLarge'});
            }

            foreach my $hSlideShowImage (@{$hSlideShowValues->{'ImagesSlideShow'}}) {
                push(@SlideShowOrder, $hSlideShowImage->{'ImageName'});
            }
            $SlideShows{ $SingleSubProduct->id } = [@SlideShowOrder];
        }

        # escape quotes
        my $ParsedSlideShows = to_json(\%SlideShows);
        $ParsedSlideShows =~ s/\"/\\\"/g;
        $Servlet->vars('SlideShowOrder', $ParsedSlideShows);
    }

    return $Pager->page;
}

sub _getExistingFilesAndOrder {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $WebPath = $Product->get('WebPath');

    # filter for image files, excluding EnergyLabel and ProductDataSheet files
    my $ImageFiles = $self->_filterImageFiles($Product);

    # grep existing files from storage (without scaled images)
    my $aOriginalImages = $self->_findOriginalImages($ImageFiles);

    my @ahExistingFiles;
    my @aFileOrder;

    # the order number of the 1st image
    my $idx = FIRST_IMAGE_ORDER_NUMBER;
    foreach my $hImageFile (sort { $a->{'LastChange'} <=> $b->{'LastChange'}} @$aOriginalImages) {
        push @aFileOrder, $idx;
        push @ahExistingFiles, { 'm_src'    => $WebPath . '/' . $hImageFile->{'FileName'},
                                 'src'      => $WebPath . '/' . $hImageFile->{'FileName'},
                                 'id'       => $idx,
                                 'file'     => $hImageFile->{'FileName'},
                                 'mime'     => $hImageFile->{'Mimetype'},
                               };
        $idx += FIRST_IMAGE_ORDER_NUMBER;
    }

    my $ParsedExist = to_json(\@ahExistingFiles);
    my $ParsedOrder = to_json(\@aFileOrder);
    # JSON string is stored in another JSON file, thus escape quotes
    $ParsedExist =~ s/\"/\\\"/g;
    $ParsedOrder =~ s/\"/\\\"/g;

    return ($ParsedExist, $ParsedOrder);

}

# filter for image files, excluding EnergyLabel and ProductDataSheet files
sub _filterImageFiles {
    my $self = shift;
    my ($Product) = @_;

    # filter out non image files form the product's public files
    my %InNonImageFileNames = map { $_ => 1 } @{ $Product->collectAllNonImageFiles() };
    my @ImageFiles = grep { !$InNonImageFileNames{$_->{'FileName'}} } @{ $Product->get('PublicFiles') };
    return \@ImageFiles;
}

sub _findOriginalImages {
    my $self = shift;
    my ($ahFiles) = @_;

    my $hhFiles;
    # exclude 'l' size as this is not added to the (original) file name
    my @Sizes = grep { $_ ne 'l' } keys %{PRODUCT_IMAGES_SIZES()};
    my @hParentFiles = ();

    foreach my $hFile (@$ahFiles) {
        my (undef, $FileName, undef) = fsplit($hFile->{'FileName'});
        $hhFiles->{$FileName} = $hFile;
    }
    my @FileNames = keys %$hhFiles;

    # find file names that have sub images in all other scales
    foreach my $FileName (@FileNames) {
        push @hParentFiles, $hhFiles->{$FileName} if all { ElementOf($FileName . '_' . $_, @FileNames) } @Sizes;
    }

    return \@hParentFiles;
}

sub attributeValues {
    my $self = shift;
    my ( $Servlet, $Object, $Class, $Prefix ) = @_;
    TestString('Prefix', $Prefix) if defined $Prefix;

    # EPG-13480: look if file attributes are supplied as strings instead of files (reason: we cannot upload the files when saving an ajax request,
    # therefore we have to upload them beforehand and assign the filename to the attribute
    if($Servlet->params('HandleFilesAsStrings')) {
        my $Form = $Servlet->form;
        my @LanguageIDs = $Form->value('LanguageID');
        @LanguageIDs = ( $Servlet->languageID ) unless scalar @LanguageIDs;
        # for all class attributes
        foreach my $Attribute (@{$Class->attributes}) {
            my $Type = $Attribute->get('Type');
            # if type is a file
            if($Type eq 'File' || $Type eq 'LocalizedFile') {
                my $ElementName = defined $Prefix ? $Prefix.$Attribute->alias : $Attribute->alias;
                my @OrgValues = $Form->value( $ElementName );
                # if there're values for that attribute in the form data
                if(@OrgValues && scalar(@OrgValues) > 0) {
                    # set value for all supplied languages
                    foreach ( my $Index=0; $Index<scalar @OrgValues; $Index++ ) {
                        my $LanguageID = $LanguageIDs[$Index];
                        $Object->set( { $ElementName => $OrgValues[$Index] }, $LanguageID);
                    }
                }
            }
        }
    }
    my $hhValues;
    # EPG-20164: user defined attributes have own prefix ('UserDefined_') to avoid errors.
    if(defined $Prefix) {
        $hhValues = $self->SUPER::attributeValues( $Servlet, $Object, $Class, $Prefix );
    } else {
        $hhValues = $self->SUPER::attributeValues( $Servlet, $Object, $Class);
    }

    # test some mandatory values if we have a master product (EPG-3022)
    if ($Object->class->existsAttribute( 'SuperProduct' ) and (not $Object->get('SuperProduct'))) {
        foreach my $AttributeName (
            grep { exists $hhValues->{'Values'}{$_} and not defined $hhValues->{'Values'}{$_} }
            qw (PriceQuantity MinOrder IntervalOrder)) {
                $Servlet->form->addFormError({
                    Reason => 'INVALID_FORM',
                    Name   => $AttributeName,
                    MinValue => 0,
                });
        }
        if( !$Object->get('IsBundleProduct') && exists $hhValues->{'Values'}{OrderUnit} && !defined $hhValues->{'Values'}{OrderUnit}) {
            $Servlet->form->addFormError({
                Reason => 'INVALID_FORM',
                Name   => 'OrderUnit',
                MinValue => 0,
            });
        }
        foreach my $AttributeName (
            grep { defined $hhValues->{'Values'}{$_} and not $hhValues->{'Values'}{$_}}
            qw (RefAmount PriceQuantity)) {

            $Servlet->form->addFormError({
                Reason => 'FLOAT_TOO_SMALL',
                Name   => $AttributeName,
                MinValue => 0,
            });
        }

        # test if we have a Unit for RefUnit
        foreach my $AttributeName (
            grep { defined $hhValues->{'Values'}{$_}}
            qw (RefAmount )) {

            $Servlet->form->addFormError({
                Reason => 'FLOAT_TOO_SMALL',
                Name   => 'RefUnit',
                MinValue => 0,
            }) unless defined $hhValues->{'Values'}{'RefUnit'};
        }

        foreach my $AttributeName (
            grep { defined $hhValues->{'Values'}{$_}}
            qw (Weight)) {

            $Servlet->form->addFormError({
                Reason => 'FLOAT_TOO_SMALL',
                Name   => 'WeightUnit',
                MinValue => 0,
            }) unless defined $hhValues->{'Values'}{'WeightUnit'};
        }


    }else {
        # is sub product
        foreach my $AttributeName (
            grep { defined $hhValues->{'Values'}{$_} and not $hhValues->{'Values'}{$_}}
            qw (RefAmount PriceQuantity)) {

            $Servlet->form->addFormError({
                Reason => 'FLOAT_TOO_SMALL',
                Name   => $AttributeName,
                MinValue => 0,
            });
        }

        # test if we have a Unit for RefUnit
        foreach my $AttributeName (
            grep { defined $hhValues->{'Values'}{$_}}
            qw (RefAmount )) {

            $Servlet->form->addFormError({
                Reason => 'FLOAT_TOO_SMALL',
                Name   => 'RefUnit',
                MinValue => 0,
            }) unless defined $hhValues->{'Values'}{'RefUnit'};
        }

        foreach my $AttributeName (
            grep { defined $hhValues->{'Values'}{$_}}
            qw (Weight)) {

            $Servlet->form->addFormError({
                Reason => 'FLOAT_TOO_SMALL',
                Name   => 'WeightUnit',
                MinValue => 0,
            }) unless defined $hhValues->{'Values'}{'WeightUnit'};
        }


    }

    return $hhValues;
}

sub SaveProductType {
    my $self = shift;
    my ($Servlet) = @_;

    my $ProductClassID = $Servlet->form->value('Class');
    return unless defined $ProductClassID;

    my $Product = $Servlet->object;

    my $ProductClassObject = $Product->getSite()->loadObject($ProductClassID, 'ProductType');
    return unless defined $ProductClassObject;

    $Product->set( { 'Class' => $ProductClassObject} );
    return;
}

sub SaveAvailabilityNotification {
    my $self = shift;
    my ($Servlet) = @_;

    my $EMail = $Servlet->form->value('EMail');

    $self->GoogleCaptcha($Servlet);

    eval {
        TestEMailAddress('EMail',$EMail);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Servlet->form->executeFormError({
            'Reason'=> $Error->code,
            'Name'  => 'SaveAvailabilityNotification',
        });
    }

    my $Product = $Servlet->object;

    my %Params = (
            'ProductID'     => $Product->id,
            'EMail'         => $EMail,
            'LocaleID'      => $Servlet->locale,
    );

    InsertProductAvailabilityEmail(\%Params) unless ExistsProductAvailabilityEmail($Product->id, $EMail);
    return;
}

sub Save {  ## no critic qw(ProhibitExcessComplexity)
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop = $Product->getSite();
    my $Calendar = undef;
    my $IsAppointment = $Product->get('IsAppointment');

    # check if we have a product name only with special chars
    my $ProductName = $Servlet->form->value('Name');
    if(defined $ProductName && $ProductName =~ /\S/){
        $Servlet->form->executeFormError({
            'Form'   => 'Save',
            'Reason' => 'NAME_NOT_VALID',
            'Name'   => 'Name',
            'Value'  => $ProductName,
        }) unless $ProductName =~ /\w/;
    }

    # set the tax class form value because we have a normal and a digital tax class
    _SetTaxClassFormValue($Servlet,$Shop );

    # test form fields
    $self->attributeValues( $Servlet, $Product, $Product->class);

    # EPG-20164: user defined attributes have own prefix ('UserDefined_') to avoid errors.
    my $hUserDefinedValues = $self->attributeValues( $Servlet, $Product, $Product->class, PREFIX_USERDEFINED_ATTRIBUTES);

    # check if another object exists with the same alias
    if(defined $hUserDefinedValues) {
        if( exists $hUserDefinedValues->{'Values'}->{'Alias'}
            && ! grep { $_->{'Name'} eq 'Alias' } @{$Servlet->form->formErrors} )
        {
            my $NewAlias = $hUserDefinedValues->{'Values'}->{'Alias'};
            if( $Product->alias !~ /^\Q$NewAlias\E$/i && $Product->parent->existsChild($NewAlias) ) {
                $Servlet->vars('OBJECT_EXISTS',1);
                $Servlet->form->addFormError({
                    'Form'  => 'Save',
                    'Reason'=> 'OBJECT_EXISTS',
                    'Name'  => 'Alias',
                    'Value' => $NewAlias,
                });
            }
        }

        # die with INVALID_FORM error if any field is invalid
        $Servlet->form->executeFormError;

        Transaction( sub {
            LogDebug( 'set userdefined attributes', $hUserDefinedValues->{'Values'} );
            $Product->set( $hUserDefinedValues->{'Values'} );
            LogDebug( 'set userdefined localized attributes', $hUserDefinedValues->{'LocalizedValues'} );
            foreach my $LanguageID ( keys %{$hUserDefinedValues->{'LocalizedValues'}} ) {
                $Product->set( $hUserDefinedValues->{'LocalizedValues'}{$LanguageID},  $LanguageID );
            }
        });

        $self->SUPER::saveFileAttributes( $Servlet, $Product, $hUserDefinedValues, PREFIX_USERDEFINED_ATTRIBUTES );
    }

    my $hhCalendarValues = undef;
    if ($IsAppointment) {
        $Calendar = $Product->businessHours;
        $hhCalendarValues = $self->attributeValues( $Servlet, $Calendar, $Calendar->class );
    }

    my $Form = $Servlet->form->form($Product, 'Save');

    # test for allowed filetypes
    $self->_testUploadFilesTypes($Servlet);


    if (defined $Servlet->form->value('StockLevelAlert') && $Servlet->form->value('StockLevelAlert') ne '') {
        $Servlet->form->addFormError({
            'Reason'  => 'WrongValue',
            'Form'    => 'Save',
            'Name'    => 'StockLevelAlert',
        }) if $Servlet->form->value('StockLevelAlert') le '0';
    }

    if($IsAppointment && defined $Servlet->form->value('StockLevel') ){
        my $hValues =  $Servlet->form->form($Product, 'SaveAppointment');

        # only for appointments: display an error if the MinOrderQuantity < StockLevel
        my $AppForm = $Servlet->form;
        my $MinOrder = $AppForm->value('MinOrder');
        if ($hValues->{'StockLevel'} < $MinOrder){
            $Servlet->form->addFormError({
                'Reason'  => 'WrongMinOrderOrStockLevel',
                'Form'    => 'SaveAppointment',
                'Name'    => 'MinOrder',
            });
        }
    }

    # Prepayment
    if ($IsAppointment) {
        foreach my $Entry (@{$Form->{'PrepaymentFixPrices'}}) {
            my $Value = $Entry->{'PrepaymentFixPrice'};
            $Servlet->form->addFormError({
                'Reason'  => 'WrongValue',
                'Form'    => 'SaveAppointment',
                'Name'    => 'PrepaymentFixPrice',
            }) if defined $Value && $Value < 0;
        }
        my $Value = $Form->{'PrepaymentValue'};
        $Servlet->form->addFormError({ ## no critic qw(ProhibitMagicNumbers)
            'Reason'  => 'WrongValue',
            'Form'    => 'SaveAppointment',
            'Name'    => 'PrepaymentValue',
        }) if defined $Value && ($Value < 0 || $Value > 100);
    }


    # if eco participation for any currency, eco participation catagory is mandatory
    foreach my $Entry (@{$Form->{'EcoParticipations'}}) {
        if ($Entry->{'EcoParticipation'} && !$Form->{'EcoParticipationCategory'}) {
            $Servlet->form->addFormError({
                'Reason'  => 'MissingEcoParticipationCategory',
                'Form'    => 'Save',
                'Name'    => 'EcoParticipationCategory',
            });
            last;
        }
    }

    # EPG-27882: check availability and expiry
    my $AvailabilityDate = $Form->{'AvailabilityDate'};
    my $ExpiryDate = $Form->{'ExpiryDate'};
    if (defined $AvailabilityDate && defined $ExpiryDate) {
        if ($AvailabilityDate > $ExpiryDate) {  ## inconsistent state
            $Servlet->form->addFormError({
                'Reason'  => 'INVALID_FORM',
                'Form'    => 'Save',
                'Name'    => 'AvailabilityDate',
            });
            $Servlet->form->addFormError({
                'Reason'  => 'INVALID_FORM',
                'Form'    => 'Save',
                'Name'    => 'ExpiryDate',
            });
        }
    }

    # die with INVALID_FORM error if any field is invalid
    $Servlet->form->executeFormError;

    my %IndexMap;
    my $Index = 0;
    my @LanguageIDs = $Servlet->form->value('LanguageID');
    foreach my $LanguageID (@LanguageIDs) {
        $IndexMap{$LanguageID} = $Index unless defined $IndexMap{$LanguageID};
        $Index++;
    }

    #If objects alias (form field 'NewAlias') equals the suggested one,
    #increase the shop attribute 'NextProductNo'. -> No need to set the alias.
    if( $Product->alias eq GetNextNo($Shop, 'Product', undef, 0) ){
        GetNextNo($Shop, 'Product');
    }

    my $Vars = {};

    RunBatchProcess('UpdateProducts', sub {

    Transaction( sub {
        eval {
            $Vars = $self->SUPER::Save($Servlet);
        };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) unless ($Error->code eq 'INVALID_CHARACTER' ||
                                  $Error->code eq 'SHORTURL_ALREADY_EXISTS' ||
                                  $Error->code eq 'ShortURL_NOT_ALLOWED');
            $Servlet->form->executeFormError({
                'Reason'=> $Error->code,
                'LoopName' => 'Save',
                'Name'  => 'URI',
                'Index' => $IndexMap{$Error->vars->{'LanguageID'}},
            });
        }
    });

    # EPG-27882: update IsAvailable
    my $IsAvailable = $Form->{'IsAvailable'};
    if (defined $IsAvailable || defined $AvailabilityDate || defined $ExpiryDate) {
        $Product->set({ IsAvailable =>_IsCurrentlyAvailable($IsAvailable, $AvailabilityDate, $ExpiryDate) });
    }

    # EPG-3367: save list prices outside of the transaction
    my $InputTaxModel = $Shop->get('InputTaxModel');
    foreach my $Entry (@{$Form->{'ProductPrices'}}) {
        my $CurrencyID = $Entry->{'CurrencyID'};
        my $Value = $Entry->{'Price'};
        $Value = undef if defined $Value and $Value !~ /\S/;
        $Product->setListPrice( $CurrencyID, $InputTaxModel, $Value );
    }

    # save manufacturer's suggested retail prices
    foreach my $Entry (@{$Form->{'ManufacturerPrices'}}) {
        my $CurrencyID = $Entry->{'ManufacturerCurrencyID'};
        my $Value = $Entry->{'ManufacturerPrice'};
        $Value = undef if defined $Value and $Value !~ /\S/;
        $Product->setManufacturerPrice( $CurrencyID, $InputTaxModel, $Value );
    }

    # save eco participation
    foreach my $Entry (@{$Form->{'EcoParticipations'}}) {
        my $CurrencyID = $Entry->{'EcoParticipationCurrencyID'};
        my $Value = $Entry->{'EcoParticipation'};
        $Value = undef if defined $Value and $Value !~ /\S/;
        $Product->setEcoParticipation( $CurrencyID, $InputTaxModel, $Value );
    }

    # save deposit
    foreach my $Entry (@{$Form->{'Deposits'}}) {
        my $CurrencyID = $Entry->{'DepositCurrencyID'};
        my $Value = $Entry->{'Deposit'};
        $Value = undef if defined $Value and $Value !~ /\S/;
        $Product->setDeposit( $CurrencyID, $InputTaxModel, $Value );
    }

    # save energylabels
    my $aEnergyLabels = [];
    foreach my $Entry (@{$Form->{'EnergyLabels'}}) {
        my $Value = $Entry->{'EnergyLabel'};
        push @$aEnergyLabels,$Value if defined $Value and $Value =~ /\S/;
    }
    $Product->set({'EnergyLabels' => $aEnergyLabels } ) if scalar(@{$Form->{'EnergyLabels'}});

    if ($IsAppointment) {
        my $PrepaymentType = $Form->{'PrepaymentType'};
        if($PrepaymentType) {
            if($PrepaymentType eq 'FIX') {
                foreach my $Entry (@{$Form->{'PrepaymentFixPrices'}}) {
                    my $CurrencyID = $Entry->{'PrepaymentCurrencyID'};
                    my $Value = $Entry->{'PrepaymentFixPrice'};
                    $Value = undef if defined $Value and $Value !~ /\S/;
                    $Product->setPrepayment( $CurrencyID, $InputTaxModel, $Value );
                }
                # set percentage value to undef as we have fixed rates
                $Product->set({'PrepaymentValue' => undef});
            }
        }

        delete $hhCalendarValues->{'Values'}{$_} foreach( qw(Alias Class) );
        $Calendar->set($hhCalendarValues->{'Values'});
    } else {
        # save product specific shipping methods
        if( $Shop->featureMaxValue('ShippingMethodsPerProduct') && $Servlet->form->value('SetShippingMethods') ) {
            my $ahShippingMethods = $Form->{'ProductShippingmethods'};
            my @ProductShippingMethods = map { $Shop->loadObject($_->{'ProductShippingmethod'}, 'ShippingMethod') } @$ahShippingMethods;
            if( scalar(@ProductShippingMethods) != scalar(@{$Shop->folder('ShippingMethods')->children()})) {
                $Product->set({'ShippingMethods' => \@ProductShippingMethods});
            }
            else {
                $Product->set({'ShippingMethods' => []});
            }
        }

        # EPG-19649: Save last order unit for product creation
        my $NewProduct = $Servlet->params('ChangeAction') eq 'NewProduct' ? 1 : 0;
        if($NewProduct) {
            my $OrderUnitID = $Servlet->form->value('OrderUnit');
            if( defined $OrderUnitID ) {
                my $OrderUnit = $Shop->loadObject($OrderUnitID, 'UnitOfMeasurement', { 'AllowSystemSite' => 1 });
                $Shop->set({'LastOrderUnitForProductCreation' => $OrderUnit});
            }
        }
    }
    # trigger refresh of SEO check
    PerformSEOCheckAndUpdateObject($Product);

    });

    return $Vars;
}

sub _testUploadFilesTypes{
    my $self      = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form->form($Servlet->object(), 'FileUploads');
    # check if we upload only supported file types like *.pdf or image formats
    # The check is done by testing the file extension and not the content itself

    # map the FormField to a test function is executed
    my $hAllowedFileTypes = {
            'ProductDataSheet'      => ['IsImageFile', 'IsPDFFile' ],
            'EnergyLabelSourceFile' => ['IsImageFile']
    };

    #check EnergyLabelSourceFile
    my $EnergyLabelSourceFileName  = $Form->{'EnergyLabelSourceFile'};
    if($EnergyLabelSourceFileName){
        my $aCheckFunction = $hAllowedFileTypes->{'EnergyLabelSourceFile'};
        my $IsValid = _doFileTypeCheck($EnergyLabelSourceFileName, $aCheckFunction);
        unless ($IsValid){
            $Servlet->form->addFormError({
                    'Form'   => 'Save',
                    'Name'   => 'EnergyLabelSourceFile',
                    'Reason' => 'UNSUPPORTED_FILETYPE',
                    'File'   => $EnergyLabelSourceFileName,
            });
        };
    }
    # check ProductDataSheet
    my $Counter = 0;
    foreach my $Entry (@{$Form->{'ProductDataSheets'}}) {
        my $ProductDataSheetFileName = $Entry->{'ProductDataSheet'};

        if($ProductDataSheetFileName){
            my $aCheckFunction = $hAllowedFileTypes->{'ProductDataSheet'};
            my $IsValid = _doFileTypeCheck($ProductDataSheetFileName, $aCheckFunction);
            unless ($IsValid){
                $Servlet->form->addFormError({
                        'Form'   => 'Save',
                        'Name'   => 'ProductDataSheet',
                        'Reason' => 'UNSUPPORTED_FILETYPE',
                        'File'   => $ProductDataSheetFileName,
                        'Index'     => $Counter,
                        'LoopName'  => 'ProductDataSheet',
                });
            };
        }
        $Counter ++;
    }
}

sub _doFileTypeCheck {
    my ($FileName, $aCheckFunctions) = @_;

    my $IsValid = 0;
    # exec test function(a) and and compare with OR
    ## no critic qw(ProhibitNoStrict)
    no strict 'refs';
    $IsValid |= &{$_}($FileName) foreach @$aCheckFunctions;
    use strict 'refs';

    return $IsValid;

}

sub SaveUnityProduct {
    my $self      = shift;
    my ($Servlet) = @_;

    my $Product   = $Servlet->object;

    my @ShopLanguageIDs = map { $_->{'LanguageID'} } @{$Product->getSite()->get('Languages')};

    # check if we already have a name set for multi language shops
    my $IsNoNameSet = 1;
    if (scalar @ShopLanguageIDs > 1) {
        foreach my $LanguageID (@ShopLanguageIDs) {
            if (defined $Product->get('Name', $LanguageID)) {
                $IsNoNameSet = 0;
                last;
            }
        }
    }

    $self->Save($Servlet);

    # fill all languages with the name in case all are still empty
    if ($IsNoNameSet && scalar @ShopLanguageIDs > 1) {
        my $Name = $Servlet->form->value('Name');
        my $FormLanguageID = $Servlet->form->value('LanguageID');
        if (defined $Name) {
            foreach my $LanguageID (@ShopLanguageIDs) {
                next if $LanguageID == $FormLanguageID;
                $Product->set({'Name' => $Name}, $LanguageID);
            }
        }
    }

    _SaveDnDUploaderImages($Servlet);
    _StepAddProductDone($Servlet);

    TriggerHook('UI_OnSaveProduct', { 'Object' => $Product, 'Servlet' => $Servlet });

    $Servlet->form->executeFormError;

    PerformSEOCheckAndUpdateObject($Product);
    $Product->updatePageCache;

    return;
}

sub _StepAddProductDone {
    my ($Servlet) = @_;

    my $Shop = $Servlet->object()->getSite;
    my $hItems = $Shop->get('NextSteps') // {};
    $hItems->{ 'AddProducts' } = '1';
    $Shop->set({ 'NextSteps' => $hItems });
}

sub _SaveDnDUploaderImages {
    my ($Servlet) = @_;

    my $Product   = $Servlet->object;
    my $Shop      = $Product->getSite;
    my $Form      = $Servlet->form;

    my $PublicPath = $Product->get('PublicPath');
    my $hValues = $Form->form($Product, 'DrapAndDropProductImages');
    my %ProductImages = ();

    # manually test if ImagesOrder exists (only if we try to upload images)
    if(scalar @{$hValues->{'DragAndDropProductImage'}}){
        $Form->executeFormError({
            'Reason' => 'ImageWriteFileError',
            'Name'   => 'ImageWriteFileError',
            'File'   => undef
        }) unless defined $hValues->{'ImagesOrder'};
    }

    foreach my $hProductImage (@{$hValues->{'DragAndDropProductImage'}}) {
        my (undef, $FileName, $Extension) = fsplit($hProductImage->{'name'});
        my $FilePath = $PublicPath . '/' . Unicode2Filename($FileName) . $Extension;

        my $Content = $hProductImage->{'dataUrl'};
        $Content =~ s/^(?:.*?base64,)(.*)$/$1/;

        my $DecodedContent = decode_base64($Content);

        eval { WriteFile($FilePath, \$DecodedContent, { 'binary' => 1, 'create_directory' => 1 }); };
        if (ExistsError()) {
            $Form->addFormError({
                'Reason' => 'ImageWriteFileError',
                'Name'   => 'ImageWriteFileError',
                'File'   => $hProductImage->{'name'}
            });
        }
        else {
            my $hScaledImages;

            eval {
                $hScaledImages = GetScaledImages($FilePath, $Shop);
            };
            if (ExistsError() or not defined $hScaledImages) {
                $Form->addFormError({
                    'Reason' => 'ImageResizeError',
                    'Name'   => 'ImageResizeError',
                    'File'   => $hProductImage->{'name'}
                });

                eval { DeleteFile($FilePath); };
                if (ExistsError()) {
                    Warning("Cannot delete file $FilePath");
                }
            }
            else {
                $ProductImages{$hProductImage->{'id'}} = $hScaledImages;
            }
        }
    }

    my @SortedImages = defined $hValues->{'ImagesOrder'} ? grep { $_ ~~ [keys %ProductImages] } @{from_json($hValues->{'ImagesOrder'})} : ();

    if (scalar @SortedImages) {
        $Product->set($ProductImages{$SortedImages[0]});
        $Product->set({'ImagesSlideShowString' => join(';', map { $ProductImages{$_}->{'ImageLarge'} } @SortedImages)});
    }
}

sub _SetTaxClassFormValue {
    my ($Servlet,$Shop) = @_;

    my $FormTaxClass = $Servlet->form->value('FormTaxClass');
    return unless $FormTaxClass;
    my $hValues = $Servlet->form->values();

    # # we have a TaxMethod if an external tax calculation cartridge is installed
     if ($Shop->get('TaxMethod')) {

        #set correct tax class on product
        $hValues->{'TaxClass'} = $FormTaxClass;
        $hValues->{'TaxClassDigital'}   = undef;
     }
     else {
        my $ClassTaxClass = LoadClassByAlias( 'TaxClass');
        my $IsNormalTaxClass = $ClassTaxClass->existsObject($FormTaxClass);
        if (not $IsNormalTaxClass) {
            # use the default normal tax class, if we use the digital tax class for the product
            my $ShopDefaultTaxClass = $Shop->get('TaxMatrix')->get('DefaultTaxClass');
            $hValues->{'TaxClass'} = $ShopDefaultTaxClass->id;
            $hValues->{'TaxClassDigital'} = $FormTaxClass;
        }
        else {
            $hValues->{'TaxClassDigital'} = undef;
            $hValues->{'TaxClass'} = $FormTaxClass;
        }
    }

    $Servlet->form->setValues($hValues);
}


sub SaveFiles {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Form = $Servlet->form->form($Product, 'SaveFiles');

    if ($Servlet->params('Delete')) {
        my $aFiles = $Form->{'Delete'};
        return unless scalar @{$aFiles};

        if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
            # set delete mark to template
            my @DeleteFiles;
            foreach my $File (@{$aFiles}) {
                my %Vars;
                $Vars{'DeleteFile'} = $File->{'DeleteFile'};
                push @DeleteFiles,\%Vars;
            }
            $Servlet->vars('DeleteFiles',\@DeleteFiles);
            $Servlet->vars('DELETE',1);
            return;
        } else {
            # else reset delete
            $Servlet->vars('DELETE',0);
        }

        # delete images
        DeleteProductImage( $Product, $_->{'DeleteFile'} ) foreach grep { $_->{'DeleteFile'} } @$aFiles;
    } else {
        # update string if file name changed
        my @FormFiles;
        my $Counter = 0;
        my $hUpdateProductData;
        my $ParentProduct = $Product->superProduct;
        my $aSubProducts = defined $ParentProduct ? $ParentProduct->subProducts : $Product->subProducts;
        my $hUpdateSubProductData;

        foreach my $FormFile (@{$Form->{'Save'}}) {
            # EPG-12387 only apply file operations for local files, not for URLs
            if(!StringIsURL($FormFile->{'NewFileName'})) {
                # get fname and ext
                my (undef, $OldFileName, $FileExt) = fsplit($FormFile->{'OldFileName'});
                # clean up
                my $NewFileName = Unicode2Filename($FormFile->{'NewFileName'});

                # filename changed?
                if($OldFileName ne $NewFileName) {
                    eval { ## no critic qw(ProhibitDeepNests)
                        # rename physical image file
                        my $hResult = RenameProductImageFile (
                            $Product,
                            $OldFileName,
                            $NewFileName,
                            $FileExt
                        );
                        # is there an easier way for this?

                        # check result for image renaming
                        if(defined $hResult) {

                            # take updated product data into account
                            if(defined $hUpdateProductData) {
                                %$hUpdateProductData = ( %$hUpdateProductData, %$hResult );
                            } else {
                                %$hUpdateProductData = %$hResult;
                            }
                        }
                    };
                    if (ExistsError()) {
                        my $Error = GetError();
                        if($Error->code eq 'ANOTHERFILEALREADYEXISTS') { ## no critic qw(ProhibitDeepNests)
                            $Servlet->form->addFormError({
                                'Name'      => 'NewFileName',
                                'Index'     => $Counter,
                                'LoopName'  => 'ProductSlideshowImages',
                                'Reason'    => 'ANOTHERFILEALREADYEXISTS',
                            });
                        } elsif($Error->code eq 'FILE_MOVE_FAILED') {
                            Warning($Error);
                        } else {
                            Error($Error);
                        }
                        next;
                    }

                    # update string
                    $FormFile->{'NewFileName'} = $NewFileName.$FileExt;
                } else {
                    # no change, old is new, new is old ^^
                    $FormFile->{'NewFileName'} = $FormFile->{'OldFileName'};
                }
                foreach my $SubProduct (@$aSubProducts){
                    my $SubProductID = $SubProduct->id;
                    my $hResult = RenameSubProductImageFile($SubProduct, $OldFileName, $NewFileName, $FileExt);
                    if (defined $hUpdateSubProductData->{$SubProductID}){
                        my %OldUpdateSubProductData = %{$hUpdateSubProductData->{$SubProductID}};
                        %$hResult = defined $hResult ? (%OldUpdateSubProductData,%$hResult) : %OldUpdateSubProductData;
                    }
                    $hUpdateSubProductData->{$SubProductID} = $hResult if defined $hResult;
                }
            }
            push(@FormFiles, $FormFile);
            $Counter++;
        }
        $Servlet->form->executeFormError;

        # build a new ImagesSlideShowString fom the form only if we not delete an image
        my @SlideShowImages = map { $_->{'NewFileName'} }
            sort { $a->{'Position'} <=> $b->{'Position'} }
            grep { $_->{'VisibleInSlideShow'} }
            @FormFiles;

        # set filename attribute for changed file
        $hUpdateProductData->{'ImagesSlideShowString'} = scalar @SlideShowImages ? join(';', @SlideShowImages) : undef;
        $Product->set($hUpdateProductData);

        my $Shop = $Product->getSite();

        foreach my $ChangedSubProductID (keys %$hUpdateSubProductData){
            my $ChangedSubProduct = $Shop->loadObject($ChangedSubProductID, 'Product');
            my $hUpdateData       = $hUpdateSubProductData->{$ChangedSubProductID};
            $ChangedSubProduct->set($hUpdateData);
        }
    }
    UpdatePageCache($Product);

    # trigger refresh of SEO check
    PerformSEOCheckAndUpdateObject($Product);

    return;
}

sub DeleteSubProducts {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;

    $self->SUPER::DeleteListedObjects( $Servlet, LoadClassByAlias('Product') );
    $Product->folder('SubProducts')->renumberChildren;

    # only for unity clean up variations on master product to have a normal product if no variations exists
    if($Product->getSite()->get('IsUnityRegistered') and not $Product->countSubProducts){
        $Product->set({'SelectedVariations' => undef});
    }

}

sub SaveVariationTypes {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my @SelectedVariations = $Servlet->form->value('SelectedVariations');
    my $Shop = $Product->getSite();

    my $rExistingValues = GetInfoSelectedVariationsByProductID($Product->id);
    my %SelectVariationIDs = map { $_->{'PreDefAttributeID'} => undef } @$rExistingValues;
    my %UsedVariationIDs = map { $_ => undef } @{ UsedVariationValues($Product) };

    my $VariationTypes = $Product->get('VariationTypeValues');
    my %AttributeIDs = map { $_->{'Attribute'}->id => undef } @$VariationTypes;

    my $aSubProducts = $Product->subProducts;
    my $SetDefault = 0;
    my @SetDefaultValueAttributes;
    my @UsedPreDefAttributes;
    my @AddPreDefAttributes;
    foreach my $ID ( @SelectedVariations ) {
        next if exists $UsedVariationIDs{$ID}; #already used
        next if exists $SelectVariationIDs{$ID}; #already selected
        my $PreDefAttribute = $Shop->loadObject($ID, 'PreDefAttribute');
        my $Attribute = $PreDefAttribute->get('Attribute');
        if ( exists $AttributeIDs{$Attribute->id} ) {
            push @AddPreDefAttributes, $PreDefAttribute;
            next;
        }
        my $AttributeName = $Attribute->alias;
        foreach my $SubProduct (@$aSubProducts) {
            if (not defined $SubProduct->get($AttributeName) ) {
                # value is not available for all subproducts
                $SetDefault = 1;
                push @SetDefaultValueAttributes, $Attribute;
                push @UsedPreDefAttributes, $PreDefAttribute;
                last;
            }
        }
    }

    if ($SetDefault) {
        my $aSelectedVariations = $Product->get('SelectedVariations');
        push @$aSelectedVariations, {'SelectedID' => $_->id} foreach @AddPreDefAttributes;
        $Product->set( { 'SelectedVariations' => $aSelectedVariations } );

        $Servlet->vars('SetDefault', 1);
        my @UniqueUsedPreDefAttributes = unique @UsedPreDefAttributes;
        $Servlet->vars('UniqueUsedPreDefAttributes', [map {$_->tleHash} unique @UniqueUsedPreDefAttributes]);
        $Servlet->vars('SetDefaultValueAttributes', [map {$_->tleHash} unique @SetDefaultValueAttributes]);
    } else {
        my $Warnings = BlockWarnings(sub {
            $Product->set( { 'SelectedVariations' => [map { {'SelectedID' => $_} } @SelectedVariations ] } );
        }, ['PreDefAttributeUsedBySubProduct']);
        $Servlet->form->executeFormError({
                'Form'  => 'SaveVariationTypes',
                'Reason'=> $Warnings->[0]->code,
                'LoopName'  => 'PreDefAttributes',
                'Name'  => 'SelectedVariations',
                'Value' => $Warnings->[0]->vars->{'PreDefAttributeID'}
        }) if scalar @$Warnings;
    }

    UpdatePageCache($Product);

    return;
}

sub SaveSubProducts {

    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();
    NewSubProducts($self,$Servlet) if $Servlet->params('NewAttributeValue') =~ /[^\0]+/ and $Servlet->params('NewAttributeValue') !~ /^\s*$/;

    my $Form;
    if ($Shop->get('IsUnityRegistered')) {
        $Form = $Servlet->form->form($Product, 'UnitySaveSubProducts');
    } else {
        $Form = $Servlet->form->form($Product, 'SaveSubProducts');
    }

    #set default if defined and we have set the flag UseDefaultSubProduct
    my $SubProduct;
    if ($Form->{'UseDefaultSubProduct'}){
        if (defined $Form->{'DefaultSubProduct'} and $Form->{'DefaultSubProduct'} ne '') {
            $SubProduct = $Shop->loadObject($Form->{'DefaultSubProduct'}, 'Product');
        } else {
            # get the first sub product as default
            my $aSubProductIDs = $Product->findSubProducts;
            if (scalar(@$aSubProductIDs) > 0) {
                $SubProduct = $Shop->loadObject($aSubProductIDs->[0], 'Product');
            }
        }
        $Servlet->testPermission($SubProduct, 'Save');
        $Product->set({'DefaultSubProduct'=> $SubProduct});
    }

    # set default sub product
    $Servlet->testPermission($SubProduct, 'Save') if defined $SubProduct ;
    $Product->set({'DefaultSubProduct'=> $SubProduct});

    # set position
    foreach my $SubProductEntry (@{$Form->{'SubProducts'}}) {
        my $SubProductToPos = $Shop->loadObject($SubProductEntry->{'SubProductID'}, 'Product');
        $Servlet->testPermission($SubProductToPos, 'Save');
        delete $SubProductEntry->{'SubProductID'};
        $SubProductToPos->set($SubProductEntry);
    }
    $Product->folder('SubProducts')->renumberChildren;
    return;
}

sub NewSubProducts {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;

    my $hForm = $Servlet->form->form($Product, 'NewSubProducts');
    my $IsUnityRegistered = $Product->getSite()->get('IsUnityRegistered');

    # patch VariationTypeValues (SelectedVariations) with additional attribute values
    # because not used attibute values are not in SelectedVariations
    if($IsUnityRegistered) {

        my $hNewIDs = {}; # container for unique ID's
        foreach my $hEntry (@{$hForm->{'NewVariations'}}){
            # get all values from attribute alias
            if ( $hEntry->{'NewAttributeValue'} =~ /\*/) {
                my $Attribute = $Product->get('Class')->folder('Attributes')->child($hEntry->{'NewAttributeAlias'});
                my $aAttributeValues = $Attribute->get('PreDefAttributes');
                $hNewIDs->{$_->get('ID')} = undef foreach (@$aAttributeValues);
            } else {
            # add single value
                $hNewIDs->{$hEntry->{'NewAttributeValue'}} = undef;
            }
        }

        # add existing SelectedVariations
        my $aExistingSelectedVariationValues = [ map { $_->{'SelectedID'} } @{$Product->get('SelectedVariations')} ] ;
        $hNewIDs->{$_} = undef foreach (@$aExistingSelectedVariationValues);

        $Product->set( { 'SelectedVariations' => [ map { {'SelectedID' => $_} } keys(%$hNewIDs) ] });

    }

    my $PosVariationTypeValues = $Product->get('VariationTypeValues');

    foreach my $PosVariationType (@$PosVariationTypeValues ) {
        my $Attribute = $PosVariationType->{'Attribute'};

        #get the matching entry
        my @VariationValues = grep { $_->{'NewAttributeAlias'} eq $Attribute->alias  } @{$hForm->{'NewVariations'}};
        my $VariationValueEntry = $VariationValues[0];

        if ( $VariationValueEntry->{'NewAttributeValue'} =~ /\*/) {
            #all possible attributes should be used -> nothing to do
        } elsif ( $VariationValueEntry->{'NewAttributeValue'} =~ /\S/) {
            #only a specific value should be used
            my @PosVariationValues = grep { $_->id == $VariationValueEntry->{'NewAttributeValue'} } @{$PosVariationType->{'Values'}};
            $PosVariationType->{'Values'} = \@PosVariationValues;
        } # else {            #do nothing -> shouldn't happen because value is required }

    }
    delete $hForm->{'NewVariations'};
    my %Values = map { s/^New//; $_ => $hForm->{'New'.$_} } grep { /^New/ } keys %$hForm;

    my $hOptions;
    $hOptions->{'Monitor'} = {'Servlet' => $Servlet} if $IsUnityRegistered;

    my $aWarnings = BlockWarnings( sub {
        InsertMultipleSubProducts($Product, $PosVariationTypeValues, \%Values, $hOptions );
    }, ['VariationCombinationExists', 'SubProductsCountFeatureLimitExceeded'] );

    $Product->folder('SubProducts')->renumberChildren if scalar @$aWarnings;
    $Servlet->form->mapFormError({'Name'  => 'NewSubProducts',
                                  'Form'  => 'NewSubProducts'}, $_) foreach @$aWarnings;
    $Servlet->form->executeFormError;
    return;
}

# called from the sf productcustomization file uploader
sub UploadLineItemCustomizedFile {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;

    my $Product = $Servlet->object;
    my $Shop = $Product->getSite();

    # upload image and save to temp path
    # upload to %EPAEGS_WEBROOT%/Store/Shops/DemoShop/CustomizableProducts<GUID PATH >/
    my $GUID = GenerateGUID();
    my $GUIDPath = GenerateSplittedGUIDPath($GUID);
    my $Directory = join( '/', $Shop->get('PublicPath'), 'CustomizableProducts', $GUIDPath).'/';

    my ($FileName, undef) = Upload($Shop, $Form, {'Path' => $Directory});
    my $File = $Directory.$FileName;

    # test, if image has the correct min size
    my $AllowSizeTest = 1;
    eval {
        TestImageFormat('UploadFile', $File);
        };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless ($Error->code eq 'UNSUPPORTED_IMAGE_FORMAT');
        $AllowSizeTest = 0;
    }

    my $FileOptionID = $Servlet->params('FileOptionID');
    if ($FileOptionID and $AllowSizeTest) {
        my $hOptionParams = $Shop->loadObject($FileOptionID, 'FileOption')->get(['MinHeight','MinWidth']);
        my $hImageInfo = GetImageInfo($File);  #Height #Width
        foreach my $Key (qw(Height Width)) {
            next unless $hOptionParams->{'Min'.$Key};
            # text if the image sizes are >= the option params
            $Form->executeFormError({
                'Reason'=> 'INVALID_IMAGE_SIZE',
                'Form'  => 'UploadFiles',
                'Name'  => $Key,
            }) if ($hOptionParams->{'Min'.$Key} > $hImageInfo->{$Key});
        }
    }

    $Servlet->vars('ResponseContent', {
        'uploadedFile' => $Shop->get('WebPath').'/'.RelativePath($Shop->get('PublicPath'), $File),
        'FileName' => $FileName,
        'uploadedObjectFile' => RelativePath($Shop->get('PublicPath'), $File),
        'GUID' => $GUID,
    });
}

# cleanup ununsed temporary files, if customizer was closed
sub CleanUpTempCustomizedFiles {
    my $self = shift;
    my ($Servlet) = @_;

    # cleanup unused temp files by using the GUID list form the customizer
    $self->_cleanUpTempCustomizedFiles($Servlet);

}

# helper function called from CleanUpTempCustomizedFiles
# and AddCustomizedProductToBasket if the files are copied to the lineitem objects
sub _cleanUpTempCustomizedFiles {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object->getSite();

    # JSON CleanUpGUIDList
    # cleanup unused temp files by using the GUID list form the customizer
    my $Base64CleanupGUIDList = $Servlet->params('CleanupGUIDList');
    if ($Base64CleanupGUIDList) {
        my $CleanupGUIDList = decode_base64($Base64CleanupGUIDList);
        #DE_EPAGES::Core::API::Log::LogDebug('CLEANUPGUID LIST ', $CleanupGUIDList );
        my $aCleanupGUIDs = $CleanupGUIDList ? from_json($CleanupGUIDList) : [] ;
        if (scalar(@$aCleanupGUIDs)) {
            # build path and delete foreach GUID
            foreach my $GUID (@$aCleanupGUIDs) {
                my $GUIDPath = GenerateSplittedGUIDPath($GUID);
                my $TempDirectory = join( '/', $Shop->get('PublicPath'), 'CustomizableProducts', $GUIDPath).'/';
                #DE_EPAGES::Core::API::Log::LogDebug('CLEANUPGUID', $TempDirectory );
                DeleteDirectory($TempDirectory ,{'recursive' => 1});
            }
        }

        # remove empty directories in CustomizableProducts path
        RunOnDirectory($Shop->get('PublicPath').'/'. 'CustomizableProducts/', sub {
                $self->_deleteEmptySubDirs('.');
            }
        ) if ExistsDirectory($Shop->get('PublicPath').'/'. 'CustomizableProducts');
    }

    return;
}

sub _deleteEmptySubDirs {
    my $self = shift;
    my ($Directory) = @_;

    # count all entries in sub directories
    my %Directories;
    find({
        wanted => sub {
            $Directories{$_} = 0 if -d;
            $Directories{dirname($_)}++;
        },
        no_chdir => 1,
    }, $Directory);

    # delete empty sub directories until no empty directories are left
    while( my @EmptyDirs = grep { 0 == $Directories{$_} } keys %Directories ) {
        foreach my $Dir ( @EmptyDirs ) {
            delete $Directories{$Dir};
            $Directories{dirname($Dir)}--;
            #DE_EPAGES::Core::API::Log::LogDebug('remove empty dir', $Dir );
            DeleteDirectory( $Dir );
        }
    }

    return;
}

sub AddCustomizedProductToBasket {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop = $Product->getSite();
    my $hValues = $Servlet->form->form($Product,'AddToBasket');

    # do we have customization data ?
    my $hhCustomizationData = {};
    my $CustomizedProductData = $Servlet->params('Base64CustomizedAreas');
    my $DecodedString = decode_base64($CustomizedProductData);
    #DE_EPAGES::Core::API::Log::LogDebug('AddCustomizedProductToBasket DecodedString', $DecodedString );

    $hhCustomizationData = from_json($DecodedString, { utf8  => 1  }) if $DecodedString;
    my $IsCustomized = scalar(keys(%$hhCustomizationData)) ? 1 : undef;

    #insert the new product as a new lineitem if we have a customizable product
    $hValues->{'AddProductStyle'} = BASKETACTION_ADDNEWPOSITION ;

    # delete old lineitem from basket, if we have changed the customization
    foreach my $hCustomizedTextOrFile (values(%$hhCustomizationData)) {
        if ($hCustomizedTextOrFile->{'ExistingLineItemID'}) {
            $Shop->loadObject($hCustomizedTextOrFile->{'ExistingLineItemID'}, 'Object')->delete;
            last;
        }
    }

    # only if areas are empty
    # delete existing product with ExistingLineItemID and add new customized product
    my $ExistingLineItemID = $Servlet->params('ExistingLineItemID');
    $Shop->loadObject($ExistingLineItemID, 'Object')->delete if $ExistingLineItemID;


    # add procuct to basket
    my $Basket = $self->_AddToBasket($Servlet,$hValues);

    # save customization date and set attribute on IsCustomized lineitem
    if (defined $IsCustomized) {

        # get the correct customized product from the basket by Quantity, product and creationdate
        # sort product lineitems by creationdate to identify the last added customized lineitem
        my $aProductLineItems = [];
        my $aLineItems = $Basket->container->get('LineItems');
        @{$aProductLineItems} = sort { $b->get('CreationDate') cmp $a->get('CreationDate') }
                                grep {$_->instanceOf('LineItemProduct') and $_->get('Product')->id == $Product->id } @{$aLineItems};

        my $LineItemProduct  = $aProductLineItems->[0];

        # save customization on lineitem
        #delete existing file and text customizations
        $LineItemProduct->folder('LineItemCustomizedTexts')->deleteChildren();
        $LineItemProduct->folder('LineItemCustomizedFiles')->deleteChildren();
        my $LineItemDeleted;
        foreach my $OptionID (keys(%$hhCustomizationData)) {
            my $hCustomizedTextOrFile = $hhCustomizationData->{$OptionID};
            $hCustomizedTextOrFile->{'OptionID'} = $OptionID;
            #DE_EPAGES::Core::API::Log::LogDebug('$hCustomizedTextOrFile', $hCustomizedTextOrFile );
            my $Type = $hCustomizedTextOrFile->{'Type'};
            my $GUID = $hCustomizedTextOrFile->{'GUID'};
            delete $hCustomizedTextOrFile->{'Type'};
            delete $hCustomizedTextOrFile->{'GUID'};
            delete $hCustomizedTextOrFile->{'ExistingLineItemID'};
            #save LineItemCustomizedText
            if($Type eq 'text' && ($hCustomizedTextOrFile->{'Text'} =~ /\S/) ){
                $LineItemProduct->insertLineItemCustomizedText($hCustomizedTextOrFile);
            # save LineItemCustomizedFile
            } elsif($Type eq 'file' ){
                my $LineItemCustomizedFile = $LineItemProduct->insertLineItemCustomizedFile($hCustomizedTextOrFile);
                #move customized images from temp path to public path of LineItemCustomizedFile object
                if (defined $GUID) {
                    # delete lineitem if we have a customized file object with the same GUID
                    # (cann occur on a double click on the AddToBasket button)
                    if (ExistsObjectByGUID($GUID)){
                        Warning('CUSTOMIZED_FILE_EXISTS', {'GUID' => $GUID });
                        $LineItemProduct->delete;
                        $LineItemDeleted = 1;
                        last;
                    }
                    # use the generated temp path guid for the new object
                    $LineItemCustomizedFile->set({'GUID' => $GUID });
                    my $GUIDPath = GenerateSplittedGUIDPath($GUID);
                    my $FileName = $hCustomizedTextOrFile->{'FileName'};;
                    my $TempDirectory = join( '/', $Shop->get('PublicPath'), 'CustomizableProducts', $GUIDPath).'/';
                    my $TargetFile = $LineItemCustomizedFile->get('PublicPath').'/'.$FileName;
                    MoveFile($TempDirectory.$FileName , $TargetFile,  { create_directory=>1 })if ExistsFile($TempDirectory.$FileName);
                }
            }
        } # end foreach

        # cleanup unused temp files by using the GUID list form the customizer
        $self->_cleanUpTempCustomizedFiles($Servlet);

        if (not $LineItemDeleted){
            # set Attribute IsCustomized
            $LineItemProduct->set({'IsCustomized' => 1});

            #save the base 64 encoded preview image to the private path of the lineitem and set attribute CustomizedPreviewImage (the filename)
            my $Base64PreviewImage = $Servlet->params('Base64PreviewImage');
            if ($Base64PreviewImage) {
                my $FileContent = decode_base64($Base64PreviewImage);
                my $Filename = 'PreviewImage.png';
                my $PathFileName = $LineItemProduct->get('PublicPath').'/'.$Filename;
                WriteFile($PathFileName,\$FileContent, { binary=>1, create_directory=>1}) if $FileContent;
                $LineItemProduct->set({'CustomizedPreviewImage' => $Filename});
            }

            # save the base 64 encoded image to the private path of the lineitem and set attribute CustomizedImage (the filename)
            my $Base64Image = $Servlet->params('Base64Image');
            if ($Base64Image) {
                my $FileContent = decode_base64($Base64Image);
                my $Filename = 'Image.png';
                my $PathFileName = $LineItemProduct->get('PublicPath').'/'.$Filename;
                WriteFile($PathFileName,\$FileContent, { binary=>1, create_directory=>1}) if $FileContent;
                $LineItemProduct->set({'CustomizedImage' => $Filename});
            }

        } #end $GUIDAlreadyExists
    } # end $IsCustomized

    # don't return basket for kelkoo, pangora, and other external sites
    return if $hValues->{'ViewProduct'};
    # display the object that's passed as parameter (from previous page)
    return $hValues->{'LastViewObject'} if defined $hValues->{'LastViewObject'};
    # display the basket page
    return $Basket;
}

sub AddToBasket {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;
    my $hValues = $Servlet->form->form($Product,'AddToBasket');

    my $Basket = $self->_AddToBasket($Servlet,$hValues);

    # don't return basket for kelkoo, pangora, and other external sites
    return if $hValues->{'ViewProduct'};
    # display the object that's passed as parameter (from previous page)
    return $hValues->{'LastViewObject'} if defined $hValues->{'LastViewObject'};
    # display the basket page
    return $Basket;

}


sub _AddToBasket {
    my $self = shift;
    my ($Servlet,$hValues) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();
    my $User    = $Servlet->user // $Servlet->createAnonymousUser($Shop);

    my $AddProductStyle = defined $hValues->{'AddProductStyle'} ? $hValues->{'AddProductStyle'} : undef ;
    delete $hValues->{'AddProductStyle'};

    if( $User->get('IsAnonymous') ) {
        # check for: Shopping basket only for registered customers
        if ($Shop->get('LoginRequired2Order')) {
            $Servlet->form->executeFormError({
                Reason => 'LoginRequired2Order',
                Name   => 'Basket',
            })
        }
    }
    else {
        my $Customer = GetCustomerForUser($User, $Shop);
        if( defined $Customer ) {
            if( ! $Customer->get('IsDoOrderAllowed') ) {
                $Servlet->form->executeFormError({
                    Reason => 'LoginRequired2InstantlyOrder',
                    Name   => 'Basket',
                });
            }
        }
    }

    # EPG-5375: check Quantity
    if (defined $hValues->{'Quantity'} && $hValues->{'Quantity'} <= 0){
        $Servlet->form->addFormError({
            'Reason'  => 'WrongQuatity',
            'Form'    => 'AddToBasket',
            'Name'    => 'Quantity',
        });
        $Servlet->form->executeFormError;
    }

    # display the basket page if the option "Opens the basket" is selected
    my $ShowBasket = $Shop->get('AddToBasketView');
    if( !$ShowBasket && $hValues->{'LastViewObjectID'} ) {
        # EPG-3414: don't add the product to the basket if the category does not exist anymore
        if( !ExistsObject($hValues->{'LastViewObjectID'}) ) {
            Error('OBJECT_NOT_FOUND', { 'ObjectID' => $hValues->{'LastViewObjectID'} });
        }
        $hValues->{'LastViewObject'} = $Shop->loadObject($hValues->{'LastViewObjectID'}, 'Category');

    }

    my ($aBaskets, $aWarnings);
    if (not $Product->get('IsAppointment')) {
        $aWarnings = BlockWarnings( sub {
            $aBaskets = AddProducts($User, {
                'LocaleID'   => $Servlet->locale,
                'CurrencyID' => $Servlet->currencyID,
                'AddProductStyle' => $AddProductStyle,
            },
            [{
                'Product'  => $Product,
                'Quantity' => $hValues->{'Quantity'},
            }]);
        }, ['ProductNoPrice', 'ProductCanNotAddToBasket', 'ProductCanNotChangedAtBasket',
            'NoShipping', 'NoPayment', 'ExclusiveShippingMethod', 'ReachMaxBasketPositions', 'ChangedShippingMethod' ]);
        if( scalar @$aWarnings ) {
            $Servlet->vars('ErrorObjectID' => $hValues->{'LastViewObject'}->id) if $hValues->{'LastViewObject'}; # EPG-3440
            foreach (@$aWarnings) {
                if ($_->code eq 'ChangedShippingMethod') {
                    $Servlet->vars('IsRestrictedByProduct', 1);
                } elsif ($_->code eq 'NoShipping' && $Shop->featureMaxValue('ShippingMethods') == 0) {
                    # this is a shop without shipping feature: ignore error
                } elsif ($_->code eq 'NoPayment' && $Shop->featureMaxValue('PaymentMethods') == 0) {
                    # this is a shop without payment feature: ignore error
                } else {
                    $Servlet->form->mapFormError({'Name'  => 'AddToBasket'}, $_);
                }
            }
            $Servlet->form->executeFormError;
        }
    }


    return $aBaskets->[0];

}

sub SelectSubProduct {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;
    my $SuperProduct = $Product->get('SuperProduct');
    $SuperProduct = $Product unless defined $SuperProduct;

    my $Form = $Servlet->form->form($Product, 'SelectSubProduct');

    my $aSelectedVariations = $Form->{'SelectedVariations'};
    unless (scalar @{ $aSelectedVariations }) {
        GetLog('USERFAULT::Product')->warn('Given form values (SelectedVariations) not present for ChangeAction \'SelectSubProduct\'');
        return;
    }

    my $aVariations = [ map {$_->{'SelectedVariation'} }(@{ $aSelectedVariations }) ];
    my $SubProduct;
    eval {
        $SubProduct = GetSubProductByAttributeValues($SuperProduct->id, $aVariations);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'SUBPRODUCT_NOT_EXISTS';
        $self->_addSelectSubProductFormError($Servlet);

        return;
    }
    return $SubProduct if $SubProduct->get('IsVisible');
    $self->_addSelectSubProductFormError($Servlet);

    return;
}

sub _addSelectSubProductFormError {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars('SUBPRODUCT_NOT_EXISTS', 1);
    $Servlet->form->executeFormError({
        'Reason'=> 'SUBPRODUCT_NOT_EXISTS',
        'Form'  => 'SelectSubProduct',
        'Name'  => 'SelectedVariations',
    });

    return;
}

sub SaveDefaultValue {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Form = $Servlet->form;
    my $Shop = $Product->getSite();
    my @DefaultAttributeIDs = unique ($Form->value('DefaultAttributeIDs'));
    my @AttributeIDs = unique ($Form->value('AttributeIDs'));

    my $aSubProducts = $Product->subProducts;
    if (scalar @$aSubProducts) {
        # set default predef attributes
        foreach my $AttributeID (@DefaultAttributeIDs) {
            my $PreDefAttributeID = $Form->value('PreDefAttributeID' . '_' . $AttributeID);
            $Servlet->form->addFormError( {
                    'Form'      => 'VariationTypes',
                    'LoopName'  => 'SelectedVariations',
                    'Reason'    => 'ValueNotAvailforAllSubs',
                    'Name'      => 'ErrorIndex',
                    'Value'     => 1,
            }) unless defined $PreDefAttributeID and $PreDefAttributeID =~ /\S/;
            my $PreDefAttribute = $Shop->loadObject($PreDefAttributeID, 'PreDefAttribute');
            foreach my $SubProduct (@$aSubProducts) {
                $SubProduct->set({$PreDefAttribute->get('Attribute')->alias => $PreDefAttribute });
            }
            $Product->set({$PreDefAttribute->get('Attribute')->alias => $PreDefAttribute });
            my $aSelectedVariations = $Product->get('SelectedVariations');
            push @$aSelectedVariations, {'SelectedID' => $PreDefAttribute->id};
            $Product->set( { 'SelectedVariations' => $aSelectedVariations } );
        }
        # set additional predef attributes
        foreach my $AttributeID (@AttributeIDs) {
            my $PreDefAttribute = $Shop->loadObject($AttributeID, 'PreDefAttribute');
            my $aSelectedVariations = $Product->get('SelectedVariations');
            push @$aSelectedVariations, {'SelectedID' => $PreDefAttribute->id};
            $Product->set( { 'SelectedVariations' => $aSelectedVariations } );
        }
    }
    return;
}

sub ViewAppointments {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;
    $Product->businessHours;

    $self->View($Servlet);
    return;
}

sub ViewNewAppointment {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;
    my $Calendar = $Product->businessHours;

    if( defined $Servlet->params('CustomerID') ) {
        $Servlet->vars('Address', $Product->getSite()->loadObject($Servlet->params('CustomerID'), 'Customer')->get('BillingAddress')->tleHash);
    }
    $Servlet->vars('OrderID', $Servlet->params('OrderID'));
    unless($Servlet->vars('FormError')) {
        my $hValues = $Servlet->form->form($Calendar, 'StartEnd');
        if (defined $hValues->{'StartDate'} and not defined $hValues->{'EndDate'}) {
            my ($Unit, $Amount) = $Calendar->defaultDuration;
            my $DurationUnit = $Unit->durationUnitPlural;
            $hValues->{'EndDate'} = AddDuration($hValues->{'StartDate'}->clone, $DurationUnit, $Amount);
        }
        $hValues->{'Quantity'} = $Product->get('MinOrder') unless defined $hValues->{'Quantity'};
        $Servlet->vars('Quantity', $hValues->{'Quantity'});
        $Servlet->vars('StartDate', $hValues->{'StartDate'}) if defined $hValues->{'StartDate'};
        $Servlet->vars('EndDate', $hValues->{'EndDate'}) if defined $hValues->{'EndDate'};
        $self->View($Servlet);
    }
    return;
}

sub _newAppointmentTestForm {
    my $self = shift;
    my ($Servlet, $CanPayAppointment) = @_;

    my $Product  = $Servlet->object;
    my $Calendar = $Product->businessHours;

    my $hValues = $Servlet->form->form($Product, 'NewAppointment',1);
    $hValues->{'Address'} = $self->checkRegistrationSet($Servlet, $Product->getSite(),'Address', 'BillingAddress');

    # test validation of zipcode
    if ($hValues->{'Address'}{'Zipcode'}) {
        my $Zipcode = $hValues->{'Address'}{'Zipcode'};
        my $CountryID = $hValues->{'Address'}{'CountryID'};
        $Servlet->form->addFormError({
            'Reason'=> 'ZIPCODE_NOT_VALID',
            'Form'  => 'Booking',
            'Name'  => 'Address.Zipcode',
        }) unless CheckZipcode($Zipcode, $CountryID);
    }

    # test quantity
    my $Quantity = $Product->correctQuantity($hValues->{'Quantity'});
    $Servlet->form->addFormError({
        'Reason' => 'FalseQuantity',
        'Form'   => 'Booking',
        'Name'   => 'Quantity'
    }) if fcmp($Quantity, $hValues->{'Quantity'});

    $Servlet->form->addFormError({
        'Reason' => 'FalseQuantityStockLevel',
        'Form'   => 'Booking',
        'Name'   => 'Quantity'
    }) if $hValues->{'Quantity'} > $Product->get('StockLevel');

    # test payment
    if ($CanPayAppointment) {
        $Servlet->form->addFormError({
            'Reason'=> 'AcceptTAC',
            'Form' => 'Booking',
            'Name' => 'tac'
        }) unless $hValues->{'tac'};

        $Servlet->form->addFormError({
            'Reason'=> 'AcceptCustomerInformation',
            'Form' => 'Booking',
            'Name' => 'tac'
        }) unless $hValues->{'customerinformation'};
    }

    # test services/options
    foreach my $Option (grep { defined $_->{'Value'}
                               and $_->{'Attribute'}->get('Type') eq 'PreDefCustomerString'
                               and scalar @{$_->{'Value'}}
                             }
                        @{$Product->get('UserDefinedAttributeValues')}) {
        my $AttributeAlias = $Option->{'Attribute'}->alias;
        my $Value = $Servlet->form->value($AttributeAlias);
        $Servlet->form->addFormError({
            'Reason'=> 'SelectOption',
            'Form'  => 'Booking',
            'Name'  => $AttributeAlias
        }) unless defined $Value and ElementOf($Value, map { $_->id } @{$Option->{'Value'}});
    }

    $Servlet->form->executeFormError;

    my $DurationUnit = defined $hValues->{'DurationUnitID'} ? $Product->getSite()->loadObject($hValues->{'DurationUnitID'}, 'UnitOfMeasurement', {'AllowSystemSite' => 1}) : undef;
    ($DurationUnit, $hValues->{'Duration'}) = $Calendar->defaultDuration unless defined $DurationUnit
        and defined $hValues->{'Duration'}
        and $Calendar->get('CanCustomerDefinedAppDuration');

    # test last booking

    $hValues->{'EndDate'} = AddDuration($hValues->{'StartDate'}->clone, $DurationUnit->durationUnitPlural, $hValues->{'Duration'});
    my $StartDate = _ConvertServerToClient($hValues->{'StartDate'});
    my $EndDate = AddDuration($StartDate->clone, $DurationUnit->durationUnitPlural, $hValues->{'Duration'});

    my $LastBookingAmount = undef;
    if(defined $Calendar->get('LastBookingAmount')) {
        $LastBookingAmount = $Calendar->get('LastBookingAmount');
    } else {
        if(defined $Calendar->get('BaseCalendar')) {
            $LastBookingAmount = $Calendar->get('BaseCalendar')->get('LastBookingAmount')
        }
    }

    my $LastBookingUnit = $Calendar->get('LastBookingUnit') ? $Calendar->get('LastBookingUnit') : $Calendar->get('BaseCalendar')->get('LastBookingUnit');
    my $Now  = GetCurrentDBHandle()->currentDateTime;
    if ($StartDate < $Now) {
        $Servlet->form->addFormError({
            'Reason'=> 'BookingTooLateNow',
            'Form' => 'Booking',
            'Name' => 'StartDate'
        });
    }
    if (defined $LastBookingAmount
        and defined $LastBookingUnit
        and $StartDate < AddDuration($Now->clone, $LastBookingUnit->durationUnitPlural, $LastBookingAmount)
        ) {
        $Servlet->form->addFormError({
            'Reason'=> 'BookingTooLate',
            'Form' => 'Booking',
            'Name' => 'StartDate'
        });
    }

    if ($Calendar->get('OnlyBookingPeriod')){
        if (not $Calendar->isBusiness($StartDate->clone(), $EndDate->clone())) {
            $Servlet->form->addFormError({
                'Reason'=> 'BookingNoBusiness',
                'Form' => 'Booking',
                'Name' => 'StartDate'
            });
        }
    }
    if ($Calendar->isBusy(_ConvertClientToServer($StartDate), _ConvertClientToServer($EndDate), $hValues->{'Quantity'}, $Product->get('StockLevel'))) {
        my $RemainingQuantity = $Calendar->getRemainingQuantity(_ConvertClientToServer($StartDate), _ConvertClientToServer($EndDate), $Product->get('StockLevel'));
        if ($hValues->{'Quantity'} > $RemainingQuantity ) {
            $Servlet->form->addFormError({
                'Reason'=> 'BookingIsBusy',
                'Form' => 'Booking',
                'Name' => 'Quantity',
                'RemainingQuantity' => $RemainingQuantity
            });
        }
        $Servlet->form->addFormError({
            'Reason'=> 'BookingIsBusy',
            'Form' => 'Booking',
            'Name' => 'StartDate'
        });
    }
    $Servlet->form->executeFormError;
    return $hValues;
}

# convert the DateTime object to the client (store) time zone
sub _ConvertServerToClient {
    my ($DateTime) = @_;

    my $TimeZone = LoadRootObject()->get('TimeZone');
    my $Clone = $DateTime->clone;
    #$Clone->set_time_zone( 'floating');
    $Clone->set_time_zone($TimeZone);
    return $Clone;

}

# convert the DateTime object to the server time zone
sub _ConvertClientToServer {
    my ($DateTime) = @_;

    my $Clone = $DateTime->clone;
    $Clone->set_time_zone( GetLocalTimeZone());
    return $Clone;
}



sub NewAppointment { ## no critic qw(ProhibitExcessComplexity)
    my $self = shift;
    my ($Servlet) = @_;

    my $Product  = $Servlet->object;
    my $Shop     = $Product->getSite();

    # EPG-8971: if there are no payment methods available, then the resource
    # can be booked without payment, like if CanPayAppointment was false
    my $CanPayAppointment = $Product->get('CanPayAppointment');

    if ($CanPayAppointment){
        $CanPayAppointment = $Servlet->form->value('ExistsNoPayment') ? 0 : 1;
    }

    my $hValues = $self->_newAppointmentTestForm($Servlet, $CanPayAppointment);
    my $User = $Servlet->user;
    my $IsAnonymousUser = 1;
    $IsAnonymousUser = 0 if defined $User;
    $User = $Servlet->createAnonymousUser($Product->getSite()) unless defined $User;

    my $aAppointmentBaskets = $User->get('AppointmentBaskets');
    my $Basket = scalar @$aAppointmentBaskets ? $aAppointmentBaskets->[0] : undef;
    my $LineItemAppointment;
    if (defined $Basket) {
        foreach my $LineItem (@{ $Basket->container->get('LineItems') }) {
            if ($LineItem->instanceOf('LineItemProductAppointment') && $LineItem->get('Product')->id eq $Product->id) {
                $LineItemAppointment = $LineItem;
            }
        }
    }

    # create appointment
    my $ExistsAppointment = 1;
    my $Appointment = defined $LineItemAppointment ? $LineItemAppointment->get('Appointment') :
                            ExistsObjectByGUID($hValues->{'GUID'}) ? LoadObjectByGUID($hValues->{'GUID'}) : undef;
    if (defined $Appointment) {
        return $Appointment unless $CanPayAppointment;

        my $hBusinessObjects = $Appointment->get(['Basket','CustomerOrder']);
        if (defined $hBusinessObjects->{'CustomerOrder'}) {
            $Servlet->vars('ViewAction', 'View');
            return $hBusinessObjects->{'CustomerOrder'};
        }
        $Appointment->set( {
            'EndDate' => $hValues->{'EndDate'},
            'StartDate' => $hValues->{'StartDate'},
            'Quantity' => $hValues->{'Quantity'},
        });
    } else {
        my @Services = map { $Shop->loadObject($Servlet->form->value($_->{'Attribute'}->alias), 'PreDefAttribute') }
            grep { defined $_->{'Value'} and $_->{'Attribute'}->get('Type') eq 'PreDefCustomerString' and scalar @{$_->{'Value'}}}
            @{$Product->get('UserDefinedAttributeValues')};

        $ExistsAppointment = 0;
        my $Calendar = $Product->businessHours;
        # create appointment
        $Appointment = $self->SUPER::NewObject( $Servlet, $Calendar, LoadClassByAlias('Appointment'), {
            Alias=> $Calendar->nextAppointmentNo,
            User => $User,
            EndDate => $hValues->{'EndDate'},
            AcceptedOn => ($Calendar->get('AcceptWithReservation')) ? undef : GetCurrentDateTime(),
            IsStorefront => 1,
            Services => \@Services,
            ServicesString => scalar @Services ? join (',', map { $_->get('NameOrAlias') } @Services) : undef,
        });
    }
    # create address for appointment
    my $Address = ($ExistsAppointment && $Appointment->existsChild('Address')) ? $Appointment->child('Address') : undef;
    if (defined $Address) {
        $Address->set($hValues->{'Address'});
    } else {
        $Address = $Appointment->insertChild('Address', {
            Parent=>$Appointment,
            Alias =>'Address',
            %{$hValues->{'Address'}},
        })
    }

    # EPG-9350: store entered address data in the anonymous user's session
    $User->set( {'BillingAddress' => $Address }) if $IsAnonymousUser;

    # create basket
    if ($CanPayAppointment) {
        if (not defined $LineItemAppointment) {
            if (defined $Basket) {
                my $Container = $Basket->container;
                SavePayment($Container, $Shop->loadObject($hValues->{'PaymentMethodID'}, 'PaymentMethod'));
            }
            my $aBaskets;
            my $aWarnings = BlockWarnings( sub {
                $aBaskets = AddProducts($User, {
                    'LocaleID'   => $Servlet->locale,
                    'CurrencyID' => $Servlet->currencyID,
                    'BillingAddress' => $Address,
                    'PaymentMethod' => $Shop->loadObject($hValues->{'PaymentMethodID'}, 'PaymentMethod'),
                }, [
                {
                    'Product'     => $Product,
                    'Appointment' => $Appointment,
                    'Quantity'    => $hValues->{'Quantity'},
                }]);
            }, ['ProductNoPrice', 'ProductCanNotAddToBasket', 'ProductCanNotChangedAtBasket', 'NoShipping', 'NoPayment', 'ExclusiveShippingMethod' ]);

            $Basket = $aBaskets->[0];
            foreach my $Warning (@$aWarnings) {
                if ($Warning->code eq 'NoShipping' && $Shop->featureMaxValue('ShippingMethods') == 0) {
                    # this is a shop without shipping feature: ignore error
                } elsif ($Warning->code eq 'NoPayment' && $Shop->featureMaxValue('PaymentMethods') == 0) {
                    # this is a shop without payment feature: ignore error
                } else {
                    $Servlet->form->mapFormError({'Name'  => 'AddToBasket'}, $Warning);
                }
            }
            $Servlet->form->executeFormError;

        } else {
            my $Container = $Basket->container;
            SavePayment($Container, $Shop->loadObject($hValues->{'PaymentMethodID'}, 'PaymentMethod'));
            $LineItemAppointment->set({
                'Quantity' => $hValues->{'Quantity'},
                'TimeFactor' => $Product->timeFactor($hValues->{'StartDate'}, $hValues->{'EndDate'})
            });
        }
    } else {
        BlockWarnings( sub {
            TriggerHook('AppointmentCreated', { Appointment=>$Appointment, Locale=>$Servlet->locale });
        }, ['MAIL_SENT']);
    }

    if (defined $Basket) {
        $Basket->set({'IsTacOK' => 1}) if $hValues->{'tac'};
        $Basket->set({'IsCustomerInformationOK' => 1}) if $hValues->{'customerinformation'};
        my $Prepayment = $Product->appointmentPrepayment($Appointment, {
            'Quantity' => $hValues->{'Quantity'},
            'CurrencyID' => $Servlet->currencyID,
            'TaxModel' => $Basket->container->get('TaxModel'),
            'User' => $User
        });
        $Basket->container->set({'HasPrepayment' => 1}) if defined $Prepayment && $Prepayment != 0;
        $Appointment->set({'Prepayment' => $Prepayment });
        $Servlet->vars('ObjectID', $Basket->id);
        my $CustomerOrder = DE_EPAGES::Order::UI::Basket->DoOrder($Servlet);
        $Servlet->vars('ViewAction', 'View');
        return $CustomerOrder;
    }
    $Servlet->vars('ViewAction', 'View');
    return $Appointment;
}

sub AddAppointmentToOrder {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product  = $Servlet->object;

    my $OrderID = $Servlet->params('OrderID');
    return $self->NewAppointment($Servlet) unless defined $OrderID;

    my $Order = $Product->getSite()->loadObject($OrderID, 'CustomerOrder');
    my $LineItemAppointment;
    foreach my $LineItem (@{ $Order->container->get('LineItems') }) {
        if ($LineItem->instanceOf('LineItemProductAppointment') && $LineItem->get('Product')->id eq $Product->id) {
            $LineItemAppointment = $LineItem;
        }
    }

    my $Calendar = $Product->businessHours;
    my $hAddress = $self->addressAttributeValues($Servlet, $Calendar, LoadClassByAlias('Address'), 'Address.')->{'Values'};

    $Servlet->form->executeFormError();

    my $Appointment = $self->SUPER::NewObject( $Servlet, $Calendar, LoadClassByAlias('Appointment'), {
        Alias => $Calendar->nextAppointmentNo,
        User => undef, # get from customer ?
    });
    $Appointment->insertChild('Address', {
        Parent=>$Appointment,
        Alias =>'Address',
        %$hAddress,
    });

    $LineItemAppointment->set({
        'TimeFactor' => $Product->timeFactor($Appointment->get('StartDate'), $Appointment->get('EndDate')),
        'Appointment' => $Appointment,
    });

    $Servlet->form->executeFormError;
    $Order->container->recalculate;
    $Servlet->vars('SendMail', 1);

    return $Appointment;
}

sub addressAttributeValues {
    my $self = shift;
    my ( $Servlet, $Object, $Class, $Prefix ) = @_;

    my $hhValues = $self->SUPER::attributeValues( $Servlet, $Object, $Class, $Prefix );
    my $Parent = $Object->parent;
    $Parent = $Parent->parent if $Parent->instanceOf('Calendar'); # save Appointment else NewAppointment

    return $hhValues unless $Class->isSubClassOf('Appointment');

    my $Shop = $Object->getSite();
    my @Services = ();
    $hhValues->{'Values'}{'Services'} = \@Services;
    foreach my $Option (grep { defined $_->{'Value'}
                               and $_->{'Attribute'}->get('Type') eq 'PreDefCustomerString'
                               and scalar @{$_->{'Value'}}
                             }
                        @{$Parent->get('UserDefinedAttributeValues')}) {
        my $AttributeAlias = $Option->{'Attribute'}->alias;
        my $Value = $Servlet->form->value($AttributeAlias);
        if (defined $Value and ElementOf($Value, map { $_->id } @{$Option->{'Value'}})) {
            push @Services, $Shop->loadObject($Value, 'Object');
        } else {
            $Servlet->form->addFormError({
                'Reason'=> 'SelectOption',
                'Form'  => 'Booking',
                'Name'  => $AttributeAlias
            });
        }
    }
    $hhValues->{'Values'}{'ServicesString'} = join (',', map { $_->get('NameOrAlias') } @Services) if scalar @Services;
    return $hhValues;
}

sub ViewFiles {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $ahFiles = $Product->get('PublicFiles');

    my @Sizes = keys %{PRODUCT_IMAGES_SIZES()};
    my $RegSizes = join '|', @Sizes;
    my @hParentFiles;
    $_->{'SubImages'} = [] foreach @$ahFiles;

    my $idx = 0;
    my $hOriginal = {};
    my $hProductImages = $Product->get(['ImageHotDeal','ImageMedium','ImageMediumLarge','ImageMediumSmall','ImageSmall','ImageThumbnail']);
    my @FileNames;

    foreach my $hFile (@$ahFiles) {
        my (undef, $FileName, undef) = fsplit($hFile->{'FileName'});
        push @FileNames, $FileName;
    }

    # loop over each file and try to find the original file
    foreach my $hFile (@$ahFiles) {

        my (undef, $FName, $Extension) = fsplit($hFile->{'FileName'});

        # test if we have  a new file or set of files (scaled files)
        unless ( $FName =~ /^(.*)_($RegSizes)$/ && ElementOf($1, @FileNames) ) {
            $idx = 0;
            $hOriginal = undef;
        }

        # filter out original images wich are set as product image attribute
        next if ($idx == 0) and ElementOf($FName.$Extension,[values(%$hProductImages)]);

        my $Size;
        # we have an original file
        # set the 1st filename as original file
        if ($idx == 0){
            $hFile->{'IsOriginal'} = 1;
            $hOriginal = $hFile;
            $hOriginal->{'Thumbnail'} = $hFile->{'FileName'};
            $Size = '';
        } else { # add sub images and set a thumbnail
            $Size= $1 if $FName =~ /_($RegSizes)$/;
            $hFile->{'ProductImageSize'} = $Size;
            push @{$hOriginal->{'SubImages'}}, $hFile;
            # test if we have an size "s" file for using as thumbnail
            $hOriginal->{'Thumbnail'} = $hFile->{'FileName'} if $Size eq 's';
        }

        # don't allow non-image mimetype for slideshow
        $hFile->{'Mimetype'} = undef if defined $hFile->{'Mimetype'} and $hFile->{'Mimetype'} !~ /image/;

        #add only original files to the list
        push @hParentFiles, $hFile if $hFile->{'IsOriginal'};

        $idx++;
    }

    my $ahImagesSlideShow = $Product->get('ImagesSlideShow');

    # EPG-12387 allow to see URL images in list, allow to delete or re-sort them
    foreach my $hSlideShowImage (@$ahImagesSlideShow) {
        my $ImageName = $hSlideShowImage->{'ImageName'};
        if(StringIsURL($ImageName)) {
            push(@hParentFiles,
                {
                    'FileName' => $ImageName,
                    'Visible'  => 1,
                    'Mimetype' => 'application/x-url',
                    'IsURL'    => 1
                }
            );
        }
    }

    # determine position of image for slideshow
    foreach my $hParentFile (@hParentFiles) {
        my (undef, $FName, $Extension) = fsplit($hParentFile->{'FileName'});
        if (defined $hParentFile->{'Mimetype'}) {
            my $i = POSITION_STEP;
            foreach my $hSlideShowImage (@$ahImagesSlideShow) {
                # EPG-12387 URL in slideshowstring present
                last if $hSlideShowImage->{'ImageName'} eq $hParentFile->{'FileName'};

                last if $hSlideShowImage->{'ImageName'} eq $FName.$Extension;
                last if ElementOf($hSlideShowImage->{'ImageName'}, map { $_->{'FileName'} } @{$hParentFile->{'SubImages'}});
                $i += POSITION_STEP;
            }
            $hParentFile->{'Position'} = ($i > (scalar @$ahImagesSlideShow * POSITION_STEP)) ? undef: $i ;
        } else {
            $hParentFile->{'Position'} = undef;
        }
        $hParentFile->{'Visible'} = defined $hParentFile->{'Position'} ? 1 : 0;
    }

    $Servlet->vars('ParentFiles', [sort {
        $b->{'Visible'} <=> $a->{'Visible'}
        || ((defined $a->{'Position'} && defined $b->{'Position'})
            ? $a->{'Position'} <=> $b->{'Position'}
            : 0)
        || $a->{'FileName'} cmp $b->{'FileName'}
        } @hParentFiles
    ]);
    $Servlet->vars('PRODUCT_IMAGE_MAX_COUNT', PRODUCT_IMAGE_MAX_COUNT);
    return;
}

sub scaleImages {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $LargeFile = $self->saveFile($Servlet, $Product, 'NewFile');

    my $ShopPath = $Product->get('Site')->get('PublicPath');
    my $ProductPath = $Product->get('FileAlias');

    $Servlet->form->executeFormError;

    return undef unless defined $LargeFile;

    my $hImageFiles;
    eval {
        # get the constants from the template for scaling images
        my $hParams = $Servlet->params();
        my $ImageFile = $Product->get('PublicPath').'/'.$LargeFile;
        TestImage($ImageFile);
        $hImageFiles = ScaleImageSizes($ImageFile, $hParams);
    };
    if (ExistsError()) {
        my $Error = GetError();

        Error($Error) unless ($Error->code eq 'IMAGE_MAGICK' ||
                              $Error->code eq 'UNSUPPORTED_IMAGE_FORMAT' ||
                              $Error->code eq 'INVALID_IMAGE_FILE' ||
                              $Error->code eq 'IMAGE_EXCEEDS_MEGAPIXELS'
                              );
        DeleteFile($Product->get('PublicPath').'/'.$LargeFile);
        CleanPath($ShopPath, $ProductPath);
        $Servlet->form->executeFormError({
            'Reason'=> $Error->code,
            'Form'  => 'SaveFiles',
            'Name'  => 'NewFile',
        });

    }
    return $hImageFiles;
}
#========================================================================================
# function SaveImage
# state public
#----------------------------------------------------------------------------------------
# syntax $Handler->SaveImage($Servlet);
#----------------------------------------------------------------------------------------
# description Called for product image upon image upload and generating of all sizes.
#----------------------------------------------------------------------------------------
# input $Servlet | servlet object | object
#========================================================================================

sub SaveImage {
    my $self = shift;
    my ($Servlet) = @_;

    # get info
    my $hImageFiles = $self->scaleImages($Servlet);
    my $Product = $Servlet->object;
    my $DefaultImage = $hImageFiles->{'ImageLarge'};

    # save image reference in object
    $Product->set($hImageFiles);

    # get current slideshow string
    my $SlideShowString = $Product->get('ImagesSlideShowString');

    # check if default image and slideshow string defined, check if slide show string doesnt contain default image
    # conclusion not executed for variation products without own slideshow
    if($SlideShowString && $DefaultImage && $SlideShowString !~ /$DefaultImage/) {
        # add default image to slideshow string
        $SlideShowString = join ';', $DefaultImage, $SlideShowString;

        # update slide show string on object
        $Product->set( { 'ImagesSlideShowString' => $SlideShowString } );
    }

    # EPG-10737 upload for all views
    $Product->getSite()->invalidatePageCache;

    # trigger refresh of SEO check
    PerformSEOCheckAndUpdateObject($Product);

    return;
}



sub SaveDesign {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object = $Servlet->object;

    $Servlet->vars('ObjectID', $Object->class->id);
    DE_EPAGES::Product::UI::ProductType->SaveDesign($Servlet);
    $Servlet->vars('ObjectID', $Object->id);
    return;
}

sub MoveAndScaleImage{
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop = $Product->getSite();
    my $FilePath = $Servlet->form->value('File');
    my (undef, $FileName, $Extension) = fsplit($FilePath);
    CopyFile($Shop->get('PublicPath').'/'.$FilePath,$Product->get('PublicPath').'/'.$FileName.$Extension);
    if (ElementOf(substr(lc($Extension),1), IMAGE_FILE_TYPES())) {
        my $hImageFiles;
        eval {
            # get the constants from the template for scaling images
            my $hParams = $Servlet->params();
            $hImageFiles = ScaleImageSizes($Product->get('PublicPath').'/'.$FileName.$Extension, $hParams);
        };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) unless ($Error->code eq 'IMAGE_MAGICK' ||
                                  $Error->code eq 'UNSUPPORTED_IMAGE_FORMAT');
            $Servlet->form->executeFormError({
                'Reason'=> $Error->code,
                'Form'  => 'SaveFiles',
                'Name'  => 'NewFile',
            });
        }
        if($Servlet->form->value('SetFilesForProduct')){
            $Product->set($hImageFiles);
        }
    }
    return;
}

sub _NormalizeName($) {
    my ($ImagePath) = @_;
    my (undef, $BaseName, $Ext) = fsplit ($ImagePath);
    return Unicode2Filename("$BaseName$Ext");
}

sub UploadMediumProductImage {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;
    my $Form = $Servlet->form;
    my $ImageBlob = $Form->files->{'NewFile'}->[0];
    my $ImageName = _NormalizeName($Form->value('NewFile'));

    # For meaning of ResponseSize and Sizes 'ml', 'm', 'ms' ...
    # see DE_EPAGES::Product::API::Constants PRODUCT_IMAGE_SIZE_NAMES and PRODUCT_IMAGES_SIZES
    StoreProductImages( $Product, $ImageName, $ImageBlob, [ qw( ml m ms ) ]);

    $Servlet->vars('ResponseContent', _BuildImageUploadResponse($Product, 'ImageMediumLarge'));

    return;
}

sub _BuildImageUploadResponse($$) {
    my ($Product, $ImageAttribute) = @_;

    my $ImageNameAndExt = $Product->get($ImageAttribute);
    my $PublicPath      = $Product->get('PublicPath');
    my $ResponseFile    = $PublicPath . '/' . $ImageNameAndExt;
    my $RelativePathAndImage = RelativePath($PublicPath , $ResponseFile);

    return {
        'uploadedFile'       => $Product->get('WebPath') . '/' . $RelativePathAndImage,
        'FileName'           => $ImageNameAndExt,
        'uploadedObjectFile' => $RelativePathAndImage,
    };
}

#========================================================================================
# function     DeleteSmallProductImage
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteSmallProductImage( $Servlet );
#----------------------------------------------------------------------------------------
# description  Remove medium images - ImageThumbnail ImageSmall - from file system
#               and delete associated product attributes
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet with product object | object
#========================================================================================
sub DeleteSmallProductImage {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;

    DeleteProductImageByAttribute($Product, [qw (ImageSmall ImageThumbnail)]);

    return;
}

#========================================================================================
# function     DeleteMediumProductImage
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteMediumProductImage( $Servlet );
#----------------------------------------------------------------------------------------
# description  Remove medium images -  ImageMediumSmall, ImageMedium, ImageMediumLarge -
#               from product, related products, slide show and filesystem
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet with product object | object
#========================================================================================
sub DeleteMediumProductImage {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;

    DeleteProductImageByAttribute($Product, [qw (ImageMedium ImageMediumSmall ImageMediumLarge)]);

    return;
}

#========================================================================================
# function     DeleteLargeProductImage
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteLargeProductImage( $Servlet );
#----------------------------------------------------------------------------------------
# description  Remove large image -  ImageLarge - from product, related products, slide show
#               and filesystem
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet with product object | object
#========================================================================================
sub DeleteLargeProductImage {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;

    DeleteProductImageByAttribute($Product, [qw (ImageLarge)], {DontDeleteSubImages => 1});

    return;
}

#========================================================================================
# function     DeleteHotDealProductImage
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteHotDealProductImage( $Servlet );
#----------------------------------------------------------------------------------------
# description  Remove hot deal image -  ImageHotDeal - from product, related products, slide show
#               and filesystem
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet with product object | object
#========================================================================================
sub DeleteHotDealProductImage {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;

    DeleteProductImageByAttribute($Product, [qw (ImageHotDeal)]);

    return;
}

#========================================================================================
# function     DeleteAllProductImages
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteAllProductImages( $Servlet );
#----------------------------------------------------------------------------------------
# description  This is convenience function to call <DeleteProductImageByAttribute> with
#               all supported image sizes. This also ensure that images are correctly removed
#               from SlideShows and SubProducts.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet with product object | object
#========================================================================================
sub DeleteAllProductImages {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;

    # ImageLarge, ImageMediumLarge, ...
    my @AttributeNames = values %{PRODUCT_IMAGE_SIZE_NAMES()};
    DeleteProductImageByAttribute($Product, \@AttributeNames);

    return;
}

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->params('MBOUI_ProductDetailPage', MBOUI_PRODUCT_DETAIL_PAGE);
    $self->SUPER::View($Servlet);
    return;
}

sub ViewUnityProduct {
    my $self      = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite;

    my @TextAttributes;
    my @SelectionAttributes;
    my @UsedSelectionAttributes;
    # get the ids of all product types that were used to create variations
    my @UsedVariations = map {$_->{'Attribute'}->id} @{$Product->getWithParent('VariationTypeValues')};
    #loop over all attributes and sort them by type
    foreach my $CurrentAttribute (@{$Product->getWithParent('UserDefinedAttributeValues')}) {
        my $Attribute = $CurrentAttribute->{'Attribute'};
        my $Value = $CurrentAttribute->{'Value'};
        # if value is blessed object (happens if it is a predefstring with option picked) transform to tleHash
        my $ParsedValue = (blessed($Value)) ? $Value->tleHash : $Value;
        my $Type = $Attribute->get('Type');
        if ($Type eq 'PreDefLocalizedString' || $Type eq 'PreDefString') {
            #test if attribute was used to create a variation
            if ($Attribute->id ~~ @UsedVariations) {
                push(@UsedSelectionAttributes, {'Attribute'  => $Attribute->tleHash,
                                               'Value'      => $ParsedValue} );
            } else {
                push(@SelectionAttributes, {'Attribute'  => $Attribute->tleHash,
                                            'Value'      => $ParsedValue});
            }
        } elsif ($Type eq 'LocalizedString' || $Type eq 'String') {
            push(@TextAttributes, {'Attribute'  => $Attribute->tleHash,
                                   'Value'      => $ParsedValue});
        }
    }

    $Servlet->vars('TextAttributes', \@TextAttributes);
    $Servlet->vars('SelectionAttributes', \@SelectionAttributes);
    $Servlet->vars('UsedSelectionAttributes', \@UsedSelectionAttributes);

    # currency
    my $CurrencyID = $Shop->get('CurrencyID');

    $Servlet->vars('ShopCurrencyID',     $CurrencyID);
    $Servlet->vars('ShopCurrencySymbol', GetInfoCurrency($CurrencyID)->{'Symbol'});

    #categories
    my $LocaleID   = $Servlet->locale();
    my $Locale     = DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my @Categories = map { {'CategoryID' => $_->id, 'CategoryName' => $_->get('NameOrAlias', $LanguageID)} }
        @{ $Product->get('Categories') };
    $Servlet->vars('Categories', \@Categories);

    my $SuperProduct = $Product->get('SuperProduct');
    if (defined $SuperProduct) {
        my @SuperProductCategories = map { {'CategoryID' => $_->id, 'CategoryName' => $_->get('NameOrAlias', $LanguageID)} }
            @{ $SuperProduct->get('Categories') };
        $Servlet->vars('SuperProductCategories', \@SuperProductCategories);
    }

    #load path and order of images
    my @ahExistingFiles;
    my @aFileOrder;

    my $WebPath = $Product->get('WebPath');
    my $ProductImage = $Product->get('ImageLarge') // "";
    my $ProductImageMedium = $Product->get('ImageMedium');

    my $ImagesSlideShowString = $Product->get('ImagesSlideShowString') // "";
    if ($ProductImage && $ImagesSlideShowString !~ m/$ProductImage/) {
        push(@ahExistingFiles, {'id' => -1, 'src' => $WebPath . '/' . $ProductImage, 'm_src' => $WebPath . '/' . $ProductImageMedium});
        push(@aFileOrder, -1);
    }

    my $ahImagesSlideShow = $Product->get('ImagesSlideShow');
    foreach my $hSlideShowImage (@$ahImagesSlideShow) {
        my $ImageName = $hSlideShowImage->{'ImageName'};
        my $Position  = $hSlideShowImage->{'Position'};

        my (undef, $FileName, $Extension) = fsplit($ImageName);
        my $MediumImage = $FileName . '_' . 'm' . $Extension;

        push(@ahExistingFiles, { 'id' => $Position,
                                 'src' => $WebPath . '/' . $ImageName,
                                 'm_src' => $WebPath . '/' . $MediumImage});
        push(@aFileOrder, $Position);
    }

    $Servlet->vars('Exist', \@ahExistingFiles);
    $Servlet->vars('Order', \@aFileOrder);


    my ($ParsedExist, $ParsedOrder) = $self->_getExistingFilesAndOrder($Servlet);

    $Servlet->vars('AllExist', $ParsedExist);
    $Servlet->vars('AllOrder', $ParsedOrder);


    #load existing slightshows for subproducts get image position on master product
    my %SlideShows;
    my @SlideShowOrder;
    my $hSlideShowValues = $Product->get(['ImagesSlideShowString', 'ImageLarge', 'ImagesSlideShow']);

    if ((defined $hSlideShowValues->{'ImageLarge'}) and
        $hSlideShowValues->{'ImagesSlideShowString'} !~ m/$hSlideShowValues->{'ImageLarge'}/) {

        push(@SlideShowOrder, $hSlideShowValues->{'ImageLarge'});
    }

    foreach my $hSlideShowImage (@{$hSlideShowValues->{'ImagesSlideShow'}}) {
        push(@SlideShowOrder, $hSlideShowImage->{'ImageName'});
    }
    $SlideShows{ $Product->id } = [@SlideShowOrder];

    # escape quotes
    my $ParsedSlideShows = to_json(\%SlideShows);
    $ParsedSlideShows =~ s/\"/\\\"/g;
    $Servlet->vars('SlideShowOrder', $ParsedSlideShows);

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Product_ViewUnityProduct', { 'Servlet' => $Servlet });

    return;
}

#========================================================================================
# function     _IsCurrentlyAvailable
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsCurrentlyAvailable = _IsCurrentlyAvailable($AvailabilityDate, $ExpiryDate);
#----------------------------------------------------------------------------------------
# description  Returns true if the product has a limited-time offer and is currently
#               available. It should be noted that $IsCurrentlyAvailable can be true
#               (a product is a limited time offer), even though
#               $Product->get('IsAvailable') returns false (i.e. the product is normally
#               not offered). 'IsCurrentlyAvailable' is independent of 'IsAvailable'.
#----------------------------------------------------------------------------------------
# input        $IsAvailable | is product available? | boolean
# input        $AvailabilityDate | product is available from this date | date
# input        $ExpiryDate | product is no longer available after this date | date
# return       $IsCurrentlyAvailable | True if the product is currently available | boolean
#========================================================================================
sub _IsCurrentlyAvailable {
    my ($IsAvailable, $AvailabilityDate, $ExpiryDate) = @_;

    return $IsAvailable unless (defined $AvailabilityDate || defined $ExpiryDate);

    my $CurrentDate = DE_EPAGES::Object::API::Object::Site->currentDate();
    return ($AvailabilityDate <= $CurrentDate) ? 1 : 0 unless defined $ExpiryDate;
    return ($ExpiryDate       >= $CurrentDate) ? 1 : 0 unless defined $AvailabilityDate;
    return ($AvailabilityDate <= $CurrentDate && $ExpiryDate >= $CurrentDate) ? 1 : 0;
}

# show the correct product if a variation attribute was changed
sub ViewProduct {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object = $Servlet->object;
    my $SearchParams = $Servlet->params('SearchParams');

    # test if we have informations about selected attributes in facets
    die 'no product' unless $Object->instanceOf('Product');
    my $Master = $Object->get('SuperProduct') // $Object;

    my $hFormValues = $Servlet->form->form($Object, 'ViewProduct');
    #do we have changed variation attribute values form the product detail page

    # find out the sub product context for displaying the correct image if we have
    # form parameters "SelectedAttributes"
    my @AttributeValueIDs = map { $_->{'SelectedVariation'} } @{$hFormValues->{'SelectedVariations'}};

    # test if we have parameters from search and find out if we have to display the correct
    # sub product if all variation values are given
    # or display the master product an use the images from the given sub product
    if ($SearchParams) {
        my $hParams = from_json($SearchParams);

        # get the count of variation attributes from product
        my $aVariationTypeValues  = $Object->getWithParent('VariationTypeValues');

        # search the selected attribute value ids by the given parameter
        foreach my $VariationTypeValue (@$aVariationTypeValues) {
            my $Attribute = $VariationTypeValue->{'Attribute'};

            # test if we have the attribute name in the list
            if ( ElementOf( $Attribute->get('NameOrAlias'), keys(%$hParams) ) ) {
                # search for attribute value in the list
                foreach my $AttrValue ( @{ $VariationTypeValue->{'Values'} } ) {
                    if ( $hParams->{ $Attribute->get('NameOrAlias') } eq $AttrValue->get('ValueOrAlias') ) {
                        push @AttributeValueIDs, $AttrValue->id;
                        last;
                    }
                }
            }
        }
    }

    my $hMatchedAttributes = {};
    my $SearchObject = $Object;
    if ( scalar(@AttributeValueIDs) ) {
        my $Shop = $Object->getSite();
        my @SelectedVariationIDs;
        foreach my $AttributeValueID (@AttributeValueIDs) {
            if (defined $AttributeValueID) {
                push @SelectedVariationIDs, $AttributeValueID;
                my $Attribute = $Shop->loadObject($AttributeValueID, 'Object')->get('Attribute');
                $hMatchedAttributes->{$Attribute->id} = $AttributeValueID;
            }
        }

        if( scalar(@SelectedVariationIDs) ) {
            # searching for a product matching this attributes criterias
            my $aSubProductIDs = DAL_GetSubProductIDsBySuperProductID($Master->id, \@SelectedVariationIDs);
            # fuzzy match: use this as preselection
            $SearchObject = $Shop->loadObject( $aSubProductIDs->[0], 'Product') if scalar @$aSubProductIDs;
            # perfect match: Use this as target product.
            if (scalar @$aSubProductIDs == 1) {
                $Object = $SearchObject;
            }
        }
    }

    my $ParamLoop = [];
    push @$ParamLoop, { 'AttributeID' => $_, 'AttributeValueID' => $hMatchedAttributes->{$_} } foreach keys(%$hMatchedAttributes);
    $Servlet->vars( 'SelectParams', $ParamLoop );
    $Servlet->vars( 'THIS', $Object->tleHash);
    $Servlet->params( 'ViewAction',   'View' );
    $Servlet->params( 'ViewObjectID', $Object->id );
    $Servlet->params( 'ObjectID',     $Object->id );
    $Servlet->params( 'SearchFoundObject', $SearchObject->tleHash);
    $Servlet->params( 'FromSearch',   1 );

    return;
}

#========================================================================================
# function     Delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->Delete( $Servlet );
#----------------------------------------------------------------------------------------
# description  Deletes the current object ($Servlet->object).
#               <br/>
#               Discards the operation if user enables DeleteConfirmation and not
#               $Servlet->params('ACCEPT'). Sets the $Servlet->vars('DELETE',1) in this
#               case.
#               Creates a BatchProcess for DeleteProduct events.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $Parent | parent object of $Servlet->object | object
#========================================================================================
sub Delete {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $Parent = undef;
    RunBatchProcess('DeleteProducts', sub {
        $Parent = $self->SUPER::Delete($Servlet);
    });

    return $Parent;
}

sub UnityViewVariations {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1) if $Servlet->object->get('IsMaster');

    return $self->ViewPagedChildren($Servlet);
}

1;
