#==============================================================================
# §package      DE_EPAGES::Shop::API::Constants
# §state        public
# §description  Constants of cartridge Shop
#==============================================================================
package DE_EPAGES::Shop::API::Constants;
use base Exporter;

use strict;
our @EXPORT_OK = qw(
    TAX_MODEL_GROSS
    TAX_MODEL_NET

    BASKETACTION_ADDQUANTITY
    BASKETACTION_REPLACEQUANTITY
    BASKETACTION_DONTCHANGEQUANTITY
    BASKETACTION_ADDNEWPOSITION

    Text2TaxModel
    TaxModel2Text

    $TAX_MODEL_GROSS_WITH_VATID
    $COUNTRIES_EXCLUDED_FROM_VIES_CHECK
    $NUMBER_FORMAT_INFIX_PATTERNS

    $COUNTRIES_WITH_SAME_TAX

    $EU_COUNTRY_AREA_EXCEPTIONS

    SHOP_JWT_LIFETIME

    STOCK_IS_ORDERABLE
    STOCK_HAS_LONGER_DELIVERYTIME
    STOCK_NO_ADDTOBASKET
    STOCK_NOT_VISIBLE
    UNITY_STOREFRONT_APP_NAME
    UNITY_STOREFRONT_APP_TYPE

    EDITOR_JWT_LIFETIME
    NOTIFICATION_POSITION_TOP

    MAXIMUM_FEATURE_VALUES_FOR_UNITY_MIGRATION
    FEATURES_TO_EXCLUDE_FOR_UNITY_MIGRATION

    UNITY_MAIL_LOGO
    UNITY_LOGO
    MAX_TRIES_FOR_PATCHING_UNITYBASEURL

    TUTORIAL_VIDEO_ATTRIBUTES

    $UNITY_RECOMMENDED_IMAGE_SIZES
);

use Readonly;
use DE_EPAGES::Core::API::EpagesConfig qw( EpagesConfig );
use DE_EPAGES::Core::API::Error qw (Error);

sub ErrorDefinition {
    return {
        'TAX_MODEL_TEXT' => 'tax model text #ModelText unknown (gross and net are allowed)',
        'TAX_MODEL_INT'  => 'tax model integer #ModelInt unknown (0 and 1 are allowed)',
    }
}

#========================================================================================
# §global       TAX_MODEL_NET | Product prices include tax. Tax is calculated from the sum of product prices. | integer
# §global       TAX_MODEL_GROSS | Product prices don't include tax. Tax is added to the sum of product prices. | integer
#========================================================================================
use constant TAX_MODEL_NET      => 0;
use constant TAX_MODEL_GROSS    => 1;

#========================================================================================
# §global       BASKETACTION_ADDQUANTITY | Shop basket action if lineitem exists: Increases quantity in existing line item. | integer
# §global       BASKETACTION_REPLACEQUANTITY | Shop basket action if lineitem exists: Replaces the quantity at line item. | integer
# §global       BASKETACTION_DONTCHANGEQUANTITY | Shop basket action if lineitem exists: Does not change line item. | integer
# §global       BASKETACTION_ADDNEWPOSITION | Shop basket action if lineitem exists: Creates a new line item. | integer
#========================================================================================
use constant BASKETACTION_ADDQUANTITY        => 0;
use constant BASKETACTION_REPLACEQUANTITY    => 1;
use constant BASKETACTION_DONTCHANGEQUANTITY => 2;
use constant BASKETACTION_ADDNEWPOSITION     => 3;

#========================================================================================
# §function     Text2TaxModel
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $TaxModel = Text2TaxModel( $TaxModelName )
#----------------------------------------------------------------------------------------
# §description  Returns the tax model id by name.
#               Returns undef if $TaxModelName is undef.
#               Throws an error if $TaxModelName is neither 'gross' nor 'net'.
#----------------------------------------------------------------------------------------
# §input        $TaxModelName | tax model name ('gross' or 'net') | string
# §return       $TaxModel | tax model id (TAX_MODEL_NET or TAX_MODEL_GROSS) | int
#========================================================================================
sub Text2TaxModel($) {
    my ($TaxModelName) = @_;

    return undef unless defined $TaxModelName;
    return TAX_MODEL_GROSS if $TaxModelName eq 'gross';
    return TAX_MODEL_NET   if $TaxModelName eq 'net';
    Error('TAX_MODEL_TEXT', { 'ModelText' => $TaxModelName });
}

#========================================================================================
# §function     TaxModel2Text
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $TaxModelName = TaxModel2Text( $TaxModel )
#----------------------------------------------------------------------------------------
# §description  Returns the tax model name by id.
#               Returns undef if $TaxModel is undef.
#               Throws an error if $TaxModel is neither TAX_MODEL_NET nor TAX_MODEL_GROSS.
#----------------------------------------------------------------------------------------
# §input        $TaxModel | tax model id (TAX_MODEL_NET or TAX_MODEL_GROSS) | int
# §return       $TaxModelName | tax model name ('gross' or 'net') | string
#========================================================================================
sub TaxModel2Text($) {
    my ($TaxModel) = @_;

    return undef unless defined $TaxModel;
    return 'gross' if $TaxModel == TAX_MODEL_GROSS;
    return 'net'   if $TaxModel == TAX_MODEL_NET;
    Error('TAX_MODEL_INT', { 'ModelInt'=> $TaxModel });
}

#========================================================================================
# §global       $TAX_MODEL_GROSS_WITH_VATID | VATID starts with one of these values then
#               use taxmodel gross | ref.hash.array.string
#========================================================================================
Readonly our $TAX_MODEL_GROSS_WITH_VATID => {
    'PT' => [ qw(
                1
                2
               ) ],

    # see http://es.wikipedia.org/wiki/N%C3%BAmero_de_identificaci%C3%B3n_fiscal
    # KLMXYZ are special prefixes for a private NIF e.g. K1234567 for persons <14 years ...
    # example:
    # 25719876Z  -> normal private NIF (suffix (here:Z) is not a hint for private or business!)  -> gross
    # K1234567   -> special private NIF -> gross
    # B64817144  -> business NIF        -> net

    'ES' => [ qw(
                K
                L
                M
                X
                Y
                Z
                0 1 2 3 4 5 6 7 8 9
               ) ],
};

#========================================================================================
# §global       $COUNTRIES_EXCLUDED_FROM_VIES_CHECK | If the VATCountryCode is contained
#               in this hash the VATIDs from this country should only be checked locally
#               and not against the VIES database.
#               Only the key of this hash matters, the value is irrelevant (hash is used
#               as a set). To check whether a country is excluded you should use defined.
#               | ref.array.int
#========================================================================================
Readonly our $COUNTRIES_EXCLUDED_FROM_VIES_CHECK => {
    'ES' => 1,
    'IT' => 1,
    'PT' => 1,
};

#========================================================================================
# §global       $NUMBER_FORMAT_INFIX_PATTERNS | Infix of numbers will be formatted with
#               one of these patterns | ref.hash.string
#========================================================================================
Readonly our $NUMBER_FORMAT_INFIX_PATTERNS => {
    'YYYYMMMDD' => '%Y%b%d',
    'YYYYMMDD' => '%Y%m%d',
    'YYYYMM' => '%Y%m',
    'YYMMDD' => '%y%m%d',
    'YYMM' => '%y%m',
    'MMMDD' => '%b%d',
    'MMDD' => '%m%d',
    'YYYY' => '%Y',
    'YY' => '%y',
    'MMM' => '%b',
    'MM' => '%m',
};


#========================================================================================
# §global       $COUNTRIES_WITH_SAME_TAX | use same tax handling for country codes
#               use taxmodel gross | ref.hash.array.string
#========================================================================================
Readonly our $COUNTRIES_WITH_SAME_TAX => {
                '756' => [ qw( 438 ) ],      # shopaddress switzerland == customer lichtenstein
                '438' => [ qw( 756 ) ],      # shopaddress lichtenstein == customer switzerland
            };


#========================================================================================
# §global       $EU_COUNTRY_AREA_EXCEPTIONS | eu countries with no eu tax handling areas
#               | ref.hash.array.hash
#========================================================================================
Readonly our $EU_COUNTRY_AREA_EXCEPTIONS => {
                '724' => [
                          {'ZipcodeFrom' => '35000',   # Spain-> Islas Canarias
                           'ZipcodeTo'   => '35999',
                          },
                          {'ZipcodeFrom' => '38000',   # Spain-> Islas Canarias
                           'ZipcodeTo'   => '38999',
                          },
                          {'ZipcodeFrom' => '51000',   # Spain-> Ceuta / Melilla
                           'ZipcodeTo'   => '52999',
                          },
                         ],
                '276' => [
                          {'ZipcodeFrom' => '27498',   # Germany-> Helgoland
                           'ZipcodeTo'   => '27498',
                          },
                          {'ZipcodeFrom' => '78266',   # Germany-> B�singen
                           'ZipcodeTo'   => '78266',
                          },
                         ],
                '380' => [
                          {'ZipcodeFrom' => '23030',   # Italy-> Livigno
                           'ZipcodeTo'   => '23030',
                          },
                          {'ZipcodeFrom' => '22060',   # Italy-> Campione
                           'ZipcodeTo'   => '22060',
                          },
                         ],
                '250' => [
                          {'ZipcodeFrom' => '97100',   # France-> Guadeloupe
                           'ZipcodeTo'   => '97199',
                          },
                          {'ZipcodeFrom' => '97200',   # France-> Martinique
                           'ZipcodeTo'   => '97299',
                          },
                          {'ZipcodeFrom' => '97300',   # France-> Guyane fran�aise
                           'ZipcodeTo'   => '97399',
                          },
                          {'ZipcodeFrom' => '97400',   # France-> R�union
                           'ZipcodeTo'   => '97499',
                          },
                          {'ZipcodeFrom' => '97600',   # France-> Mayotte
                           'ZipcodeTo'   => '97699',
                          },
                          {'ZipcodeFrom' => '97500',   # France-> Isle Miquelon and Isle Saint Piere
                           'ZipcodeTo'   => '97500',
                          },
                         ],
                '826' => [
                          {'ZipcodeFrom' => 'GY1 1AA',   # United Kingdom-> Guernsey, Alderney, Sark
                           'ZipcodeTo'   => 'GY9 9ZZ',
                          },
                          {'ZipcodeFrom' => 'JE1 0AA',   # United Kingdom-> Jersey
                           'ZipcodeTo'   => 'JE5 9ZZ',
                          },
                         ],
                '208' => [
                          {'ZipcodeFrom' => '100',   # Danmark-> F�r�er
                           'ZipcodeTo'   => '970',
                          },
                          {'ZipcodeFrom' => '3900',  # Danmark-> Gr�nland
                           'ZipcodeTo'   => '3999',
                          },
                         ],
                '246' => [
                          {'ZipcodeFrom' => '22000',   # Finland-> Alandinseln
                           'ZipcodeTo'   => '22999',
                          },
                         ],
                '300' => [
                          {'ZipcodeFrom' => '63086',   # Greece-> Berg Athos
                           'ZipcodeTo'   => '63087',
                          },
                         ],

            };

#========================================================================================
# §global    SHOP_JWT_LIFETIME | lifetime of the JSON Web Token for shop provisioning
#            with validity in seconds | integer
# §global    UNITY_STOREFRONT_APP_NAME | app name for registering
#               Unity Storefront with ePages J  | string
# §global    UNITY_STOREFRONT_APP_TYPE | app type for registering
#               Unity Storefront with ePages J  | string
#========================================================================================
use constant SHOP_JWT_LIFETIME => 60;
use constant UNITY_STOREFRONT_APP_NAME => 'epages-ui' ;
use constant UNITY_STOREFRONT_APP_TYPE => 'PREINSTALLED_APP' ;

#========================================================================================
# §global       STOCK_IS_ORDERABLE | product can be ordered even if out of stock, no
#                   message (0) | integer
# §global       STOCK_HAS_LONGER_DELIVERYTIME | product can be ordered even if out of
#                   customer is informed about longer delivery time (1) | integer
# §global       STOCK_NO_ADDTOBASKET | if the product is out of stock, it is still visible,
#                   but cannot be added to the basket (2) | integer
# §global       STOCK_NOT_VISIBLE | the product is hidden if out of stock (3) | integer
#========================================================================================
use constant STOCK_IS_ORDERABLE             => 0;
use constant STOCK_HAS_LONGER_DELIVERYTIME  => 1;
use constant STOCK_NO_ADDTOBASKET           => 2;
use constant STOCK_NOT_VISIBLE              => 3;

#========================================================================================
# §global    EDITOR_JWT_LIFETIME | lifetime of the JSON Web Token for the Unity editor
#               in seconds (180 s = 3 min) | integer
#========================================================================================
use constant EDITOR_JWT_LIFETIME => 180;

#========================================================================================
# §global       POSITION_TOP | top position number for shop notification | integer
#========================================================================================
use constant NOTIFICATION_POSITION_TOP => 15;

#========================================================================================
# §global       MAXIMUM_FEATURE_VALUES_FOR_UNITY_MIGRATION | maximum values of COUNTABLE features
#                   currently used in the shop,, with which it is possible to do a Unity
#                   migration, defined in epages.conf | ref.array.int
#========================================================================================
sub MAXIMUM_FEATURE_VALUES_FOR_UNITY_MIGRATION() { ## no critic qw(RequireCommentForExportedFunctions)
    return EpagesConfig()->section('MaximumFeatureValuesForUnityMigration');
}

#========================================================================================
# §global       FEATURES_TO_EXCLUDE_FOR_UNITY_MIGRATION | ON/OFF features that should be excluded
#                   if a shop wants to do a Unity migration, defined in epages.conf | ref.array.int
#========================================================================================
sub FEATURES_TO_EXCLUDE_FOR_UNITY_MIGRATION() { ## no critic qw(RequireCommentForExportedFunctions)
    return EpagesConfig()->section('FeaturesToExcludeForUnityMigration');
}

#========================================================================================
# §global       UNITY_MAIL_LOGO | file name (without extension) of the mail logo for unity | string
#========================================================================================
use constant UNITY_MAIL_LOGO => 'Mail_Logo';

#========================================================================================
# §global       UNITY_LOGO | file name (without extension) of the shop logo for unity | string
#========================================================================================
use constant UNITY_LOGO => 'Logo';

#========================================================================================
# §global       MAX_TRIES_FOR_PATCHING_UNITYBASEURL |maximum number of retires
#                   when patching the UnityBaseURL to all shops in epages-ui | integer
#========================================================================================
use constant MAX_TRIES_FOR_PATCHING_UNITYBASEURL => 3;

#========================================================================================
# §global       TUTORIAL_VIDEO_ATTRIBUTES | System attributes that contain the IDs for
#                   the Unity tutorial videos | ref.array.string
#========================================================================================
use constant TUTORIAL_VIDEO_ATTRIBUTES => [
    'DesignVideoID',
    'ProductVideoID',
    'ContentVideoID',
    'OverviewVideoID',
    'SEOVideoID',
    'DeliveryVideoID'
];

#========================================================================================
# §global       $UNITY_RECOMMENDED_IMAGE_SIZES | text for recommended minimal image sizes
#                   for image uploads | ref.hash.string
#========================================================================================
Readonly our $UNITY_RECOMMENDED_IMAGE_SIZES => {
    'Shop_Logo'     => '200 &times; 200 px',
    'Product_Image' => '1100 &times; 1100 px',
};

1;
