#==============================================================================
# package      DE_EPAGES::ShoppingList::UI::ShoppingList
# state        private
#==============================================================================
package DE_EPAGES::ShoppingList::UI::ShoppingList;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Order::API::Basket qw (GetExistingBasket);
use DE_EPAGES::Product::API::Basket qw (AddProducts);
use DE_EPAGES::ShoppingList::API::Constants qw (UNITY_SHOPPINGLIST_ALIAS);

#==============================================================================
# Event handlers
#==============================================================================

sub View {
    my $self = shift;
    my ($Servlet) = @_;
    my $ProductID = $Servlet->params('ProductID');
    my $User = $Servlet->object();

    # load shopping list for unity 
    if ($User->getSite()->get('IsUnityRegistered')) {
        my $Folder = $User->folder('ShoppingLists');
        my $Alias = UNITY_SHOPPINGLIST_ALIAS;
        if( $Folder->existsChild($Alias) ) {
            my $ShoppingList = $Folder->child($Alias);
            $Servlet->vars('ShoppingList', $ShoppingList->tleHash);
        }
    }

    return unless defined $ProductID;
    my $Product = $Servlet->object()->getSite()->loadObject($ProductID, 'Product');
    $Servlet->testPermission($Product, 'View');
    $Servlet->vars('Product',$Product->tleHash);

    return;
}

sub ViewUserShoppingList {
    my $self = shift;
    my ($Servlet) = @_;

    my $User = $Servlet->user;

    $Servlet->vars('ActivateLink', 1);

    if (defined($User)) {
        my $aShoppingLists = $User->get('ShoppingLists');

        if (scalar(@$aShoppingLists) == 1) {
            # show the one shopping list
            Error('INVALID_FORM', {
                'ViewVars'=> {
                    'FormError' => 0,
                    'ViewObjectID' => @$aShoppingLists[0]->id,
                    'ViewAction' => 'View',
                    'ActivateLink' => 1,
                },
            });
        }
        else {
            # show shopping lists
            Error('INVALID_FORM', {
                'ViewVars'=> {
                    'FormError' => 0,
                    'ViewObjectID' => $User->id,
                    'ViewAction' => 'ViewUserShoppingLists',
                    'ActivateLink' => 1,
                },
            });
        }
    }
    return;
}

sub Update {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShoppingList = $Servlet->object;
    my $Form = $Servlet->form->form($ShoppingList, 'Update');
    my $Shop = $ShoppingList->getSite();

    my $Quantities = $Form->{'Items'};
    foreach my $QuantityEntry (@$Quantities) {
        my $ObjectID = $QuantityEntry->{'ItemID'};
        delete $QuantityEntry->{'ItemID'};

        my $ShoppingListEntry = $Shop->loadObject($ObjectID, 'ShoppingListItem');
        my $Product = $ShoppingListEntry->get('Product');

        if ( $QuantityEntry->{'Quantity'} > 0 and defined $Product) {
            $QuantityEntry->{'Quantity'} = $Product->correctQuantity($QuantityEntry->{'Quantity'})
             if exists $QuantityEntry->{'Quantity'};
            $ShoppingListEntry->set($QuantityEntry);
        } else {
            $ShoppingListEntry->delete();
        }
    }
    #set shoppinglist change date
    $ShoppingList->set( { 'Alias'      => $Form->{'Alias'},
                          'ChangeDate' => GetCurrentDBHandle()->currentDateTime() });

    return;
}

sub AddToBasket {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShoppingList = $Servlet->object;
    my $User         = $Servlet->user;
    my $Shop         = $ShoppingList->getSite();

    my $Form = $Servlet->form->form($ShoppingList, 'Update');
    my $Quantities = $Form->{'Items'};
    my @Products;
    my $Index = 1;
    foreach my $QuantityEntry (@$Quantities) {
        my $ShoppingListItemID = $QuantityEntry->{'ItemID'};
        #add to basket
        if ( $QuantityEntry->{'Quantity'} > 0 ) {
            my $ShoppingListItem = $Shop->loadObject($ShoppingListItemID, 'ShoppingListItem');
            my $Product = $ShoppingListItem->get('Product');

            next if not defined $Product or $Product->get('IsAppointment'); #product not defined or is an appointment
            next if $Product->get('IsCustomizable'); # ignore customizable products

            my %Info;
            $Info{'Product'} = $Product;
            $Info{'Quantity'} =  $QuantityEntry->{'Quantity'};
            $Info{'Position'} = $Index * 10; ## no critic qw( ProhibitMagicNumbers )
            push @Products , \%Info;
            $Index++;
        }
    }

    my $aBaskets;
    my $aWarnings = BlockWarnings( sub {
        $aBaskets = AddProducts($User, {
            'LocaleID'   => $Servlet->locale,
            'CurrencyID' => $Servlet->currencyID,
        }, \@Products);
    }, ['ProductNoPrice', 'ProductCanNotAddToBasket', 'ProductCanNotChangedAtBasket', 'NoShipping', 'NoPayment', 'ExclusiveShippingMethod' ]);

    $Servlet->form->mapFormError({'Name'  => 'AddToBasket'}, $_) foreach @$aWarnings;
    $Servlet->form->executeFormError;

    return $aBaskets->[0];
}

sub NewShoppingList {
    my $self = shift;
    my ($Servlet) = @_;

    my $User = $Servlet->object;
    my $Form = $Servlet->form->form($User, 'ShoppingListsNew');
    my $Shop = $User->getSite();

    my $NewAlias = $Form->{'NewAlias'};
    my $Folder = $User->folder('ShoppingLists');
    if (not defined $NewAlias or $NewAlias !~ /\S/) {
        $Servlet->form->executeFormError({
            'Form'  => 'ShoppingListsNew',
            'Reason'=> 'OBJECT_NO_ALIAS',
            'Name'  => 'NewAlias',
            'Value' => $NewAlias,
        });
    } elsif ($Folder->existsChild($NewAlias)) {
        $Servlet->form->executeFormError({
            'Form'  => 'ShoppingListsNew',
            'Reason'=> 'OBJECT_EXISTS',
            'Name'  => 'NewAlias',
            'Value' => $NewAlias,
        });
    }

    my $ShoppingList = $Folder->insertChild('ShoppingList',{
        'Alias' => $NewAlias ne '' ? $NewAlias : $Folder->nextAlias('DefaultShoppingList%s') ,
        'User' =>  $User,
        'Shop' =>  $Shop,
    });

    my $Products = $Form->{'Products'};
    foreach my $ProductEntry (@$Products) {
        my $Product = $Shop->loadObject( $ProductEntry->{'ProductID'}, 'Product' );
        $ShoppingList->addItem( $Product, {'Quantity' => $ProductEntry->{'Quantity'}});
    }
    return $ShoppingList;
}

sub ShoppingListsAssignProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $User     = $Servlet->object;
    my $Shop     = $User->getSite();
    my $IsUnity  = $Shop->getSite()->get('IsUnityRegistered');
    my $FormName = $IsUnity ? 'UnityShoppingListsAssignProducts' : 'ShoppingListsAssignProducts';
    my $Form     = $Servlet->form->form($User, $FormName);

    my $ShoppingLists;
    # if we have unity we create or load the shopping list from the user directly
    if( $IsUnity ) {
        my $Folder = $User->folder('ShoppingLists');
        my $Alias = UNITY_SHOPPINGLIST_ALIAS;
        my $ShoppingList;
        unless ($Folder->existsChild($Alias) ){
            # create new ShoppingList
            $ShoppingList = $Folder->insertChild('ShoppingList',{
                'Alias' => $Alias,
                'User' =>  $User,
                'Shop' =>  $Shop,
            });
        } else {
            $ShoppingList = $Folder->child($Alias);
        }

        $ShoppingLists = [ {'ShoppingListID' => $ShoppingList->id }  ];
    } else {
        $ShoppingLists  = $Form->{'ShoppingLists'};
    }

    my $Products = $Form->{'Products'};
    foreach my $ShoppingListEntry (@$ShoppingLists) {
        my $ShoppingList = $Shop->loadObject( $ShoppingListEntry->{'ShoppingListID'}, 'ShoppingList' );
        foreach my $ProductEntry (@$Products) {
            my $Product = $Shop->loadObject( $ProductEntry->{'ProductID'}, 'Product' );
            $ShoppingList->addItem( $Product, {'Quantity' => $ProductEntry->{'Quantity'}});
         }
    }
    return;
}

sub CopyBasketToShoppingList {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShoppingList = $Servlet->object;
    my $User         = $Servlet->user;
    my $Shop         = $ShoppingList->getSite();

    my $Basket;
    if (defined $Servlet->params('BasketID')) {
        $Basket = $Shop->loadObject($Servlet->params('BasketID'), 'Basket');
    } else {
        $Basket = GetExistingBasket($User, $Shop );
    }
    if (not defined $Basket) {
        $Servlet->vars('NO_BASKET',1);
        $Servlet->form->executeFormError({
            'Reason'=> 'NO_BASKET',
            'Form'  => 'Update',
            'Name'  => 'BasketID',
        }) ;
        return;
    }

    my $aLineItems = $Basket->container->get('LineItems');
    foreach my $LineItem ( grep { $_->instanceOf('LineItemProduct') } @$aLineItems ) {
        my $Product = $LineItem->get('Product');
        next unless $Product;
        $ShoppingList->addItem( $Product, {'Quantity' => $LineItem->get('Quantity') });
    }
    #set shoppinglist change date
    $ShoppingList->set( {'ChangeDate' => GetCurrentDBHandle()->currentDateTime() });

    return;
}

1;

